####### VATStuff #######
#
# report_functions.py  - creates reports
#
# Version : 0.7.0
# Date : 20100911
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""report functions, accessing the database"""

# Do not import wx - so these database functions can be done and used by the testing
# process without the graphical wx being present


import decimal
import os
import datetime

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify import form_objects

from vatstuff_packages.vs_database import vat_standard
from vatstuff_packages.vs_database import vat_cash
from vatstuff_packages.vs_database import vat_flat_basic
from vatstuff_packages.vs_database import vat_flat_cash


D = decimal.Decimal   # D used to avoid typing



def create_report(report_type, from_date, to_date, **kwds):
    "Depending on report_type, return the appropiate ReportForm object"

    if report_type == "items":
        return create_items_report(kwds["item_type"], kwds["all_items"])

    if report_type == "contacts":
        return create_contacts_report(kwds["contact_list"])

    if report_type == "accounts":
        return create_accounts_report(kwds["account_folder"], kwds["account_type"])

    # the following reports all require a to_date and from_date

    assert from_date

    if not to_date:
        to_date = datetime.date.today()

    if to_date < from_date:
        to_date = from_date

    if report_type == "statement":
        return create_statement_report(from_date, to_date, kwds["account"])

    if report_type == "invoices":
        return create_invoices_report(from_date, to_date)

    if report_type == "creditnotes":
        return create_creditnotes_report(from_date, to_date)

    if report_type == "payments":
        return create_payments_report(from_date, to_date)

    if report_type == "supplierinvoices":
        return create_supplierinvoices_report(from_date, to_date)

    if report_type == "suppliercreditnotes":
        return create_suppliercreditnotes_report(from_date, to_date)

    if report_type == "makepayments":
        return create_makepayments_report(from_date, to_date)

    if report_type == "vatpayments":
        return create_vatpayments_report(from_date, to_date)

    if report_type == "ecsaleslist":
        return create_ecsaleslist_report(from_date, to_date)

    if report_type == "purchaseorders":
        return create_purchaseorders_report(from_date, to_date)

    if report_type == "project":
        return create_project_report(from_date, to_date, kwds["project"])

    if report_type == "transactions":
        return create_transactions_report(from_date, to_date)

    if report_type == "customer_invoices":
        return create_customer_invoices_report(from_date, to_date, kwds["customer"])

    if report_type == "customer_creditnotes":
        return create_customer_creditnotes_report(from_date, to_date, kwds["customer"])

    if report_type == "customer_payments":
        return create_customer_payments_report(from_date, to_date, kwds["customer"])

    if report_type == "supplier_orders":
        return create_supplier_orders_report(from_date, to_date, kwds["supplier"])

    if report_type == "supplier_invoices":
        return create_supplier_invoices_report(from_date, to_date, kwds["supplier"])

    if report_type == "supplier_creditnotes":
        return create_supplier_creditnotes_report(from_date, to_date, kwds["supplier"])

    if report_type == "supplier_payments":
        return create_supplier_payments_report(from_date, to_date, kwds["supplier"])

    # The following vat reports have their own module rather than being defined here

    if report_type == "vat_report_standard":
        return vat_standard.standard_report(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_detail_standard":
        return vat_standard.standard_detail(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_report_cash":
        return vat_cash.cash_report(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_detail_cash":
        return vat_cash.cash_detail(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_report_flat_basic":
        return vat_flat_basic.flat_basic_report(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_detail_flat_basic":
        return vat_flat_basic.flat_basic_detail(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_report_flat_cash":
        return vat_flat_cash.flat_cash_report(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])

    if report_type == "vat_detail_flat_cash":
        return vat_flat_cash.flat_cash_detail(from_date, to_date, kwds["vat_return"], kwds["vat_scheme"])



    raise VS_ERROR, "The report type has not been recognised"

    return None


    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database


####### ACCOUNT STATEMENT

def create_statement_report(from_date, to_date, account):
    """Create statement report of an account between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Account Statement")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Folder : %s" % account.folder.name,
                     "Account : %s" % account.name,
                     "Description : %s" % account.description,
                     "Type : %s" % account.acc_type.name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Transaction'),
                      ('HBL1', 'Date'),
                      ('HBL1', 'Description'),
                      ('HBR1', 'Debit'),
                      ('HBR1', 'Credit'),
                      ('HBR1', 'Balance')     ]]

    # Get the database table name which stores this account
    ledger_table_name = account.ledger

    sql_string = """SELECT COUNT(*) FROM TRANSACTIONS, %s
                    WHERE DATESTAMP BETWEEN ? AND ?
                    AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""" % (ledger_table_name,ledger_table_name)

    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No transactions in the account exist within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many transactions to display</b>"]
        return report

    sql_string = """SELECT TRANSACTIONS.TRANSACTION_ID,DATESTAMP,DESCRIPTION,DEBIT,CREDIT,BALANCE
                    FROM TRANSACTIONS, %s
                    WHERE DATESTAMP BETWEEN ? AND ?
                    AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                    ORDER BY TRANSACTIONS.GRIDROW
                    """ % (ledger_table_name,ledger_table_name)
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))

        for info in c:
            report.add_row_to_table( ('DNL1', str(info[0])),
                                     ('DNL1', info[1].strftime("%d %b %Y")),
                                     ('DNL1', info[2]),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )
    finally:
        if conn:
            conn.close()
    return report


####### INVOICES

def create_invoices_report(from_date, to_date):
    """Create report of invoices sent to all customers between
       the two dates and return the ReportForm object"""

    report = form_objects.ReportForm("Invoices to customers")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Invoices issued:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Issue Date'),
                      ('HBL1', 'Tax Point Date'),
                      ('HBL1', 'Customer'),
                      ('HBL1', 'Invoice number'),
                      ('HBL1', 'Status'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM INVOICES, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No invoices were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many invoices to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, DATE_OF_TAXPOINT, NAME, INVOICE_NUMBER, OPEN, CANCELLED, COST, VAT
                        FROM INVOICES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))

        for info in c:
            if info[4]:
                status = "Open"
            elif info[5]:
                status = "Cancelled"
            else:
                status = "Closed"

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1].strftime("%d %b %Y")),
                                     ('DNL1', info[2]),
                                     ('DNL1', info[3]),
                                     ('DNL1', status),
                                     ('DNC1', info[6]),
                                     ('DNC1', info[7]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### CREDITNOTES

def create_creditnotes_report(from_date, to_date):
    """Create report of creditnotes sent to customers between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Credit Notes to customers")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Credit notes issued:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Customer'),
                      ('HBL1', 'Credit Note'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM CREDITNOTES, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No credit notes were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many credit notes to display</b>"]
        return report

    sql_string = """SELECT DATESTAMP, NAME, CREDITNOTE_NUMBER, COST, VAT
                        FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        ORDER BY CREDITNOTES.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))

        for info in c:
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', str(info[2])),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### PAYMENTS RECEIVED

def create_payments_report(from_date, to_date):
    """Create report of payments received between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Payments received")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Payments received:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Customer'),
                      ('HBL1', 'Customer reference'),
                      ('HBL1', 'Payment transaction'),
                      ('HBR1', 'Amount')      ]]

    # Customer payment

    sql_string = """SELECT COUNT(*)
                        FROM CUSTOMERPAYMENTS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result1 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] > 1000:
        report.footer = ["<b>Too many received payments to display</b>"]
        return report

    # payments from suppliers with creditnotes

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND STATUS = 1
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result2 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] == 0 and result2[0] == 0:
        report.footer = ["<b>No payments were received within the given dates</b>"]
        return report

    if result1[0] != 0:
        sql_string = """SELECT DATESTAMP, NAME, CUSTOMER_REFERENCE, CUSTOMERPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', str(info[3])),
                                         ('DNC1', info[4]) )
        finally:
            if conn:
                conn.close()

    if result2[0] != 0:
        row = [ ('HBL1', 'Date'),
                ('HBL1', 'Supplier'),
                ('HBL1', 'Supplier Credit Note'),
                ('HBL1', 'Payment transaction'),
                ('HBR1', 'Amount')  ]

        if result1[0] == 0:
            report.table = [row]
        else:
            report.add_row_to_table(*row)

        # list all payments received with supplier creditnotes
        sql_string = """SELECT TRANSACTIONS.DATESTAMP, NAME, SUPPLIERCREDITNOTES.NUMBER, SUPPLIERCREDITNOTES.TRANSACTION_ID, COST, VAT
                        FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND STATUS = 1
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date))

            for info in c:
                amount = D(info[4]) + D(info[5])
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', str(info[3])),
                                         ('DNC1', str(amount)) )
        finally:
            if conn:
                conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIERINVOICES

def create_supplierinvoices_report(from_date, to_date):
    """Create report of supplierinvoices received from all suppliers between
       the two dates and return the ReportForm object"""

    report = form_objects.ReportForm("Invoices from suppliers")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Invoices received:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier'),
                      ('HBL1', 'Supplier Invoice'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Invoice Status'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERINVOICES, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No supplier invoices were received within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many invoices to display</b>"]
        return report

    # UNION select needed as purchaseorders may be NULL if a delete old data has occurred
    sql_string = """SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERS.NAME,
                           SUPPLIERINVOICES.NUMBER,
                           PURCHASEORDERS.ORDER_NUMBER,
                           OPEN,
                           SUPPLIERINVOICES.COST,
                           SUPPLIERINVOICES.VAT
                        FROM SUPPLIERINVOICES, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERINVOICES.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                     UNION
                        SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERS.NAME,
                           SUPPLIERINVOICES.NUMBER,
                           NULL,
                           OPEN,
                           SUPPLIERINVOICES.COST,
                           SUPPLIERINVOICES.VAT
                        FROM SUPPLIERINVOICES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERINVOICES.PURCHASEORDER_ID IS NULL

                        ORDER BY TRANSACTIONS.DATESTAMP"""

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, from_date, to_date))

        for info in c:
            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if not isinstance(info[0], datetime.date):
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")
            else:
                datestamp = info[0].strftime("%d %b %Y")

            # The purchaseorder number could be None if no order exists
            # in which case display a '?' string 
            if not info[3]:
                ordernumber = "?"
            else:
                ordernumber = str(info[3])

            if info[4]:
                status = "Open"
            else:
                status = "Closed"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', ordernumber),
                                     ('DNL1', status),
                                     ('DNC1', info[5]),
                                     ('DNC1', info[6]) )

    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIERCREDITNOTES

def create_suppliercreditnotes_report(from_date, to_date):
    """Create report of creditnotes recieved from suppliers between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Credit Notes from suppliers")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Credit notes received:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier'),
                      ('HBL1', 'Supplier Credit Note'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No credit notes were received within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many credit notes to display</b>"]
        return report


    sql_string = """SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERS.NAME,
                           SUPPLIERCREDITNOTES.NUMBER,
                           COST,
                           VAT
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))

        for info in c:
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]) )

    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### PAYMENTS MADE

def create_makepayments_report(from_date, to_date):
    """Create report of payments made between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Payments made")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Payments made:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Payment transaction'),
                      ('HBR1', 'Amount')      ]]

    # payments to suppliers

    sql_string = """SELECT COUNT(*)
                        FROM MAKEPAYMENTS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result1 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] > 1000:
        report.footer = ["<b>Too many payments to display</b>"]
        return report

    # payments made to customers with creditnotes

    sql_string = """SELECT COUNT(*)
                        FROM CREDITNOTES, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND STATUS = 1
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result2 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] == 0 and result2[0] == 0:
        report.footer = ["<b>No payments were made within the given dates</b>"]
        return report

    if result1[0] != 0:
        # UNION select needed as purchaseorders may be NULL if a delete old data has occurred

        sql_string = """SELECT TRANSACTIONS.DATESTAMP, NAME, PURCHASEORDERS.ORDER_NUMBER, MAKEPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM MAKEPAYMENTS, TRANSACTIONS, PURCHASEORDERS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                        SELECT TRANSACTIONS.DATESTAMP, NAME, NULL, MAKEPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                        ORDER BY TRANSACTIONS.DATESTAMP"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date, from_date, to_date))
 
            for info in c:
                # It looks as if the UNION function returns a unicode value of the datestamp, rather
                # than a datetime.date object
                if isinstance(info[0], datetime.date):
                    datestamp = info[0].strftime("%d %b %Y")
                else:
                    int_dates = map(int, info[0].split("-"))
                    datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

                # The purchaseorder number could be None if no order exists
                # in which case display a '?' string 
                if info[2]:
                    ordernumber = str(info[2])
                else:
                    ordernumber = "?"

                report.add_row_to_table( ('DNL1', datestamp),
                                         ('DNL1', info[1]),
                                         ('DNL1', ordernumber),
                                         ('DNL1', str(info[3])),
                                         ('DNC1', info[4]) )
        finally:
            if conn:
                conn.close()

    if result2[0] != 0:
        row = [ ('HBL1', 'Date'),
                ('HBL1', 'Customer'),
                ('HBL1', 'Credit Note'),
                ('HBL1', 'Payment transaction'),
                ('HBR1', 'Amount')  ]

        if result1[0] == 0:
            report.table = [row]
        else:
            report.add_row_to_table(*row)

        # list all payments made with creditnotes
        sql_string = """SELECT TRANSACTIONS.DATESTAMP, NAME, CREDITNOTES.CREDITNOTE_NUMBER, CREDITNOTES.TRANSACTION_ID, COST, VAT
                        FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND STATUS = 1
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date))

            for info in c:
                amount = D(info[4]) + D(info[5])
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', str(info[2])),
                                         ('DNL1', str(info[3])),
                                         ('DNC1', str(amount)) )
        finally:
            if conn:
                conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### VAT PAYMENTS

def create_vatpayments_report(from_date, to_date):
    """Create report of VAT payments to HMRC between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("VAT Payments to HMRC")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "VAT Payments to HMRC:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBR1', 'Payment type'),
                      ('HBL1', 'From Date'),
                      ('HBL1', 'To Date'),
                      ('HBR1', 'Amount')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM VAT_RETURNS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result1 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] > 1000:
        report.footer = ["<b>Too many payments to display</b>"]
        return report

    sql_string = """SELECT COUNT(*)
                        FROM VATCORRECTIONS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND VATCORRECTIONS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result2 = c.fetchone()
    finally:
        if conn:
            conn.close()

    result = result1[0] + result2[0]

    if result > 1000:
        report.footer = ["<b>Too many payments to display</b>"]
        return report

    if result == 0:
        report.footer = ["<b>No VAT payments were made within the given dates</b>"]
        return report


    sql_string1 = """SELECT GRIDROW, DATESTAMP, FROM_DATE, TO_DATE, BOX_FIVE
                        FROM VAT_RETURNS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""

    sql_string2 = """SELECT GRIDROW, DATESTAMP, AMOUNT
                        FROM VATCORRECTIONS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND VATCORRECTIONS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        result = []

        if result1[0]:
            c.execute(sql_string1, (from_date, to_date))
            result = [ (info[0],
                        info[1].strftime("%d %b %Y"),
                        "VAT Return",
                        info[2].strftime("%d %b %Y"),
                        info[3].strftime("%d %b %Y"),
                        info[4]) for info in c ]

        if result2[0]:
            c.execute(sql_string2, (from_date, to_date))
            result3 = [ (info[0], info[1].strftime("%d %b %Y"), "VAT Correction", "", "", info[2]) for info in c ]
            if result1[0]:
                result.extend(result3)
            else:
                result = result3

        # result to be sorted by gridrow
        result.sort(key = lambda x:x[0])

        for item in result:
            report.add_row_to_table( ('DNL1', item[1]),
                                     ('DNL1', item[2]),
                                     ('DNL1', item[3]),
                                     ('DNL1', item[4]),
                                     ('DNC1', item[5]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### EC SALES LIST

def create_ecsaleslist_report(from_date, to_date):
    "Create an EC Sales List returns a ReportForm object"

    report = form_objects.ReportForm("EC Sales List")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "For EC Sales:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Country Code'),
                      ('HBL1', 'VAT Registration'),
                      ('HBL1', 'Customer'),
                      ('HBL1', 'Value of supplies') ]]

    conn = None
    try:
        conn, c = vs_connect()

        # Get all none-GB vat registered customers 
        c.execute("""SELECT CUSTOMER_ID, VAT_COUNTRY_CODE, VAT_REGNUMBER, NAME FROM CUSTOMERS
                     WHERE
                     EC_COUNTRY = ?
                     AND VAT_REGISTERED = ?
                     AND VAT_COUNTRY_CODE <> ?""",
                     (1, 1, vs_cfg["VAT_COUNTRY_CODE"]) )
        customer_list = c.fetchall()

        # for each customer id
        # add up the invoices, subtract the credit notes, and append the total


        for customer_id, customer_vat_cc, customer_reg, customer_name in customer_list:
            amount = D("0.00")

            c.execute("""SELECT VAT, COST FROM INVOICES
                         WHERE DATE_OF_TAXPOINT >= ?
                         AND DATE_OF_TAXPOINT <= ?
                         AND CUSTOMER_ID = ?""", (from_date, to_date, customer_id))
            for vat,cost in c:
                amount += (D(vat) + D(cost))

            c.execute("""SELECT VAT, COST FROM CREDITNOTES, TRANSACTIONS
                         WHERE DATESTAMP >= ?
                         AND DATESTAMP <= ?
                         AND TRANSACTIONS.TRANSACTION_ID = CREDITNOTES.TRANSACTION_ID
                         AND CUSTOMER_ID = ?""", (from_date, to_date, customer_id))
            for vat,cost in c:
                amount -= (D(vat) + D(cost))

            if amount == D(0):
                continue

            report.add_row_to_table( ('DNL1', customer_vat_cc),
                                     ('DNL1', customer_reg),
                                     ('DNL1', customer_name),
                                     ('DNC1', str(amount)) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()

    if len(report.table) == 1:
        report.footer = ["<b>No invoices were issued to EC Customers within the given dates</b>"]

    return report


####### PURCHASEORDERS

def create_purchaseorders_report(from_date, to_date):
    """Create report of purchase orders sent to suppliers between the two dates
       and return the ReportForm object"""

    report = form_objects.ReportForm("Purchase orders to suppliers")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Purchase orders issued:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Status'),
                      ('HBR1', 'Order Cost'),
                      ('HBR1', 'Order VAT')  ]]

    sql_string = """SELECT COUNT(*)
                        FROM PURCHASEORDERS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No purchase orders were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many purchase orders to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, NAME, ORDER_NUMBER, STATUS, COST, VAT
                        FROM PURCHASEORDERS, TRANSACTIONS, SUPPLIERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND PURCHASEORDERS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date))

        for info in c:
            if info[3]>1:
                status = "Closed"
            else:
                status = "Open"
            
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', str(info[2])),
                                     ('DNL1', status),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### PROJECT

def create_project_report(from_date, to_date, project):
    """Create report of purchase orders sent to suppliers between the two dates
       and assigned to the given project, return the ReportForm object"""

    project_ref = project.number

    report = form_objects.ReportForm("Project %s" % project_ref )

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y")]

    if project.description:
        report.header.append(project.description)

    if project.contact:
        report.header.extend(["Project owner : %s" % project.contact.name,
                              "Email : %s" % project.contact.email])

    report.header.extend( [ "Purchase orders assigned to this project:",
                            "From: %s" % from_date.strftime("%d %b %Y"),
                            "To: %s" % to_date.strftime("%d %b %Y")] )

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Status'),
                      ('HBR1', 'Order Cost'),
                      ('HBR1', 'Order VAT')  ]]

    sql_string = """SELECT COUNT(*)
                        FROM PURCHASEORDERS, TRANSACTIONS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND PROJECT_ID = ?
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, project.id))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No purchase orders in this project were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many purchase orders to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, NAME, ORDER_NUMBER, STATUS, COST, VAT
                        FROM PURCHASEORDERS, TRANSACTIONS, SUPPLIERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND PROJECT_ID = ?
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND PURCHASEORDERS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, project.id))

        for info in c:
            if info[3]>1:
                status = "Closed"
            else:
                status = "Open"

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', str(info[2])),
                                     ('DNL1', status),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### ACCOUNTS

def create_accounts_report(account_folder, account_type):
    """Create a list report of those accounts in folder account_folder
       (any folder if account_folder is None)
       and with types account_type (any type if account_type is None)."""

    report = form_objects.ReportForm("Accounts List")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y")]

    report.table = [[ ('HBL1', 'Folder'),
                      ('HBL1', 'Account'),
                      ('HBL1', 'Description'),
                      ('HBL1', 'Type'),
                      ('HBR1', 'Debit'),
                      ('HBR1', 'Credit') ]]

    if not account_folder and not account_type:
        report.header.append("All Accounts:")

    if account_folder:
        report.header.append("For Accounts in Folder: %s" % account_folder.name)
        report.table = [[ ('HBL1', 'Account'),
                          ('HBL1', 'Description'),
                          ('HBL1', 'Type'),
                          ('HBR1', 'Debit'),
                          ('HBR1', 'Credit') ]]

    if account_folder and account_type:
        report.header.append("and with Account Type: %s" % account_type.name)
        report.table = [[ ('HBL1', 'Account'),
                          ('HBL1', 'Description'),
                          ('HBR1', 'Debit'),
                          ('HBR1', 'Credit') ]]

    if not account_folder and account_type:
        report.header.append("For Accounts with Type: %s" % account_type.name)
        report.table = [[ ('HBL1', 'Folder'),
                          ('HBL1', 'Account'),
                          ('HBL1', 'Description'),
                          ('HBR1', 'Debit'),
                          ('HBR1', 'Credit') ]]

    if account_type:
        report.header.append(account_type.description)


    conn = None
    try:
        conn, c = vs_connect()
        if not account_folder and not account_type:
            # Get all accounts
            c.execute("""SELECT FOLDERS.NAME,
                                ACCOUNTS.NAME,
                                ACCOUNTS.DESCRIPTION,
                                ACCOUNTTYPES.NAME,
                                ACCOUNTS.BALANCE
                     FROM ACCOUNTS, FOLDERS, ACCOUNTTYPES
                     WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ACCOUNTTYPES.ACCOUNTTYPE_ID
                     ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""")
            for info in c:
                if D(info[4]) < D(0):
                    debit = "0.00"
                    credit = str(abs(D(info[4])))
                else:
                    debit = info[4]
                    credit = "0.00"
                report.add_row_to_table( ('DNL1', info[0]),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', debit),
                                         ('DNC1', credit) )
        elif account_folder and not account_type:
            # Get all accounts in a folder
            c.execute("""SELECT ACCOUNTS.NAME,
                                ACCOUNTS.DESCRIPTION,
                                ACCOUNTTYPES.NAME,
                                ACCOUNTS.BALANCE
                     FROM ACCOUNTS, FOLDERS, ACCOUNTTYPES
                     WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ACCOUNTTYPES.ACCOUNTTYPE_ID
                     AND ACCOUNTS.FOLDER_ID = ?
                     ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""", (account_folder.id,))
            for info in c:
                if D(info[3]) < D(0):
                    debit = "0.00"
                    credit = str(abs(D(info[3])))
                else:
                    debit = info[3]
                    credit = "0.00"
                report.add_row_to_table( ('DNL1', info[0]),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNC1', debit),
                                         ('DNC1', credit) )
        elif not account_folder and account_type:
            # Get all accounts of account type
            c.execute("""SELECT FOLDERS.NAME,
                                ACCOUNTS.NAME,
                                ACCOUNTS.DESCRIPTION,
                                ACCOUNTS.BALANCE
                     FROM ACCOUNTS, FOLDERS, ACCOUNTTYPES
                     WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ACCOUNTTYPES.ACCOUNTTYPE_ID
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ?
                     ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""", (account_type.id,))
            for info in c:
                if D(info[3]) < D(0):
                    debit = "0.00"
                    credit = str(abs(D(info[3])))
                else:
                    debit = info[3]
                    credit = "0.00"
                report.add_row_to_table( ('DNL1', info[0]),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNC1', debit),
                                         ('DNC1', credit) )
        else:
            # Get all accounts of account type in a particular folder
            c.execute("""SELECT ACCOUNTS.NAME,
                                ACCOUNTS.DESCRIPTION,
                                ACCOUNTS.BALANCE
                     FROM ACCOUNTS, FOLDERS, ACCOUNTTYPES
                     WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ACCOUNTTYPES.ACCOUNTTYPE_ID
                     AND ACCOUNTS.FOLDER_ID = ?
                     AND ACCOUNTS.ACCOUNTTYPE_ID = ?
                     ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""", (account_folder.id, account_type.id))
            for info in c:
                if D(info[2]) < D(0):
                    debit = "0.00"
                    credit = str(abs(D(info[3])))
                else:
                    debit = info[2]
                    credit = "0.00"
                report.add_row_to_table( ('DNL1', info[0]),
                                         ('DNL1', info[1]),
                                         ('DNC1', debit),
                                         ('DNC1', credit) )

    finally:
        if conn:
            conn.close()

    if len(report.table) == 1:
        report.footer = ["<b>No matching accounts found</b>"]
    else:
        # Add totals line
        report.create_totals_row()

    return report


################## TRANSACTIONS

def create_transactions_report(from_date, to_date):
    """Create reports of all transactions between the two dates.
       Different from similar functions in that it returns a tuple
       of two ReportForm objects.  One report is suitable for html
       and pdf rendering, whereas the second is suitable for
       csv rendering"""

    report = form_objects.ReportForm("Transactions")
    csv_report = form_objects.ReportForm("Transactions")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s"   % to_date.strftime("%d %b %Y") ]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    csv_report.header = report.header

    report.table = [[ ('HBL1', 'Transaction'),
                      ('HBL1', 'Date'),
                      ('HBL1', 'Description'),
                      ('HBL1', 'Account'),
                      ('HBR1', 'Debit'),
                      ('HBR1', 'Credit')  ]]

    csv_report.table = [[ ('HBL1', 'Transaction'),
                          ('HBL1', 'Date'),
                          ('HBL1', 'Description'),
                          ('HBL1', 'Account'),
                          ('HBR1', 'Debit'),
                          ('HBR1', 'Credit')  ]]

    conn = None
    try:
        conn, c = vs_connect()
        c.execute("""SELECT COUNT(*)
                     FROM TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?""", (from_date, to_date))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No transactions in the account exist within the given dates</b>"]
        csv_report.footer = report.footer
        return (report, csv_report)

    if result[0] > 1000:
        report.footer = ["<b>Too many transactions to display</b>"]
        csv_report.footer = report.footer
        return (report, csv_report)

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute("""SELECT TRANSACTIONS.TRANSACTION_ID,
                            TRANSACTIONS.DATESTAMP,
                            TRANSACTIONS.DESCRIPTION,
                            TRANSACTIONACCOUNTS.ACCOUNT_ID,
                            ACCOUNTS.NAME,
                            FOLDERS.NAME
                         FROM TRANSACTIONS, TRANSACTIONACCOUNTS, ACCOUNTS, FOLDERS
                         WHERE TRANSACTIONACCOUNTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                         AND TRANSACTIONACCOUNTS.ACCOUNT_ID = ACCOUNTS.ACCOUNT_ID
                         AND ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                         AND TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                         ORDER BY TRANSACTIONS.GRIDROW""", (from_date, to_date))

        result = c.fetchall()

        last_id = 0
        for info in result:
            t_id = str(info[0])
            t_date = info[1].strftime("%d %b %Y")
            t_description = info[2]
            folder_account = "%s:%s" % (info[5], info[4])
            # For each transaction, get the ledger entry
            sql_string = """SELECT DEBIT, CREDIT
                             FROM LEDGER_%i
                             WHERE TRANSACTION_ID = ?""" % info[3]
            c.execute(sql_string, (info[0],))
            debit,credit = c.fetchone()
            row = [  ('DNL1', t_id ),
                     ('DNL1', t_date ),
                     ('DNL1', t_description ),
                     ('DNL1', folder_account ),
                     ('DNC1', debit ),
                     ('DNC1', credit )  ]
            csv_report.add_row_to_table(*row)
            if info[0] == last_id:
                row = [  ('DNL1', "" ),
                         ('DNL1', "" ),
                         ('DNL1', "" ),
                         ('DNL1', folder_account ),
                         ('DNC1', debit ),
                         ('DNC1', credit )  ]
            else:
                last_id = info[0]
            report.add_row_to_table(*row)
    finally:
        if conn:
            conn.close()

    return (report, csv_report)


####### ITEMS REPORT

def create_items_report(item_type, all_items):
    """Create list of items for sale or to purchase"""

    vat_scheme = VATScheme()
    if vat_scheme.method == "NOT_REGISTERED" and item_type == "sale":
        show_vat = False
    else:
        show_vat = True

    if item_type == "purchase":
        report = form_objects.ReportForm("Purchase Items list")
    else:
        report = form_objects.ReportForm("Product list")

    # VAT is a choice of VAT symbols
    vat_rates = VATRate.get_all()
    symbol_selection = []
    for rate in vat_rates:
        symbol_selection.append(rate.symbol + " - " + rate.name)

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y")]

    if show_vat:
        report.table = [[ ('HBL1', 'Description'),
                          ('HBL1', 'Cost'),
                          ('HBL1', 'VAT Rate'),
                          ('HBL1', 'VAT'),
                          ('HBL1', 'Total') ]]
    else:
        report.table = [[ ('HBL1', 'Description'),
                          ('HBL1', 'Cost') ]]


    for item in all_items:
        if show_vat:
            vat_rate = item.vatrate.rate
            item_cost = item.cost
            vat_amount = (item_cost * vat_rate) / D("100.0000")
            VAT = vat_amount.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
            row = [ ('DNL1', item.description),
                    ('DNC1', str(item_cost)),
                    ('DNR1', str(vat_rate).replace(".", vs_cfg["CP"])),
                    ('DNC1', str(VAT)),
                    ('DNC1', str(item_cost+VAT)) ]
        else:
            row = [ ('DNL1', item.description),
                    ('DNC1', str(item.cost))  ]

        report.add_row_to_table(*row)

    if len(report.table) == 1:
        report.footer = ["<b>No items found</b>"]

    return report


####### CONTACTS REPORT

def create_contacts_report(contact_list):
    """Create list of contacts"""

    report = form_objects.ReportForm("Contact list")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y")]

    report.table = [[ ('HBL1', "Name"),
                      ('HBL1', "Email"),
                      ('HBL1', "Phone"),
                      ('HBL1', "Mobile"),
                      ('HBL1', "Notes" ) ]]   

    for item in contact_list:
        report.add_row_to_table( ('DNL1', item.name),
                                 ('DNL1', item.email),
                                 ('DNL1', item.phone),
                                 ('DNL1', item.mobile),
                                 ('DNL1', item.notes) )

    if len(report.table) == 1:
        report.footer = ["<b>No items found</b>"]

    return report


####### CUSTOMER_INVOICES

def create_customer_invoices_report(from_date, to_date, customer):
    """Create report of invoices sent to the given customer between
       the two dates and return the ReportForm object"""

    customer_name = customer.name
    customer_number = customer.number

    # These quieries must work on the customer number rather than the customer id,
    # as invoices by customer must refer to the same customer - even if the customer
    # has changed their name and hence id

    if len(customer_name)<20:
        report = form_objects.ReportForm("Invoices to %s" % customer_name)
    else:
        report = form_objects.ReportForm("Invoices to customer")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Invoices issued:",
                     "To customer: %s" % customer_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Issue Date'),
                      ('HBL1', 'Tax Point Date'),
                      ('HBL1', 'Invoice number'),
                      ('HBL1', 'Status'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM INVOICES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No invoices were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many invoices to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, DATE_OF_TAXPOINT, INVOICE_NUMBER, OPEN, CANCELLED, COST, VAT
                        FROM INVOICES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))

        for info in c:
            if info[3]:
                status = "Open"
            elif info[4]:
                status = "Cancelled"
            else:
                status = "Closed"
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1].strftime("%d %b %Y")),
                                     ('DNL1', info[2]),
                                     ('DNL1', status),
                                     ('DNC1', info[5]),
                                     ('DNC1', info[6]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### CUSTOMER_CREDITNOTES

def create_customer_creditnotes_report(from_date, to_date, customer):
    """Create report of credit notes sent to the given customer between
       the two dates and return the ReportForm object"""

    customer_name = customer.name
    customer_number = customer.number

    # These quieries must work on the customer number rather than the customer id,
    # as credit notes by customer must refer to the same customer - even if the customer
    # has changed their name and hence id

    if len(customer_name)<20:
        report = form_objects.ReportForm("Credit Notes to %s" % customer_name)
    else:
        report = form_objects.ReportForm("Credit Notes to customer")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Credit notes issued:",
                     "To customer: %s" % customer_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Credit Note'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No credit notes were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many credit notes to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, CREDITNOTE_NUMBER, COST, VAT
                        FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        ORDER BY CREDITNOTES.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))

        for info in c:
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', str(info[1])),
                                     ('DNC1', info[2]),
                                     ('DNC1', info[3]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### CUSTOMER_PAYMENTS

def create_customer_payments_report(from_date, to_date, customer):
    """Create report of payments from the given customer between
       the two dates and return the ReportForm object"""

    customer_name = customer.name
    customer_number = customer.number

    # These quieries must work on the customer number rather than the customer id,
    # as payments by customer must refer to the same customer - even if the customer
    # has changed their name and hence id


    if len(customer_name)<20:
        report = form_objects.ReportForm("Payments from %s" % customer_name)
    else:
        report = form_objects.ReportForm("Customer payments")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Payments received:",
                     "From customer: %s" % customer_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Customer reference'),
                      ('HBL1', 'Payment transaction'),
                      ('HBR1', 'Amount')      ]]

    # Customer payment

    sql_string = """SELECT COUNT(*)
                        FROM CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))
        result1 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] > 1000:
        report.footer = ["<b>Too many received payments to display</b>"]
        return report

    # payments made to customers with creditnotes

    sql_string = """SELECT COUNT(*)
                        FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND STATUS = 1
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, customer_number))
        result2 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] == 0 and result2[0] == 0:
        report.footer = ["<b>No payments were received within the given dates</b>"]
        return report

    if result1[0] != 0:
        sql_string = """SELECT DATESTAMP, CUSTOMER_REFERENCE, CUSTOMERPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date, customer_number))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', str(info[2])),
                                         ('DNC1', info[3]) )
        finally:
            if conn:
                conn.close()

    if result2[0] != 0:
        row = [ ('HBL1', 'Date'),
                ('HBL1', 'Credit Note'),
                ('HBL1', 'Payment transaction'),
                ('HBR1', 'Amount')  ]

        if result1[0] == 0:
            report.table = [row]
        else:
            report.add_row_to_table(*row)

        # list all payments made with creditnotes
        sql_string = """SELECT TRANSACTIONS.DATESTAMP, CREDITNOTES.CREDITNOTE_NUMBER, CREDITNOTES.TRANSACTION_ID, COST, VAT
                        FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND CUSTOMERS.NUMBER = ?
                        AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                        AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND STATUS = 1
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date, customer_number))

            for info in c:
                amount = D(info[3]) + D(info[4])
                if amount > D(0):
                    # As this is a report of payment received - but this amount is a payment made
                    # make the value negative
                    amount = D(0)-amount
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', str(info[1])),
                                         ('DNL1', str(info[2])),
                                         ('DNC1', str(amount)) )
        finally:
            if conn:
                conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIER_ORDERS

def create_supplier_orders_report(from_date, to_date, supplier):
    """Create report of purchase orders sent to the given supplier between
       the two dates and return the ReportForm object"""

    supplier_name = supplier.name
    supplier_number = supplier.number

    # These quieries must work on the supplier number rather than the supplier id,
    # as orders by supplier must refer to the same supplier - even if the supplier
    # has changed their name and hence id

    if len(supplier_name)<20:
        report = form_objects.ReportForm("Purchase orders to %s" % supplier_name)
    else:
        report = form_objects.ReportForm("Purchase orders")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Purchase orders issued:",
                     "To supplier: %s" % supplier_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Status'),
                      ('HBR1', 'Order Cost'),
                      ('HBR1', 'Order VAT')  ]]

    sql_string = """SELECT COUNT(*)
                        FROM PURCHASEORDERS, TRANSACTIONS, SUPPLIERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND PURCHASEORDERS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No purchase orders were issued within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many purchase orders to display</b>"]
        return report


    sql_string = """SELECT DATESTAMP, ORDER_NUMBER, STATUS, COST, VAT
                        FROM PURCHASEORDERS, TRANSACTIONS, SUPPLIERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND PURCHASEORDERS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))

        for info in c:
            if info[2]>1:
                status = "Closed"
            else:
                status = "Open"
            
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', str(info[1])),
                                     ('DNL1', status),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIER_INVOICES

def create_supplier_invoices_report(from_date, to_date, supplier):
    """Create report of supplierinvoices received from the given supplier
       between the two dates and return the ReportForm object"""

    supplier_name = supplier.name
    supplier_number = supplier.number

    # These quieries must work on the supplier number rather than the supplier id,
    # as invoices by supplier must refer to the same supplier - even if the supplier
    # has changed their name and hence id

    if len(supplier_name)<20:
        report = form_objects.ReportForm("Invoices from %s" % supplier_name)
    else:
        report = form_objects.ReportForm("Invoices from supplier")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Invoices received:",
                     "From supplier: %s" % supplier_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier Invoice'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Invoice Status'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERINVOICES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No supplier invoices were received within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many invoices to display</b>"]
        return report

    # UNION select needed as purchaseorders may be NULL if a delete old data has occurred
    sql_string = """SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERINVOICES.NUMBER,
                           PURCHASEORDERS.ORDER_NUMBER,
                           SUPPLIERINVOICES.OPEN,
                           SUPPLIERINVOICES.COST,
                           SUPPLIERINVOICES.VAT
                        FROM SUPPLIERINVOICES, TRANSACTIONS, PURCHASEORDERS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERINVOICES.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                        SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERINVOICES.NUMBER,
                           NULL,
                           SUPPLIERINVOICES.OPEN,
                           SUPPLIERINVOICES.COST,
                           SUPPLIERINVOICES.VAT
                        FROM SUPPLIERINVOICES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERINVOICES.PURCHASEORDER_ID IS NULL

                     ORDER BY TRANSACTIONS.DATESTAMP"""

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number, from_date, to_date, supplier_number))

        for info in c:
            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if not isinstance(info[0], datetime.date):
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")
            else:
                datestamp = info[0].strftime("%d %b %Y")

            # The purchaseorder number could be None if no order exists
            # in which case display a '?' string 
            if not info[2]:
                ordernumber = "?"
            else:
                ordernumber = str(info[2])

            if info[3]:
                status = "Open"
            else:
                status = "Closed"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', ordernumber),
                                     ('DNL1', status),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIER_CREDITNOTES

def create_supplier_creditnotes_report(from_date, to_date, supplier):
    """Create report of creditnotes recieved from the given supplier
       between the two dates and return the ReportForm object"""

    supplier_name = supplier.name
    supplier_number = supplier.number

    # These quieries must work on the supplier number rather than the supplier id,
    # as creditnotes by supplier must refer to the same supplier - even if the supplier
    # has changed their name and hence id

    if len(supplier_name)<20:
        report = form_objects.ReportForm("Credit Notes from %s" % supplier_name)
    else:
        report = form_objects.ReportForm("Credit Notes from supplier")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Credit notes received:",
                     "From supplier: %s" % supplier_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Supplier Credit Note'),
                      ('HBR1', 'Net Cost'),
                      ('HBR1', 'VAT')       ]]

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))
        result = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result[0] == 0:
        report.footer = ["<b>No credit notes were received within the given dates</b>"]
        return report

    if result[0] > 1000:
        report.footer = ["<b>Too many credit notes to display</b>"]
        return report


    sql_string = """SELECT TRANSACTIONS.DATESTAMP,
                           SUPPLIERCREDITNOTES.NUMBER,
                           COST,
                           VAT
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        ORDER BY TRANSACTIONS.GRIDROW"""
    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))

        for info in c:
            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNC1', info[2]),
                                     ('DNC1', info[3]) )
    finally:
        if conn:
            conn.close()
    # Add totals line
    report.create_totals_row()
    return report


####### SUPPLIER_PAYMENTS

def create_supplier_payments_report(from_date, to_date, supplier):
    """Create report of payments received from the given supplier
       between the two dates and return the ReportForm object"""

    supplier_name = supplier.name
    supplier_number = supplier.number

    # These quieries must work on the supplier number rather than the supplier id,
    # as payments by supplier must refer to the same supplier - even if the supplier
    # has changed their name and hence id


    if len(supplier_name)<20:
        report = form_objects.ReportForm("Payments to %s" % supplier_name)
    else:
        report = form_objects.ReportForm("Payments to supplier")

    report.header = ["Report Generated: %s" % datetime.date.today().strftime("%d %b %Y"),
                     "Payments made:",
                     "To: %s" % supplier_name,
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y")]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    report.table = [[ ('HBL1', 'Date'),
                      ('HBL1', 'Order number'),
                      ('HBL1', 'Payment transaction'),
                      ('HBR1', 'Amount')      ]]

    # payments to supplier

    sql_string = """SELECT COUNT(*)
                        FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                        WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))
        result1 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] > 1000:
        report.footer = ["<b>Too many payments to display</b>"]
        return report

    # payments received from suppliers with creditnotes

    sql_string = """SELECT COUNT(*)
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND STATUS = 1
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""
    conn = None
    try:
        conn, c = vs_connect()
        c.execute(sql_string, (from_date, to_date, supplier_number))
        result2 = c.fetchone()
    finally:
        if conn:
            conn.close()

    if result1[0] == 0 and result2[0] == 0:
        report.footer = ["<b>No payments were made within the given dates</b>"]
        return report

    if result1[0] != 0:
        # UNION select needed as purchaseorders may be NULL if a delete old data has occurred
        sql_string = """SELECT TRANSACTIONS.DATESTAMP, PURCHASEORDERS.ORDER_NUMBER, MAKEPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM MAKEPAYMENTS, TRANSACTIONS, PURCHASEORDERS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                     UNION
                        SELECT TRANSACTIONS.DATESTAMP, NULL, MAKEPAYMENTS.TRANSACTION_ID, AMOUNT
                        FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                        ORDER BY TRANSACTIONS.DATESTAMP"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date, supplier_number, from_date, to_date, supplier_number))

            for info in c:
                # It looks as if the UNION function returns a unicode value of the datestamp, rather
                # than a datetime.date object
                if isinstance(info[0], datetime.date):
                    datestamp = info[0].strftime("%d %b %Y")
                else:
                    int_dates = map(int, info[0].split("-"))
                    datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

                # The purchaseorder number could be None if no order exists
                # in which case display a '?' string 
                if info[1]:
                    ordernumber = str(info[1])
                else:
                    ordernumber = "?"

                report.add_row_to_table( ('DNL1', datestamp),
                                         ('DNL1', ordernumber),
                                         ('DNL1', str(info[2])),
                                         ('DNC1', info[3]) )
        finally:
            if conn:
                conn.close()

    if result2[0] != 0:
        row = [ ('HBL1', 'Date'),
                ('HBL1', 'Supplier Credit Note'),
                ('HBL1', 'Payment transaction'),
                ('HBR1', 'Amount')  ]

        if result1[0] == 0:
            report.table = [row]
        else:
            report.add_row_to_table(*row)

        # list all payments received with supplier creditnotes
        sql_string = """SELECT TRANSACTIONS.DATESTAMP, SUPPLIERCREDITNOTES.NUMBER, SUPPLIERCREDITNOTES.TRANSACTION_ID, COST, VAT
                        FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                        WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERS.NUMBER = ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND STATUS = 1
                        ORDER BY TRANSACTIONS.GRIDROW"""
        conn=None
        try:
            # Make database connection
            conn, c = vs_connect()
            c.execute(sql_string, (from_date, to_date, supplier_number))

            for info in c:
                amount = D(info[3]) + D(info[4])
                if amount > D(0):
                    # As this is a report of payments made - but this amount is a payment received
                    # make the value negative
                    amount = D(0)-amount
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', str(info[2])),
                                         ('DNC1', str(amount)) )
        finally:
            if conn:
                conn.close()
    # Add totals line
    report.create_totals_row()
    return report

