####### VATStuff #######
#
# save_supplierinvoice.py  - Provides function to create a suppliers invoice
#
# Version : 0.7.0
# Date : 20100914
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Provides function to create a suppliers invoice"""


from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_database.transactions import *


def save(invoice, transaction_form):
    "Saves a SupplierInvoice as defined by SupplierInvoiceForm object invoice"

    assert invoice
    assert transaction_form.system == 9

    # Get the purchaseorder
    purchaseorder = invoice.purchaseorder
    assert purchaseorder.is_open

    # Get the running totals of invoice cost and vat from the purchaseorder
    total_invoices_cost = purchaseorder.invoice_cost
    total_invoices_vat = purchaseorder.invoice_vat

    total_invoices_cost += invoice.cost
    total_invoices_vat += invoice.vat

    if invoice.claim_vat:
        claim_vat = 1
    else:
        claim_vat = 0


    # Create a new SupplierInvoice

    new_supplierinvoice = [None,
                           invoice.supplier.id,
                           purchaseorder.id,
                           invoice.number,
                           str(invoice.cost),
                           str(invoice.vat),
                           None,
                           claim_vat
                           ]

    # Open database,  Note - all these transactions are done, and only committed at the end.
    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction
        transaction_id = implement(c, transaction_form)

        new_supplierinvoice.append(transaction_id)

        c.execute("""INSERT INTO SUPPLIERINVOICES
                          (SUPPLIERINVOICE_ID,
                           SUPPLIER_ID,
                           PURCHASEORDER_ID,
                           NUMBER,
                           COST,
                           VAT,
                           VAT_RETURN_ID,
                           CLAIM_VAT,
                           TRANSACTION_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_supplierinvoice)

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        supplierinvoice_id = result[0]

        # and insert running totals into the purchase order
        c.execute("""UPDATE PURCHASEORDERS
                       SET INVOICE_VAT = ?, INVOICE_COST = ?
                       WHERE PURCHASEORDER_ID = ?""",
                      (str(total_invoices_vat), str(total_invoices_cost), purchaseorder.id))

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Supplier invoice saved number: %s : id: %s" % (invoice.number, supplierinvoice_id))
    return SupplierInvoice(supplierinvoice_id)


