####### VATStuff #######
#
# save_vatcorrection.py  - Provides functions to save a vat correction transaction
#
# Version : 0.2
# Date : 20091017
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Provides functions to save a vat correction transaction"""


import datetime
import decimal

from vatstuff_packages.vs_utilities import *


from vatstuff_packages.vs_database.transactions import *



D = decimal.Decimal   # D used to avoid typing


def save(long_description, amount, transaction_date, transaction_text, transaction_list):
    """Implements the transaction and saves the long description and amount in VATCORRECTIONS table"""

    # Open database,  Note - all these transactions are done, and only committed at the end. 

    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction

        # Note - this is a transaction system type 11
        #
        #  0 - User defined transaction - using the move buttons (The default)
        #  1 - An opening balance transaction
        #  2 - A purchase order transaction
        #  3 - Pay a supplier invoice transaction
        #  4 - Supplier credit note transaction
        #  5 - An invoice transaction
        #  6 - Customer payment transaction
        #  7 - Create a credit note transaction
        #  8 - An order close transaction
        #  9 - Supplier invoice received transaction
        # 10 - A VAT return transaction
        # 11 - A VAT correction transaction    

        transaction_id = implement(c, transaction_date, transaction_text, transaction_list, 11)

        c.execute("""INSERT INTO VATCORRECTIONS
                          (VATCORRECTION_ID,
                           TRANSACTION_ID,
                           LONGDESCRIPTION,
                           AMOUNT
                           ) VALUES (?, ?, ?, ?)""", (None, transaction_id, long_description, str(amount)))

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()
    info_log("VAT Correction saved : transaction id : %s" % transaction_id)




