####### VATStuff #######
#
# edit_folder.py  - defines the EditFolderDialog
#
# Version : 0.6
# Date : 20100718
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the EditFolderDialog

It is run as part of the vatstuff gui interface.
"""

import wx






from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_verify.accounts import edit_folder_check


def edit_folder_dialog(parent, folder):
    """Creates an EditFolderDialog"""
    edit_folder_dialog = EditFolderDialog(parent, folder)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_folder_dialog.ShowModal()
    edit_folder_dialog.Destroy()


class EditFolderDialog(vsDialog):
    "The edit folder dialog"

    def __init__(self, parent, folder):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Edit Folder")

        # Store the folder in this instance
        self.folder=folder

        # Create the sizer
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""Accounts are organized into folders, you can
set this folder name and description here.

Each folder name must be unique."""

        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create the input fields and text in a grid sizer
        fields_sizer=wx.FlexGridSizer(rows=0, cols=2, hgap=20, vgap=20)

        # --------------------- The folder name label and field

        # Folder Name
        name_text=wx.StaticText(self, -1, "Folder name")
        fields_sizer.Add(name_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.name_field = wx.TextCtrl(self, -1, self.folder.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.name_field.SetToolTipString("The name of this folder (20 characters maximum)")
        self.name_field.SetMaxLength(20)

        fields_sizer.Add(self.name_field, 0, wx.ALIGN_LEFT)

        # --------------------- The folder description label and field

        # Description
        description_text=wx.StaticText(self, -1, "Folder description")
        fields_sizer.Add(description_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.description_field = wx.TextCtrl(self, -1, self.folder.description, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("The description of this folder (50 characters maximum)")
        self.description_field.SetMaxLength(50)

        fields_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        sizer.Add(fields_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # ---------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 20)
        
        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel changes.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT, 20)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # Button binding methods

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """ Apply button - check and apply all fields"""

        name=self.name_field.GetValue().strip()
        description=self.description_field.GetValue().strip()

        if (name, description) == (self.folder.name, self.folder.description):
            # No change
            self.EndModal(wx.ID_OK)
            return

        # Check new folder name is ok
        status, message = edit_folder_check(self.folder, name)
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Set these into the database
        self.folder.name = name
        self.folder.description = description

        top_refresh_accounts(self.folder)
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close parent"""
        self.EndModal(wx.ID_CANCEL)

