####### VATStuff #######
#
# admin_tab.py  - displays the Admin panel and buttons
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" This is a module in the vatstuff_packages.vs_gui.admin package, and defines the Admin panel
which contains buttons to undo a transaction, backup, restore, delete old data,
open a new or existing database."""


import os
import datetime
import tarfile

import wx

from vatstuff_packages.vs_gui.admin import restore_backup
from vatstuff_packages.vs_gui.admin import new_database
from vatstuff_packages.vs_gui.admin import open_database
from vatstuff_packages.vs_gui.admin import undo_action

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_database import delete_old_data

from vatstuff_packages.vs_utilities import *


class AdminPanel(wx.Panel):
    """The admin panel, contains buttons to backup and restore the database."""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        self.parent = parent
        # Current Database filepath
        self.old_filepath=vs_cfg["DATABASE_FILEPATH"]
        self.old_logo=vs_cfg["LOGO_FILE"]
        self.old_write_access = vs_cfg["WRITE_ACCESS"]
        self.old_lockfile = vs_cfg["LOCK_FILEPATH"]

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
        panel_sizer=wx.BoxSizer(wx.VERTICAL)
             
        sizer=wx.GridBagSizer(hgap=20, vgap=20)

        # undo button
        self.undo_button = wx.Button(self, label="Undo", size=button_size)
        self.undo_button.SetToolTipString("Deletes the last transaction")   
        sizer.Add(self.undo_button, pos=(0,0), flag=wx.ALIGN_RIGHT)

        undo_text=wx.StaticText(self, -1, "Reverse and delete the last transaction")
        sizer.Add(undo_text, pos=(0,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Title
        text2=wx.StaticText(self, -1, "Backup your data - do regularly!")
        sizer.Add(text2, pos=(1,0), span=(1,2), flag=wx.ALIGN_LEFT)

        # backup button
        self.backup_button = wx.Button(self, label="Backup data", size=button_size)
        self.backup_button.SetToolTipString("Saves the database to a file")   
        sizer.Add(self.backup_button, pos=(2,0), flag=wx.ALIGN_RIGHT)

        backup_text=wx.StaticText(self, -1, "Create a compressed backup file of the database")
        sizer.Add(backup_text, pos=(2,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # restore button
        self.restore_button = wx.Button(self, label="Restore backup", size=button_size)
        self.restore_button.SetToolTipString("Loads in data from a previously created backup")   
        sizer.Add(self.restore_button, pos=(3,0), flag=wx.ALIGN_RIGHT)

        restore_text=wx.StaticText(self, -1, "WARNING! - Overwrites current data with the backup")
        sizer.Add(restore_text, pos=(3,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)


        # Text giving further info about delete old data function
        text3=wx.StaticText(self, -1, "Delete data older than a given date - to improve database performance:")
        sizer.Add(text3, pos=(4,0), span=(1,2), flag=wx.ALIGN_LEFT)

        # Delete old data button
        self.delete_old_button = wx.Button(self, label="Delete old data", size=button_size)
        self.delete_old_button.SetToolTipString("Deletes data older than a date you set")   
        sizer.Add(self.delete_old_button, pos=(5,0), flag=wx.ALIGN_RIGHT)

        delete_old_text=wx.StaticText(self, -1, "WARNING! - do backup first")
        sizer.Add(delete_old_text, pos=(5,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # new database button
        self.new_database_button = wx.Button(self, label="Create new database", size=button_size)
        self.new_database_button.SetToolTipString("Creates a new database file")   
        sizer.Add(self.new_database_button, pos=(6,0), flag=wx.ALIGN_RIGHT)

        new_database_text=wx.StaticText(self, -1, "Create a new, empty, database")
        sizer.Add(new_database_text, pos=(6,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # open database button
        self.open_database_button = wx.Button(self, label="Open a database", size=button_size)
        self.open_database_button.SetToolTipString("Opens an existing database file")   
        sizer.Add(self.open_database_button, pos=(7,0), flag=wx.ALIGN_RIGHT)

        open_database_text=wx.StaticText(self, -1, "Open a different, existing database file")
        sizer.Add(open_database_text, pos=(7,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # about button
        label_text = "About %s" % vs_cfg["PROGRAM_ID"]
        self.about_button = wx.Button(self, label=label_text, size=button_size)
        self.about_button.SetToolTipString("Display program information")   
        sizer.Add(self.about_button, pos=(8,0), flag=wx.ALIGN_RIGHT)

        about_text=wx.StaticText(self, -1, "Show version and license information")
        sizer.Add(about_text, pos=(8,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # quit button
        self.quit_button = wx.Button(self, wx.ID_EXIT, size=button_size)
        self.quit_button.SetToolTipString("Close the program")   
        sizer.Add(self.quit_button, pos=(9,0), flag=wx.ALIGN_RIGHT)

        quit_text=wx.StaticText(self, -1, "Close the program")
        sizer.Add(quit_text, pos=(9,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        panel_sizer.Add(sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30) 
        
        # Set the size to the panel
        self.SetSizer(panel_sizer)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.undo_button_pressed, self.undo_button)
        self.Bind(wx.EVT_BUTTON, self.backup_button_pressed, self.backup_button)
        self.Bind(wx.EVT_BUTTON, self.restore_button_pressed, self.restore_button)
        self.Bind(wx.EVT_BUTTON, self.delete_old_button_pressed, self.delete_old_button)
        self.Bind(wx.EVT_BUTTON, self.new_database_button_pressed, self.new_database_button)
        self.Bind(wx.EVT_BUTTON, self.open_database_button_pressed, self.open_database_button)
        self.Bind(wx.EVT_BUTTON, self.about_button_pressed, self.about_button)
        # The quit button is bound in the application frame, as it closes the whole app


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def undo_button_pressed(self, event):
        """Finds the last transaction, and if user confirms, deletes it with any associated
           database entries, such as the purchase order etc.,"""
        undo_action.undo_confirm_and_implement(self)


    @button_wrap
    def backup_button_pressed(self, event):
        "Create database backup file"
        self.create()


    @writecheck
    @button_wrap
    def restore_button_pressed(self, event):
        "Restore database from backup file"
        restore_backup.restore(self)


    @writecheck
    @button_wrap
    def delete_old_button_pressed(self, event):
        "Delete old data from the database"
         # Offer to do a backup
        if vs_yesno(self, "Deletes old data! Do you want to do a backup first?",
                            "Database backup"):
            # Create the backup
            self.create()
        else:
            # no backup - but close any open frames to ensure no database access
            # is taking place
            close_frames()
        busy_cursor_on()
        wx.Yield()
        status, to_date = get_deletion_date_dialog(self)
        if not status:
            return
        wx.Yield()
        # Show a progress bar
        dlg = None
        try:
            dlg = wx.ProgressDialog("Deleting",
                                    "Please wait as data is being deleted",
                                    100,   # progress bar max is 100
                                    self,
                                    wx.PD_APP_MODAL|wx.PD_CAN_ABORT)

            # do the deletion, logging the action at info level
            info_log("Delete database requested, prior to : %s" % to_date.isoformat())
            status, message = delete_old_data.do_delete(to_date, dlg)
        finally:
            if dlg: dlg.Destroy()
        wx.Yield()
        if status:
            vs_message(self, message, "DONE")
        else:
            vs_message(self, message, "ABORTED")
            info_log("Database deletion failed")


    @button_wrap
    def new_database_button_pressed(self, event):
        "Create a new database"
        new_database.build(self)


    @button_wrap
    def open_database_button_pressed(self, event):
        "Open an existing database"
        open_database.load(self)


    def about_button_pressed(self,event):
        "Show program info"
        # try except rather than a button_wrap as there is no need
        # to log this or to display a busy cursor
        try:
            # Create an AboutBox
            # First we create and fill the info object
            info = wx.AboutDialogInfo()
            info.Name = vs_cfg["PROGRAM_ID"]
            info.Version = vs_cfg["PROGRAM_VERSION"]
            info.Copyright = "(C) 2009,2010 Bernard Czenkusz"
            info.Description = """Double entry book keeping and VAT
recording for small businesses."""
            info.WebSite = (vs_cfg["PROGRAM_URL"], vs_cfg["PROGRAM_URL"])
            info.Developers = [ "Bernard Czenkusz  :  bernie@vatstuff.co.uk" ]
            info.License = """
VATStuff is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

VATStuff is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VATStuff, available within the pages under the
Manual tab.  If not, see <http://www.gnu.org/licenses/>."""
            # Then we call wx.AboutBox giving it that info object
            wx.AboutBox(info)
        except Exception, e:
            error_message("Error showing the About Box", e)


    def create(self):
        """Opens a save file dialog and creates the backup file"""

        # Database filepath
        filepath=vs_cfg["DATABASE_FILEPATH"]

        # logo image filepath
        logo_filepath = vs_cfg["LOGO_FILEPATH"]

        today_is = datetime.date.today()

        # default backup filename - a version of the database filename with .vat removed
        extension = ".%s" % vs_cfg["DATABASE_EXTENSION"]
        filepart = vs_cfg["DATABASE_FILE"].replace(extension, "")
        # put in the date, and give it a .bak extension 
        filename= "%s%s.bak" % (filepart, today_is.isoformat())

        # Create the save file dialog.
        status, path = save_file_as(self, 'bak', vs_cfg["BACKUP_FOLDER"], filename)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()
        # save the directory to the per-user persistent store
        directory = os.path.dirname(path)
        set_stored_item("BACKUP_FOLDER", directory)
        # save the backup filename to the per-user persistent store
        backup_file = os.path.basename(path)
        set_stored_item("BACKUP_FILE", backup_file)
        # close any open frames to ensure no database access is taking place
        # this function is under gui_functions
        close_frames()
        wx.Yield()
        fp=None
        try:
            fp = tarfile.open(path, 'w:bz2')
            fp.add(filepath, vs_cfg["ARCHIVED_FILE"], False)
            if os.path.exists(logo_filepath):
                fp.add(logo_filepath, vs_cfg["LOGO_IMAGE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        info_log("Backup file %s has been saved" % path)
        vs_message(self, "Your file has been saved", "Saved")


############ Get the date of database deletion

def get_deletion_date_dialog(parent):
    """Creates a GetDeletionDateDialog dialog, returns (True, to_date) on confirmation
       or (False, None) otherwise"""

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    get_deletion_date_dialog = GetDeletionDateDialog(parent)

    if get_deletion_date_dialog.ShowModal() != wx.ID_OK:
        get_deletion_date_dialog.Destroy()
        return False, None

    to_date = get_deletion_date_dialog.to_date
    get_deletion_date_dialog.Destroy()
    return True, to_date



class GetDeletionDateDialog(vsDialog):
    """Called to get the date to which data is to be deleted"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Delete old data")

        # function get_latest_delete_date is in odd_functions
        self.to_date = get_latest_delete_date()
        wx_todate = date_to_wxdate(self.to_date)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        sizer.Add(wx.StaticText(self, -1, "Delete data prior to the date:"), 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # The to date
        self.to_date_field = wx.DatePickerCtrl(self, -1,
                                               wx_todate, 
                                               size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                               style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.to_date_field.SetToolTipString("The to date")

        sizer.Add(self.to_date_field, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to delete data.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @button_wrap
    def apply_button_pressed(self, event):
        """ Apply button - check and apply all fields"""
        # Convert the dates input to python date objects
        # function wxdate_to_date is in module vatstuff_packages.vs_utilities.wx_utilities
        try:
            to_date = wxdate_to_date(self.to_date_field.GetValue())
        except Exception:
            vs_message(self, "The date is invalid", "ERROR")
            return

        # Check to_date is ok
        if to_date > self.to_date:
            vs_message(self, self.to_date.strftime("The latest date to which data can be deleted is %d, %B, %Y"),
                                                   "ERROR")
            return
        self.to_date = to_date
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)

