####### VATStuff #######
#
# company_config.py  - set company name, address, vat registration number and vat method
#
# Version : 0.6
# Date : 20100819
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Setup the company name, address, vat registration number and vat method

It is run as part of the vatstuff gui interface.
"""


import wx





from vatstuff_packages.vs_database.create_objects import create_Address

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import make_AddressForm



def company_config_dialog(parent):
    """Called when the company_configuration_button pressed"""
    company_config_dialog = CompanyConfigDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    company_config_dialog.ShowModal()
    company_config_dialog.Destroy()


class CompanyConfigDialog(vsDialog):
    "The dialog which configures the users own company"

    def __init__(self, parent):
        "Create the dialog"
        vsDialog.__init__(self, parent, "Company Configuration")

        self.company = Company()

        main_sizer=wx.BoxSizer(wx.VERTICAL)

        top_text=wx.StaticText(self, -1, "Set your companies name and registered office address:")
        main_sizer.Add(top_text, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # Add registered office panel
        self.company_name = self.company.name
        self.reg_office_address = self.company.registered_address

        # make a list of temporary copies
        if self.reg_office_address == None:
            self.address_list = []
        else:
            self.address_list = [make_AddressForm(self.reg_office_address)]

        flag_mask = { 'reg_office'     : 0,
                      'delivery_point' : 1,
                      'invoice_point'  : 1,
                      'order_point'    : 1,
                      'payment_point'  : 1 }
        flag_defaults = { 'reg_office'     : 1,
                          'delivery_point' : 1,
                          'invoice_point'  : 1,
                          'order_point'    : 1,
                          'payment_point'  : 1 }

        self.reg_office_panel = AddressPanel(self,
                                             self.company_name,
                                             self.address_list,
                                             flag_mask,
                                             flag_defaults,
                                             buttons = False)

        main_sizer.Add(self.reg_office_panel, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create fields and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=5)

        # Company registration number
        self.company_number_field = wx.TextCtrl(self, -1, self.company.company_number, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.company_number_field.SetToolTipString("Your company registration number")
        self.company_number_field.SetMaxLength(20)
        b_sizer.Add(self.company_number_field, 0, wx.ALIGN_RIGHT)

        company_number_text=wx.StaticText(self, -1, "Set your company registration number")
        b_sizer.Add(company_number_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Country of registration
        self.country_field = wx.TextCtrl(self, -1, self.company.country_registration, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.country_field.SetToolTipString("Your country of registration")
        self.country_field.SetMaxLength(20)
        b_sizer.Add(self.country_field, 0, wx.ALIGN_RIGHT)

        country_text=wx.StaticText(self, -1, "Country of registration")
        b_sizer.Add(country_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        main_sizer.Add(b_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 10) 

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        main_sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(main_sizer)
        main_sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        # Company office address
        status, company_name, address_list, deleted_addresses = self.reg_office_panel.get_addresses()
        if not status:
            return

        # Check registered office flag is set
        if address_list[0].reg_office !=1:
            vs_message(self, """This should be the companies registered office, therefore 
the registered office checkbox should be set.""", "ERROR")
            return

        # Company number
        cntext=self.company_number_field.GetValue()
        cntext=str(cntext).strip()
        if len(cntext) > 20:
            vs_message(self, "The company number is incorrect\nNo items have been changed.", "ERROR")
            return

        # Country of Registration
        crtext=self.country_field.GetValue()
        crtext=str(crtext).strip()
        if len(crtext) < 5 or len(crtext) > 20:
            vs_message(self, "The country of registration is incorrect\nNo items have been changed.", "ERROR")
            return


        # Set changes
        self.company.country_registration = crtext
        self.company.company_number = cntext
        self.company.name = company_name

        # Set the title of the top frame
        top_title(company_name)

        # Write registered office

        # Create an address
        address = address_list[0]

        if address.id < 0 :
            # This is a new address which has to be created
            database_address = create_Address(system=1)
        else:
            database_address = Address(address.id)

        database_address.lines = address.lines
        database_address.flags = address.flags

        self.company.registered_address = database_address

        # Make the default country for addresses, the same country
        # as the registered office
        vs_cfg["COUNTRY"] = database_address.country

        # Refresh top level buttons
        top_refresh_panels()
        vs_message(self, "Configuration saved.", "INFORMATION")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Close window after an information message"
        vs_message(self, "No changes have been made.", "INFORMATION")
        self.EndModal(wx.ID_CANCEL)

