####### VATStuff #######
#
# company_tab.py  - defines the company panel
#
# Version : 0.6
# Date : 20100723
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the company panel

It is run as part of the vatstuff gui interface.
"""

import os

import wx

from vatstuff_packages.vs_gui.company import company_config
from vatstuff_packages.vs_gui.company import edit_addresses

from vatstuff_packages.vs_gui import view_contacts
from vatstuff_packages.vs_gui import view_bank

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *


class CompanyPanel(wx.Panel):
    """The Company panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=7, cols=2, hgap=20, vgap=20)

        # company configuration
        self.company_config_button = wx.Button(self, label="Company configuration", size=button_size)
        self.company_config_button.SetToolTipString("Company name and VAT registration number")   
        b_sizer.Add(self.company_config_button, 0, wx.ALIGN_RIGHT)
        company_config_text=wx.StaticText(self, -1, "Press to configure your company")
        b_sizer.Add(company_config_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # addresses button
        self.address_button = wx.Button(self, label="Company Addresses", size=button_size)
        self.address_button.SetToolTipString("View and edit your company addresses")   
        b_sizer.Add(self.address_button, 0, wx.ALIGN_RIGHT)
        address_text=wx.StaticText(self, -1, "Add, view and edit your company addresses")
        b_sizer.Add(address_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # banking details button
        self.bank_button = wx.Button(self, label="Company bank details", size=button_size)
        self.bank_button.SetToolTipString("View and edit your company bank details")   
        b_sizer.Add(self.bank_button, 0, wx.ALIGN_RIGHT)
        bank_text=wx.StaticText(self, -1, "View and edit your company bank details")
        b_sizer.Add(bank_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # contacts button
        self.contacts_button = wx.Button(self, label="Company Contacts", size=button_size)
        self.contacts_button.SetToolTipString("These names can appear on order forms and invoices")   
        b_sizer.Add(self.contacts_button, 0, wx.ALIGN_RIGHT)

        contacts_text=wx.StaticText(self, -1, "Set contacts to be shown on Orders and Invoices")
        b_sizer.Add(contacts_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Company logo button
        self.logo_button = wx.Button(self, label="Company Logo", size=button_size)
        self.logo_button.SetToolTipString("Import a logo used on Invoices and Purchase Orders")   
        b_sizer.Add(self.logo_button, 0, wx.ALIGN_RIGHT)
        logo_text=wx.StaticText(self, -1, "Import your company logo image")
        b_sizer.Add(logo_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # delete logo
        self.delete_logo_button = wx.Button(self, label="Clear logo", size=button_size)
        self.delete_logo_button.SetToolTipString("Delete the current company logo image")   
        b_sizer.Add(self.delete_logo_button, 0, wx.ALIGN_RIGHT)
        delete_logo_text=wx.StaticText(self, -1, "Press to delete the current company logo")

        b_sizer.Add(delete_logo_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        self.sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # Get the logo image
        if os.path.exists(vs_cfg["LOGO_FILEPATH"]):
            logo_bitmap = wx.BitmapFromImage(wx.Image(vs_cfg["LOGO_FILEPATH"], wx.BITMAP_TYPE_PNG))
        else:
            logo_bitmap = wx.EmptyBitmap(vs_cfg["LOGO_MAXWIDTH"], vs_cfg["LOGO_MAXHEIGHT"])
            self.delete_logo_button.Enable(False)

        self.show_logo_bitmap=wx.StaticBitmap(self, -1, logo_bitmap)
        self.sizer.Add(self.show_logo_bitmap, 0, wx.TOP | wx.ALIGN_CENTER, 30)

        if not self.delete_logo_button.IsEnabled():
            # do not show the bitmap if it is empty
            self.show_logo_bitmap.Hide()
        
        # Set the size to the panel
        self.SetSizer(self.sizer)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.company_config_button_pressed, self.company_config_button)
        self.Bind(wx.EVT_BUTTON, self.bank_button_pressed, self.bank_button)
        self.Bind(wx.EVT_BUTTON, self.address_button_pressed, self.address_button)
        self.Bind(wx.EVT_BUTTON, self.contacts_button_pressed, self.contacts_button)
        self.Bind(wx.EVT_BUTTON, self.logo_button_pressed, self.logo_button)
        self.Bind(wx.EVT_BUTTON, self.delete_logo_button_pressed, self.delete_logo_button)


    # ----------------- Handler functions

    @button_wrap
    def company_config_button_pressed(self, event):
        "Configure company name and registered office"
        company_config.company_config_dialog(self)


    @button_wrap
    def bank_button_pressed(self, event):
        "Create and show the BankDetailsDialog to edit bank info for the company"
        view_bank.bank_details_dialog(self, Company())


    @button_wrap
    def address_button_pressed(self, event):
        "Edit and add company addresses"
        edit_addresses.edit_company_address_dialog(self)


    @button_wrap
    def contacts_button_pressed(self, event):
        "Edit and add company contacts"
        view_contacts.view_contacts_dialog(self, system = 1)


    @writecheck
    @button_wrap
    def logo_button_pressed(self, event):
        "Load a company logo .png image"
        # Create the load file dialog.
        # Pre-establish a file filter to show the .png extension
        wildcard = "Image file (*.png;*.gif;*.jpg)|*.png;*.gif;*.jpg|All files (*.*)|*.*"
        # Assume the image is where the backup files are kept
        busy_cursor_off()
        dlg = wx.FileDialog(self, message="Load image file...",
                                  defaultDir=vs_cfg["BACKUP_FOLDER"],
                                  wildcard=wildcard,
                                  style=wx.OPEN)
        # Show the dialog and retrieve the user response.
        # If it is not the OK response, abandon operation
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            dlg.Destroy()
        else:
            dlg.Destroy()
            return False

        busy_cursor_on()
        wx.Yield()

        # Test if the image is loadable
        if not wx.Image.CanRead(path):
            vs_message(self, "Sorry, cannot read that file.", "ERROR")
            return

        old_logo_filepath = vs_cfg["LOGO_FILEPATH"]
        old_logo_file = vs_cfg["LOGO_FILE"]

        # create a new filename for "LOGO_FILEPATH" and "LOGO_FILE"
        vs_cfg["LOGO_FILEPATH"], vs_cfg["LOGO_FILE"] = create_logo_name()

        # save image as png file to vs_cfg["LOGO_FILEPATH"]

        try:
            logo_image= wx.Image(path, wx.BITMAP_TYPE_ANY)

            # Scale the image to fit inside vs_cfg["LOGO_MAXWIDTH"], vs_cfg["LOGO_MAXHEIGHT"]
            logo_width, logo_height = logo_image.GetSize()
            if logo_width > vs_cfg["LOGO_MAXWIDTH"] or logo_height > vs_cfg["LOGO_MAXHEIGHT"]:
                # scale the image down to the maximum width or height
                mul_w = float(vs_cfg["LOGO_MAXWIDTH"])/logo_width
                mul_h = float(vs_cfg["LOGO_MAXHEIGHT"])/logo_height
                # multiplier is the smallest of the two
                multiplier = mul_w if mul_w < mul_h else mul_h
                logo_image.Rescale(int(logo_width * multiplier),
                                   int(logo_height * multiplier),
                                   wx.IMAGE_QUALITY_HIGH)

            logo_image.SaveFile(vs_cfg["LOGO_FILEPATH"], wx.BITMAP_TYPE_PNG)

            # save this in the database
            set_config_item("LOGO_FILE", vs_cfg["LOGO_FILE"])
        except Exception, e:
            vs_cfg["LOGO_FILEPATH"] = old_logo_filepath
            vs_cfg["LOGO_FILE"] = old_logo_file
            error_message("ERROR: Failed to load image", e)
            return

        if os.path.exists(old_logo_filepath):
            os.remove(old_logo_filepath)
        
        # So a new image has been loaded
        self.refresh()
            
       
    @writecheck
    @button_wrap
    def delete_logo_button_pressed(self, event):
        "Delete the logo"
        # Ask for confirmation
        if not vs_yesno(self, "Delete the company logo image, are you sure?", "Delete logo"):
            return
        if os.path.exists(vs_cfg["LOGO_FILEPATH"]):
            os.remove(vs_cfg["LOGO_FILEPATH"])
        self.show_logo_bitmap.Hide()
        self.delete_logo_button.Enable(False)


    def refresh(self):
        # Get the logo image
        if os.path.exists(vs_cfg["LOGO_FILEPATH"]):
            logo_bitmap = wx.BitmapFromImage(wx.Image(vs_cfg["LOGO_FILEPATH"], wx.BITMAP_TYPE_PNG))
            self.show_logo_bitmap.SetBitmap(logo_bitmap)
            self.show_logo_bitmap.Show()
            self.delete_logo_button.Enable(True)
        else:
            logo_bitmap = wx.EmptyBitmap(vs_cfg["LOGO_MAXWIDTH"], vs_cfg["LOGO_MAXHEIGHT"])
            self.show_logo_bitmap.SetBitmap(logo_bitmap)
            self.show_logo_bitmap.Hide()
            self.delete_logo_button.Enable(False)
        self.sizer.Layout()


