####### VATStuff #######
#
# manual_tab.py  - displays the Manual panel and html pages
#
# Version : 0.2
# Date : 20090926
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Displays the Manual panel, with navigation buttons."""

import os
import os.path
import sys

import wx
import wx.html

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *


class ManualPanel(wx.Panel):
    """The Manual panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # top navigation buttons
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        self.top_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Index button
        self.index_button = wx.Button(self, label="Index", size=button_size)
        self.index_button.SetToolTipString("Display the first index page")   
        self.top_sizer.Add(self.index_button, 0, wx.RIGHT, 10)

        # Back button
        self.back_button = wx.Button(self, label="Back", size=button_size)
        self.back_button.SetToolTipString("Display the previous page")   
        self.top_sizer.Add(self.back_button, 0, wx.RIGHT, 10)

        # Forwards button
        self.forward_button = wx.Button(self, label="Forward", size=button_size)
        self.forward_button.SetToolTipString("Display the next page")   
        self.top_sizer.Add(self.forward_button, 0, wx.RIGHT, 10)

        # Statistics button
        self.stats_button = wx.Button(self, label="Statistics", size=button_size)
        self.stats_button.SetToolTipString("Display the program parameters")   
        self.top_sizer.Add(self.stats_button, 0, wx.RIGHT, 10)

        # Logs button
        self.logs_button = wx.Button(self, label="Logs", size=button_size)
        self.logs_button.SetToolTipString("Display the program logs")   
        self.top_sizer.Add(self.logs_button, 0)

        self.sizer.Add(self.top_sizer, 0, wx.TOP | wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 20)

        # The HTML window

        # The manual is stored as a series of htm pages within the folder vs_cfg["HTML_FOLDER"]
        # the index page being the first in the vs_cfg["HTML_FILES"] list

        self.manual = wx.html.HtmlWindow(self)
        if "gtk2" in wx.PlatformInfo:
            self.manual.SetStandardFonts()

        # Note - putting self.manual.LoadFile at this point sometimes hangs
        # and sometimes it don't - seems to be a bug in wxPython, so instead
        # I am putting a startup page

        text = """\
<html>
<body>

<p><b>PROGRAM IN DEVELOPMENT - THIS PROGRAM HAS NOT YET BEEN SUFFICIENTLY TESTED.<br />
PLEASE TEST AND GIVE FEEDBACK BUT DO NOT RELY ON THIS PROGRAM YET.</b></p>

<h3>%s</h3>
<p><em>&nbsp;&nbsp;- a program providing double entry bookkeeping and VAT calculations</em></p>

<p>Choose the 'Index' button for the index to this manual.</p>

<p>Under the Accounts tab you will see a list of accounts organised into folders, double click
on any account to show a statement list of account transactions.</p>

<p>The Sales and Purchases tabs allow you add customers and suppliers, then you can create
invoices and orders.</p>

<p>The Move Money tab allows you to move money between accounts.</p>

<p>The VAT tab, allows you to set VAT rates, your VAT scheme, and produce VAT returns.</p>

<p>The Reports tab allows you to generate miscallaneous reports.</p>

<p>The Company tab allows you to set your own company information.</p>

<p>The Admin tab gives you the facility to create backups,
start new databases - and also has the Quit button.</p>

</body>
</html>
""" % vs_cfg["PROGRAM_ID"]

        self.manual.SetPage(text)

        # This widget needs to expand in all directions
        self.sizer.Add(self.manual, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.index_button_pressed, self.index_button)
        self.Bind(wx.EVT_BUTTON, self.back_button_pressed, self.back_button)
        self.Bind(wx.EVT_BUTTON, self.forward_button_pressed, self.forward_button)
        self.Bind(wx.EVT_BUTTON, self.stats_button_pressed, self.stats_button)
        self.Bind(wx.EVT_BUTTON, self.logs_button_pressed, self.logs_button)


        self.dynamic_page = True
        self.max_page = len(vs_cfg["HTML_FILES"])-1


    def refresh(self):
        "If a dynamic page, call a refreshed stats page"
        if not self.dynamic_page:
            return
        reporttext = self.create_report()
        self.manual.SetPage(reporttext)

    def index_button_pressed(self, event):
        "Go to the index page"
        self.dynamic_page = False
        try:
            self.manual.LoadFile(os.path.join(vs_cfg["HTML_FOLDER"], vs_cfg["HTML_FILES"][0]))
        except Exception, e:
            error_message("Unable to load the index page", e)

    def back_button_pressed(self, event):
        "Go back a page"
        try:
            if self.dynamic_page:
                # If a dynamic page is being shown, go to the index
                self.index_button_pressed(event)
                return
            # Find out at which point in this list we are currently displaying
            filename = os.path.basename( self.manual.GetOpenedPage() )
            page_index=vs_cfg["HTML_FILES"].index(filename)
            if not page_index:
                # At first index page, cannot go further
                return
            next_page = os.path.join(vs_cfg["HTML_FOLDER"], vs_cfg["HTML_FILES"][page_index-1])
            self.manual.LoadFile(next_page)
        except Exception, e:
            error_message("Unable to load the previous page", e)

    def forward_button_pressed(self, event):
        "If stats or logs page is showing, go to index, otherwise go forward a page"
        try:
            if self.dynamic_page:
                self.index_button_pressed(event)
                return
            # Find out at which point in this list we are currently displaying
            filename = os.path.basename( self.manual.GetOpenedPage() )
            page_index=vs_cfg["HTML_FILES"].index(filename)
            if page_index == self.max_page:
                # At last page, cannot go further
                return
            next_page = os.path.join(vs_cfg["HTML_FOLDER"], vs_cfg["HTML_FILES"][page_index+1])
            self.manual.LoadFile(next_page)
        except Exception, e:
            error_message("Unable to load the next page", e)


    def stats_button_pressed(self, event):
        "Show the page of program statistics"
        self.dynamic_page = True
        try:
            reporttext = self.create_report()
            self.manual.SetPage(reporttext)
        except Exception, e:
            error_message("Unable to load the statistics page", e)


    def logs_button_pressed(self, event):
        "Show the logs"
        self.dynamic_page = True
        try:
            reporttext = self.show_logs()
            self.manual.SetPage(reporttext)
        except Exception, e:
            error_message("Unable to load the logs", e)


    def create_report(self):
        "Creates a report showing software versions"
        report_text = """\
<html>
<body>
<h3>Program statistics</h3>
<p>%s Version: %s<br />
Database schema: %s<br />
Release date: %s</p>
<table cellspacing="10">
<tr><td>Installation directory:</td><td>%s</td></tr>
<tr><td>Database file:</td><td>%s</td></tr>
<tr><td>Logs directory:</td><td>%s</td></tr>
<tr><td>Configuration file:</td><td>%s</td></tr>
</table>
<h3>Python</h3>
<p>%s</p>
<h3>wxPython</h3>
<p>%s</p>""" % (vs_cfg["PROGRAM_ID"],
              vs_cfg["PROGRAM_VERSION"],
              vs_cfg["DATABASE_VERSION"],
              vs_cfg["RELEASE_DATE"],
              vs_cfg["SCRIPTDIRECTORY"],
              vs_cfg["DATABASE_FILEPATH"],
              vs_cfg["LOG_FOLDER"],
              vs_cfg["CONFIG_LOCATION"],
              sys.version,
              wx.version())

        if vs_cfg["REPORTLABS_AVAILABLE"]:
            import reportlab
            report_text += """\
<h3>ReportLab</h3>
<p>%s</p>""" % reportlab.Version
        else:
            report_text += """\
<h3>ReportLab</h3>
<p>Module unavailable</p>"""

        if vs_cfg["PIL_AVAILABLE"]:
            import Image
            report_text += """\
<h3>Python Imaging Library</h3>
<p>%s</p>""" % Image.VERSION
        else:
            report_text += """\
<h3>Python Imaging Library</h3>
<p>Module unavailable</p>"""

        report_text += "</body></html>"

        return report_text


    def show_logs(self):
        "Creates a report showing logs"
        if not vs_cfg["LOGGING"]:
            report_text = """\
<html>
<body>
<p>No logs are available: Logging has been turned off</p>
</body>
</html>"""
            return report_text

        report_text = """\
<html>
<body>
<h3>Program Logs</h3>
<p>Contents of log file: %s</p>
<pre>
""" % vs_cfg["LOG_FILEPATH"]


        f = None
        try:
            f = open(vs_cfg["LOG_FILEPATH"], 'r')
            report_text += f.read()
        except Exception:
            report_text = """\
<html>
<body>
<h3>Program Logs</h3>
<p>Error trying to read log file: %s</p>
</body>
</html>
""" % vs_cfg["LOG_FILEPATH"]
        else:
            report_text += "</pre></body></html>"
        finally:
            if f:
                f.close()

        return report_text


