####### VATStuff #######
#
# complex_move.py  - ComplexMoveDialog displayed when a button on the movemoney panel is pressed
#
# Version : 0.6
# Date : 20100805
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Holds ComplexMoveDialog displayed when a button on the movemoney panel is pressed

It is run as part of the vatstuff gui interface.
"""


import datetime
import decimal

import wx





from vatstuff_packages.vs_database.create_objects import create_Definition

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.movemoney import simple_move

D = decimal.Decimal   # D used to avoid typing


# --------------- ComplexMoveDialog is called by the memory buttons - it defines
# --------------- and implements a set of definitions, stores them in the
# --------------- databases, and associates them with a label and hence a button



def complex_move_dialog(parent, label):
    """Creates a ComplexMoveDialog"""
    complex_move_dialog = ComplexMoveDialog(parent, label)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    complex_move_dialog.ShowModal()
    complex_move_dialog.Destroy()


class ComplexMoveDialog(vsDialog):
    """ComplexMoveDialog is called by the memory buttons - it defines
       and implements a set of definitions, and stores them under a label"""

    def __init__(self, parent, label):
        """Create the Dialog - holding a complex money move definition"""
        if label.button < vs_cfg["MAX_BUTTONS"]:
            on_button = True
            dialog_title = "Define a transaction on button %s" % (label.button + 1)
        else:
            on_button = False
            dialog_title = "Define a transaction"

        vsDialog.__init__(self, parent, dialog_title)
        
        # Take a note of the variables
        self.parent = parent
        self.label = label

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # Button text
        if on_button:
            button_text=wx.StaticText(self, -1, "Button text")
        else:
            button_text=wx.StaticText(self, -1, "Selection text")
        b_sizer.Add(button_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.button_field = wx.TextCtrl(self, -1, self.label.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.button_field.SetToolTipString("The label text to be saved")
        self.button_field.SetMaxLength(20)

        b_sizer.Add(self.button_field, 0, wx.ALIGN_LEFT)

        # Description text
        description_text=wx.StaticText(self, -1, "Description")
        b_sizer.Add(description_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.description_field = wx.TextCtrl(self, -1, self.label.description, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("The description to be displayed")
        self.description_field.SetMaxLength(50)

        b_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        # add the b_sizer to the main vertical sizer

        sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help_text=wx.StaticText(self, -1, """\
Set the accounts to either Debit or Credit. The Value can be either a currency amount, or could refer to
a previous line using a reference such as 'Value1', 'Operand3' or 'Result2'. For example; 'Result2' refers
to the result of line 2 - so the Result of one line can be used as the Value of a subsequent line. If an
operator is chosen, the operand can be a value, refer to a previous line, or be a percentage, such as 20%.""")

        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # -------------------- The definition panel

        # The definition_panel
        self.definition_panel = DefinitionPanel(self, self.label)

        # Create a horizontal box sizer to hold the definition panel
        definition_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # add a bit of space
        definition_sizer.Add((10,10), 1)

        # Add the definition panel to the sizer
        definition_sizer.Add(self.definition_panel, 0, wx.ALIGN_CENTER)

        # add a bit of space
        definition_sizer.Add((10,10), 1)

        sizer.Add(definition_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)
        
        # Save button
        self.save_button = wx.Button(self, wx.ID_SAVE, size=button_size)
        self.save_button.SetToolTipString("Click to save this definition.")
        button_sizer.Add(self.save_button, 0, wx.RIGHT, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to implement the transaction.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 40)
        
        # Calculate button
        self.calculate_button = wx.Button(self, label="Calculate", size=button_size)
        self.calculate_button.SetToolTipString("Click to calculate the above results.")
        button_sizer.Add(self.calculate_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.save_button_pressed, self.save_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.calculate_button_pressed, self.calculate_button)


    # ----------------- Handler functions
    
    @writecheck
    @button_wrap
    def save_button_pressed(self, event):
        """ Save button - check and save all fields"""

        # Get top fields
        button_text=self.button_field.GetValue().strip()
        description_text = self.description_field.GetValue().strip()

        # Check transaction fields,
        status, transaction_list = self.definition_panel.check_fields()
        if not status:
            return

        if not vs_yesno(self,
                        """You are about to save this definition. Are you sure?
NOTE - This does not implement the transaction - press Apply to do that.""",
                       "Save Transaction Definition"):
            return

        busy_cursor_on()
        wx.Yield()

        # Update the label in the database
        self.label.name = button_text
        self.label.description = description_text

        # Store the definition, with the label
        self.definition_panel.store_definition(self.label)

        # refresh the parent - either the movemoney tab panel, or the more moves frame
        wx.Yield()
        self.parent.refresh()
        vs_message(self, "Transaction definition saved", "INFORMATION")


    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        # Get button description, and pass it on so it could be used
        # as the transaction memo
        description_text = self.description_field.GetValue().strip()

        # Check the definition fields, and get a list of transactions [account, debit, credit]
        status, transaction_list = self.definition_panel.check_fields()
        if not status:
            return

        # Call and populate simple_move_dialog to do the transaction
        simple_move.simple_move_dialog(self, description_text, transaction_list)
        

    def close_button_pressed(self, event):
        """Close button"""
        self.EndModal(wx.ID_OK)


    def calculate_button_pressed(self, event):
        """Calculate button - check and calculate all fields"""
        # Check transaction fields,
        try:
            self.definition_panel.check_fields()
        except Exception, e:
            error_message("Failure when checking the panel fields", e)



############################################################################################
#
# DefinitionPanel - this class displays, checks and saves the definitions of a complex transaction
#
############################################################################################

class DefinitionPanel(wx.Panel):
    """The initiator takes a label, reads and displays the corresponding definitions
       the user can edit the definition, and this class has methods to store the updated
       definitions and to return a transaction list of [account, debit, credit]'s """

    max_items = 8    # the maximum number of rows to display

    def __init__(self, parent, label):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.label = label

        # Get a list of the stored definitions that have label_id

        self.definition_list = self.label.get_definitions()

        # self.definition_list is a list, either empty or containing Definition objects
        # DEFINITIONS table consists of

        # DEFINITION_ID
        # LABEL_ID
        # ACCOUNT_ID
        # ACTION         - 0 for debit, 1 for credit
        # VALUE          - held as a text string
        # OPERATOR       - 0,1,2,3 or 4 for " ","+","-","x" or "/"
        # OPERAND        - text string

        def_len = len(self.definition_list)  # The number of records (could be zero)

        assert def_len <= DefinitionPanel.max_items

        # Show definitions in a grid self.sizer
        self.sizer=wx.FlexGridSizer(rows=DefinitionPanel.max_items+1, cols=9, hgap=10, vgap=10)
        # DefinitionPanel.max_items+1 the +1 is for the header text titles

        # header titles
        
        row_text=wx.StaticText(self, -1, "  ")
        self.sizer.Add(row_text, 0, wx.ALIGN_CENTER)

        account_text=wx.StaticText(self, -1, "Account")
        self.sizer.Add(account_text, 0, wx.ALIGN_CENTER)

        action_text=wx.StaticText(self, -1, "Action")
        self.sizer.Add(action_text, 0, wx.ALIGN_CENTER)

        by_text=wx.StaticText(self, -1, "")
        self.sizer.Add(by_text, 0, wx.ALIGN_CENTER)

        value_text=wx.StaticText(self, -1, "Value")
        self.sizer.Add(value_text, 0, wx.ALIGN_CENTER)

        operator_text=wx.StaticText(self, -1, "Operator\n(optional)")
        self.sizer.Add(operator_text, 0, wx.ALIGN_CENTER)

        operand_text=wx.StaticText(self, -1, "Operand\n(optional)")
        self.sizer.Add(operand_text, 0, wx.ALIGN_CENTER)

        equals_text=wx.StaticText(self, -1, "")
        self.sizer.Add(equals_text, 0, wx.ALIGN_CENTER)

        result_text=wx.StaticText(self, -1, "        Result", size=(100,-1))
        self.sizer.Add(result_text, 0, wx.ALIGN_CENTER)
        
        # self.account_objects_list is a list of account objects
        # self.account_names_list is a list of account names in foldername:accountname format
        # the two lists are in the same order
        self.account_objects_list, self.account_names_list = get_folder_account_lists()

        self.account_names_list.insert(0, "")              # Insert a blank at the beginning of the list
        self.account_objects_list.insert(0, None)          # Keep two lists ordered together

        # So self.account_objects_list holds all account objects in the same order as does self.account_names_list
        # which is used to display the accounts in the choice menu

        self.action_list=["Debit", "Credit"]        # Stored as 0 or 1 in the Definitions table
        self.operator_list = [" ","+","-","x","/"]  # Stored as 0,1,2,3 or 4 in the Definitions table

        self.widget_list=[] # to consist of lists of account, action, value, operator, operand, result, field id's
                            # note - widgit field id's not to be confused with account id's
        
        # create definition fields
        for tran in range(0,DefinitionPanel.max_items):

	        # Row number text
            self.sizer.Add(wx.StaticText(self, -1, "%i" % (tran+1,)), 0, wx.ALIGN_CENTER)

            # Account field
            acc_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.account_names_list)
            acc_id = acc_field.GetId()
            acc_field.SetToolTipString("Set the account to change")
            self.sizer.Add(acc_field, 0, wx.ALIGN_CENTER)

            # Action field
            act_field = wx.Choice(self, -1, size=(100, vs_cfg["BUTTON_HEIGHT"]), choices=self.action_list)
            act_id = act_field.GetId()
            act_field.SetToolTipString("Set Debit or Credit")
            self.sizer.Add(act_field, 0, wx.ALIGN_CENTER)

	        # By text
            self.sizer.Add(wx.StaticText(self, -1, "by"), 0, wx.ALIGN_CENTER)

            # Value field
            val_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(100, vs_cfg["BUTTON_HEIGHT"]))
            val_id = val_field.GetId()
            val_field.SetToolTipString("Set the value to debit or credit the account by")
            val_field.SetMaxLength(10)
            self.sizer.Add(val_field, 0, wx.ALIGN_CENTER)

            # Operator field
            opr_field = wx.Choice(self, -1, size=(50, vs_cfg["BUTTON_HEIGHT"]), choices=self.operator_list)
            opr_id = opr_field.GetId()
            opr_field.SetToolTipString("If required set an operator to change the value")
            self.sizer.Add(opr_field, 0, wx.ALIGN_CENTER)

            # Operand field
            opd_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(100, vs_cfg["BUTTON_HEIGHT"]))
            opd_id = opd_field.GetId()
            opd_field.SetToolTipString("Set the operand, as well as a value,\nthis could be a percentage, such as 25%")
            opd_field.SetMaxLength(10)
            self.sizer.Add(opd_field, 0, wx.ALIGN_CENTER)

	        # Equals text
            self.sizer.Add(wx.StaticText(self, -1, "="), 0, wx.ALIGN_CENTER)

	        # Results text
            res_field = wx.StaticText(self, -1, "")
            res_id = res_field.GetId()
            self.sizer.Add(res_field, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
                       
            # record the id's
            self.widget_list.append([acc_id, act_id, val_id, opr_id, opd_id, res_id])

            # Set the field values, where a definition exists
            if tran < def_len:
                if self.definition_list[tran].account == None:              # account_id
                    acc_field.SetSelection(0)
                else:
                    account_selected = self.definition_list[tran].account
                    acc_field.SetSelection(self.account_objects_list.index(account_selected))

                act_field.SetSelection(self.definition_list[tran].action)        # Action field
                val_field.SetValue(self.definition_list[tran].value)             # Value
                opr_field.SetSelection(self.definition_list[tran].operator)      # Operator field
                opd_field.SetValue(self.definition_list[tran].operand)           # Operand

        # Set the size to the panel
        self.SetSizer(self.sizer)

        self.Fit()


    def store_definition(self, label):
        "Store this definition in the Definitions database"
         
        self.label = label
        # Delete existing definitions with this label from database
        self.label.delete_definitions()

        # Add new definitions with label
        
        for def_item in range(0, DefinitionPanel.max_items):

            # Account field
            acc_field = self.FindWindowById(self.widget_list[def_item][0])
            account_selected_list_item = acc_field.GetCurrentSelection()
            if account_selected_list_item > 0:
                account = self.account_objects_list[account_selected_list_item]
            else:
                account = None


            # Action field
            act_field = self.FindWindowById(self.widget_list[def_item][1])
            action_selected_list_item = act_field.GetCurrentSelection()
            if action_selected_list_item < 0:
                action_selected_list_item = 0
            action = action_selected_list_item

            # Value field
            val_field = self.FindWindowById(self.widget_list[def_item][2])
            value = val_field.GetValue()
            
            # Operator field
            opr_field = self.FindWindowById(self.widget_list[def_item][3])
            operator_selected_list_item = opr_field.GetCurrentSelection()
            if operator_selected_list_item < 0:
                operator_selected_list_item = 0
            operator = operator_selected_list_item    

            # Operand field
            opd_field = self.FindWindowById(self.widget_list[def_item][4])
            operand=opd_field.GetValue()     

            # Create definition for button defined by this label 
            definition = create_Definition(label, account, action, value, operator, operand)

    def check_fields(self):
        """Returns [boolean status, list of results]

        status is True if all fields are ok, False otherwise
        results is a list of lists - [account, debit, credit]"""
        
        # self.widget_list[item] consists of [acc_id, act_id, val_id, opr_id, opd_id, res_id]
        # these being widget field id's - not to be confused with account id's
        
        self.result_list = []

        for item in range(0,DefinitionPanel.max_items):
        
            # Get account id into result
            result = []

            # Account field
            acc_field = self.FindWindowById(self.widget_list[item][0])
            account_selected_list_item = acc_field.GetCurrentSelection()
            if account_selected_list_item < 1:         # blank or unselected
                result.append(None)
            else:
                account_selected = self.account_objects_list[account_selected_list_item]
                result.append(account_selected)
            
            # Action Field
            act_field = self.FindWindowById(self.widget_list[item][1])
            action_selected_list_item = act_field.GetCurrentSelection()
            if action_selected_list_item < 0: action_selected_list_item = 0
            # 0 if this is a debit
            # 1 if this is a credit

            # Check value, operator and operand, and return a result value
            # in pennies
            status, result_value = self.check_value_operator_operand(item)
            if not status:
                return False, None
                
            debit_amount = D("0.00")
            credit_amount = D("0.00")
                
            if action_selected_list_item == 0:
                debit_amount = result_value
            else:
                credit_amount = result_value

            result.append(debit_amount)
            result.append(credit_amount)
            # Add this result list to the list of lists
            self.result_list.append(result)
            result = []

        return True, self.result_list


    def check_value_operator_operand(self, item):
        "Check value, operator and operand, and return a decimal result"
        
        # value here is the string from the value field
        status, value = self.get_value(item)
        if not status: return False, None     

        status, result_value = string_to_decimal(value)
        if not status:
            vs_message(self, "Invalid Value at line %i" % (item+1), "ERROR")
            return False, None
            
        
        # Operator field
        opr_field = self.FindWindowById(self.widget_list[item][3])
        operator_selected_list_item = opr_field.GetCurrentSelection()
        if operator_selected_list_item < 0 : operator_selected_list_item = 0
        
        # Operand field
        status, result_value = self.check_operand(item, result_value, operator_selected_list_item)
        if not status:
            return False, None
        
        # Get the result_value as a string
        string_result_value = unicode(Currency(result_value))
        # Put the value into the results column
        res_field = self.FindWindowById(self.widget_list[item][5])
        res_field.SetLabel(string_result_value)
        # Re-do the layout to sort out right justification of this field
        self.sizer.Layout()
        return True, result_value


    def check_operand(self, item, result_value, operator_selected_list_item):
        result = result_value
        status, operand = self.get_operand(item)
        if not status: return False, None
        
        if operator_selected_list_item < 1 and operand != vs_cfg["ZERO_STRING"]:
            # No operator has been selected, operand must be a percentage
            if operand[-1:] == "%":
                return self.get_op_percent(item, result_value, operand)
            else:
                vs_message(self,
                           "Error at line %i: With no operator selected, the operand must\
 be either %s to indicate no action, or it could be a percentage, (for example 10%%) and the\
 result will be that percentage of the Value. No other operand is accepted" % (item+1, vs_cfg["ZERO_STRING"]),
                           "ERROR")
                return False, None
                
        if operator_selected_list_item < 1:
            # No operator selected, and operand must be at default 0.00
            return True, result_value
        
        if operator_selected_list_item == 1 or operator_selected_list_item == 2:
            # + or - has been selected, operand must be currency or percentage
            if operand[-1:] == "%":
                status, op_value = self.get_op_percent(item, result_value, operand)
                if not status: return False, None
            else:
                # operand must be a currency
                status, op_value = string_to_decimal(operand)
                if not status:
                    vs_message(self,
                               "Invalid Operand at line %i, it is not a currency amount\
 (ie, a number with two decimal places) or a percentage" % (item+1),
                               "ERROR")
                    return False, None
            if operator_selected_list_item == 1:
                return True, result_value + op_value
            else:
                return True, result_value - op_value
            
        if operator_selected_list_item == 3 or operator_selected_list_item == 4:
            # operator must be * or /, so operand must be a number
            # Check for a numeric value
            try:
                op_value = D(operand.replace(vs_cfg["CP"], u"."))
            except Exception:
                vs_message(self, "Invalid Operand at line %i" % (item+1), "ERROR")
                return False, None

        if operator_selected_list_item == 3:
            result = result_value * op_value
            return True, result.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
            
        if operator_selected_list_item == 4:
            try:
                result = result_value / op_value
            except Exception:
                # Division fails
                vs_message(self, "Invalid division at line %i" % (item+1), "ERROR")
                return False, None
        return True, result.quantize(D('.01'), rounding=decimal.ROUND_DOWN)


    def get_op_percent(self, item, result_value, operand):
        "Returns status, and the percentage of result_value defined by operand"
        operand = operand[:-1]
        try:
            op_value = D(operand.replace(vs_cfg["CP"], u"."))
        except Exception:
            wvs_message(self, "Invalid Operand at line %i" % (item+1), "ERROR")
            return False, None
        
        result = (result_value * op_value) / D("100.0")
        return True, result.quantize(D('.01'), rounding=decimal.ROUND_DOWN)

        
    def get_value(self, item):
        "Get the string value from line number item"
        val_field = self.FindWindowById(self.widget_list[item][2])
        value=val_field.GetValue()
        value=unicode(value).strip()
        if value=="": value = vs_cfg["ZERO_STRING"]
        status, value = self.check_for_references("Value", value, item)        
            
        return status, value
        
        
    def get_operand(self, item):
        "Get the string operand from line number item"
        opd_field = self.FindWindowById(self.widget_list[item][4])
        operand=opd_field.GetValue()
        operand=unicode(operand).strip()
        if operand=="": operand = vs_cfg["ZERO_STRING"]
        status, operand = self.check_for_references("Operand", operand, item)        
   
        return status, operand
        
        
    def get_result(self, item):
        "Get a result from line item, from list self.result_list"
        number_in_result_list = len(self.result_list)
        if item >= number_in_result_list:
            vs_message(self, "Error at line %i, only a previous line can be referenced" % (item+1), "ERROR")
            return False, None
        # add the debit and credit, since one must be zero this gives the result
        result = self.result_list[item][1] + self.result_list[item][2]
        return True, unicode(Currency(result))
        

    def check_for_references(self, field, reference, item):
        if (reference[0] != "V" and
            reference[0] != "v" and
            reference[0] != "O" and
            reference[0] != "o" and
            reference[0] != "R" and
            reference[0] != "r"):
            # There is no call for a reference, so just return
            # with the original value
            return True, reference

        # Check last character is a digit, 1 to 8
        try:
            line = int(reference[-1:]) -1
        except Exception:
            vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
            return False, None

        if line<0 or line>7:
            vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
            return False, None

        # The first line cannot reference any other line
        if item == 0:
            vs_message(self,
                       "The first line cannot reference any other line\
as only previous lines can be referenced",
                       "ERROR")
            return False, None

        # A line can only reference a previous line
        if line >= item:
            vs_message(self,
                       "%s is incorrect at line %i, only a previous line can be referenced" % (field, item+1),
                       "ERROR")
            return False, None

        # Check for variable such as Value1
        if reference[0]=="V" or reference[0]=="v":
            # Check if this is a correct reference
            if len(reference) != 6:
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None

            if reference[0:-1] != "Value" and reference[0:-1] != "VALUE" and reference[0:-1] != "value":
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None
            return self.get_value(line)
            
        # Check for variable such as Operand1
        if reference[0]=="O" or reference[0]=="o":
            # Check if this is a correct reference
            if len(reference) != 8:
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None

            if reference[0:-1] != "Operand" and reference[0:-1] != "OPERAND" and reference[0:-1] != "operand":
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None
            return self.get_operand(line)
            
        # Check for variable such as Result1
        if reference[0]=="R" or reference[0]=="r":
            # Check if this is a correct reference
            if len(reference) != 7:
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None

            if reference[0:-1] != "Result" and reference[0:-1] != "RESULT" and reference[0:-1] != "result":
                vs_message(self, "Invalid %s at line %i" % (field, item+1), "ERROR")
                return False, None
            return self.get_result(line)

        # Should never get here
        assert False
        
        return True, reference

