####### VATStuff #######
#
# movemoney_tab.py  - defines the movemoney panel
#
# Version : 0.6
# Date : 20100805
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the movemoney panel

It is run as part of the vatstuff gui interface.
"""


import wx

from vatstuff_packages.vs_utilities import *






from vatstuff_packages.vs_gui.movemoney import complex_move
from vatstuff_packages.vs_gui.movemoney import simple_move
from vatstuff_packages.vs_gui.movemoney import more_moves

from vatstuff_packages.vs_gui.gui_functions import *


class MovemoneyPanel(wx.Panel):
    """The movemoney panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)
  
        # The number of buttons to display, and the button size   
        self.button_max_number = vs_cfg["MAX_BUTTONS"]
        button_size = (200, vs_cfg["BUTTON_HEIGHT"])

        button_labels = get_button_labels()
        # a list of label objects, which have attributes: id, name, description, button
        # ordered by button number
      
        # Make a buttons list, each button consisting of the list:
        # [label, button widget id, description text widget id]
        # initially the widget id's are set at None,
        self.buttons = [ [label, None, None] for label in button_labels ]

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create a top sizer, and top button
        top_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.top_button = wx.Button(self, label="Move Money", size=button_size)    
        top_sizer.Add(self.top_button, 0, wx.ALIGN_RIGHT, 0)
        top_text=wx.StaticText(self, -1, "Press to simply move money between accounts")
        top_sizer.Add(top_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        sizer.Add(top_sizer, 0, wx.ALL | wx.ALIGN_LEFT, 30)

        help_text="""
Alternatively; choose a button below to define a complex transaction - which can then be saved
to the button, and will be available as a shortcut if you wish to do a similar transaction again"""

        self.hlp_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.hlp_text, 0, wx.LEFT | wx.ALIGN_CENTER, 10)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=self.button_max_number, cols=2, hgap=20, vgap=20)

        for button in self.buttons:                                        # button is [label, None, None]
            btn = wx.Button(self, label=button[0].name, size=button_size)    
            button[1] = btn.GetId()                                        # Store the button widget id in button[1]
            b_sizer.Add(btn, 0, wx.LEFT|wx.ALIGN_RIGHT, 10)                # put the button in the sizer

            txt=wx.StaticText(self, -1, button[0].description)             # create static text for the description
            button[2] = txt.GetId()                                        # store the text widget id in button[2]
            b_sizer.Add(txt, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)  # Add the description to the sizer

                                                                           # button is now [label, btn_id, dsc_id]
        # add the b_sizer to the main vertical sizer

        sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_LEFT, 20)

        # Create a more sizer, and more button
        more_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.more_button = wx.Button(self, label="More ..", size=button_size)    
        more_sizer.Add(self.more_button, 0, wx.LEFT|wx.ALIGN_RIGHT, 10)
        more_text=wx.StaticText(self, -1, "Press for further move money definitions.")
        more_sizer.Add(more_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        sizer.Add(more_sizer, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.ALIGN_LEFT, 20)


        # Set the size to the panel
        self.SetSizer(sizer)

        # Set the button bindings - each button is bound to the same function,
        # it is up to the function to work out which button was pressed from
        # the button widget id

        for button in self.buttons:
            self.Bind(wx.EVT_BUTTON, self.button_pressed, None, button[1])   # button[1] = button widget id

        # set the top_button and more_button bindings
        self.Bind(wx.EVT_BUTTON, self.top_button_pressed, self.top_button)
        self.Bind(wx.EVT_BUTTON, self.more_button_pressed, self.more_button)


    @button_wrap
    def top_button_pressed(self, evt):
        """The Move Money button was pressed - this does not hold
           a memory, but always brings up a blank transaction list"""
        # The parameters None, None are transaction text and transaction list
        # From this button, these values are set at None so defaults are applied
        simple_move.simple_move_dialog(self, None, None)


    @button_wrap
    def more_button_pressed(self, evt):
        """The More.. button was pressed - this creates a frame with further move money definitions"""
        more_moves.more_moves_frame()


    @button_wrap
    def button_pressed(self, evt):
        "One of the memory buttons has been pressed, identify it and call MoveDefinition"

        button_id = evt.GetId()

        # Get the button which was pressed
        button_number_pressed = -1
        for b_number, button in enumerate(self.buttons):
            if button[1] == button_id:
                button_number_pressed = b_number

        debug_log("Memory button %s chosen" % button_number_pressed)

        # if no button pressed, an error has occurred
        assert  button_number_pressed > -1 and button_number_pressed < self.button_max_number

        # Set the ComplexMove frame, with the label of the button pressed
        complex_move.complex_move_dialog(self, self.buttons[button_number_pressed][0])


    def refresh(self):
        "Refresh memory button labels / descriptions"
        button_labels = Label.get_all()

        # a list of label objects, which have attributes: id, name, description, button
        # Ordered by button

        for b_number, button in enumerate(self.buttons):
            button[0] = button_labels[b_number]
            btn = self.FindWindowById(button[1])
            btn.SetLabel(button[0].name)
            dsc = self.FindWindowById(button[2])
            dsc.SetLabel(button[0].description)


