####### VATStuff #######
#
# purchases_tab.py  - defines the purchases panel
#
# Version : 0.6
# Date : 20100723
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""This is a module in the vatstuff_packages.vs_gui package, and defines the purchases panel

It is run as part of the vatstuff gui interface.
"""

import os

import wx

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.purchases import make_payment
from vatstuff_packages.vs_gui.purchases import supplier_invoice
from vatstuff_packages.vs_gui.purchases import supplier_creditnote
from vatstuff_packages.vs_gui.purchases import view_orders
from vatstuff_packages.vs_gui.purchases import make_order
from vatstuff_packages.vs_gui.purchases import quick_purchase

from vatstuff_packages.vs_gui import view_contacts
from vatstuff_packages.vs_gui import view_bank
from vatstuff_packages.vs_gui import organization
from vatstuff_packages.vs_gui import add_item

from vatstuff_packages.vs_gui.gui_functions import *


class PurchasesPanel(wx.Panel):
    "The Purchases panel"

    def __init__(self, parent):

        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create top 'purchase order -> Recieve Invoice -> 'Make Payment' buttons in a horizontal sizer

        top_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Purchase order
        self.order_button = wx.Button(self, label="Create purchase order", size=button_size)
        self.order_button.SetToolTipString("Press to create a purchase order")   
        top_sizer.Add(self.order_button, 0, wx.ALIGN_RIGHT)

        # Arrow to right image
        arrow_right_png=os.path.join(vs_cfg["IMAGES_FOLDER"],'short_arrow_right.png')
        arrow_right_image= wx.Image(arrow_right_png, wx.BITMAP_TYPE_PNG)
        arrow_right=wx.StaticBitmap(self, -1, wx.BitmapFromImage(arrow_right_image))
        top_sizer.Add(arrow_right, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Receive invoice
        self.invoice_button = wx.Button(self, label="Receive Invoice", size=button_size)
        self.invoice_button.SetToolTipString("Receive an invoice for goods recieved")   
        top_sizer.Add(self.invoice_button, 0, wx.ALIGN_LEFT)

        # Another arrow to the right
        arrow_right2=wx.StaticBitmap(self, -1, wx.BitmapFromImage(arrow_right_image))
        top_sizer.Add(arrow_right2, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Payments
        self.make_payment_button = wx.Button(self, label="Make Payment", size=button_size)
        self.make_payment_button.SetToolTipString("Pay for goods recieved")   
        top_sizer.Add(self.make_payment_button, 0, wx.ALIGN_LEFT)

        sizer.Add(top_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 40)

        help_text="""\
Create a purchase order to send to your supplier, and on receiving an invoice choose 'Receive Invoice'
and then 'Make Payment' to input your payments. Alternatively, for a simple purchase such as a meal,
choose 'Quick Purchase' to purchase a single item together with the payment in one step."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create buttons and descriptions in a grid sizer
        button_sizer=wx.FlexGridSizer(rows=7, cols=2, hgap=20, vgap=20)

        # Quick purchase
        self.quick_purchase_button = wx.Button(self, label="Quick Purchase", size=button_size)
        self.quick_purchase_button.SetToolTipString("Input a quick purchase")   
        button_sizer.Add(self.quick_purchase_button, 0, wx.ALIGN_RIGHT)
        quick_purchase_text=wx.StaticText(self, -1, "Create order and payment for a single item")
        button_sizer.Add(quick_purchase_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # view orders
        self.view_orders_button = wx.Button(self, label="View orders", size=button_size)
        self.view_orders_button.SetToolTipString("View orders")   
        button_sizer.Add(self.view_orders_button, 0, wx.ALIGN_RIGHT)
        view_orders_text=wx.StaticText(self, -1, "View, close or cancel previous orders")
        button_sizer.Add(view_orders_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # recieve credit note
        self.credit_note_button = wx.Button(self, label="Receive Credit Note", size=button_size)
        self.credit_note_button.SetToolTipString("Accept a credit note from your supplier")   
        button_sizer.Add(self.credit_note_button, 0, wx.ALIGN_RIGHT)
        credit_note_text=wx.StaticText(self, -1, "Accept a credit note from your supplier")
        button_sizer.Add(credit_note_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)


        # Add purchase item
        self.item_button = wx.Button(self, label="Purchase items", size=button_size)
        self.item_button.SetToolTipString("Add or edit items you purchase")   
        button_sizer.Add(self.item_button, 0, wx.ALIGN_RIGHT)
        item_text=wx.StaticText(self, -1, "Create a list of frequently purchased items")
        button_sizer.Add(item_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Add a new supplier
        self.new_supplier_button = wx.Button(self, label="Add a new supplier", size=button_size)
        self.new_supplier_button.SetToolTipString("Press to add a new supplier")   
        button_sizer.Add(self.new_supplier_button, 0, wx.ALIGN_RIGHT)
        new_supplier_text=wx.StaticText(self, -1, "Press to add a new supplier")
        button_sizer.Add(new_supplier_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Edit a supplier
        self.all_suppliers = Supplier.get_all()
        supplier_list = [ supplier.name for supplier in self.all_suppliers]
        
        if supplier_list:
            self.edit_supplier_choice = wx.Choice(self, -1, size=button_size, choices=supplier_list)
            # set selection to last item on list - looks best under gnome
            self.edit_supplier_choice.SetSelection(len(supplier_list)-1)
        else:
            self.edit_supplier_choice = wx.Choice(self, -1, size=button_size, choices=[" "])
            self.edit_supplier_choice.SetSelection(0)
        self.edit_supplier_choice.SetToolTipString("Choose to edit a supplier details")

        button_sizer.Add(self.edit_supplier_choice, 0, wx.ALIGN_RIGHT)

        # Edit, contacts and bank details
        eandc_sizer = wx.BoxSizer(wx.HORIZONTAL)

        self.edit_supplier_button = wx.Button(self, label="Edit", size=wx.DefaultSize)
        self.edit_supplier_button.SetToolTipString("Press to edit the chosen supplier")   
        eandc_sizer.Add(self.edit_supplier_button, 0, wx.ALIGN_LEFT)

        self.contacts_button = wx.Button(self, label="Contacts", size=wx.DefaultSize)
        self.contacts_button.SetToolTipString("Edit and list contacts for the chosen supplier")   
        eandc_sizer.Add(self.contacts_button, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        self.bank_button = wx.Button(self, label="Bank", size=wx.DefaultSize)
        self.bank_button.SetToolTipString("Edit and show bank details for the chosen supplier")   
        eandc_sizer.Add(self.bank_button, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        button_sizer.Add(eandc_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # sort out which buttons are enabled or not
        self.refresh()

        sizer.Add(button_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 20) 

        help2_text="""\
Choose a supplier and press Edit to change or add further addresses.
Press Contacts or Bank to optionally store further supplier details."""
        
        self.bottom_text=wx.StaticText(self, -1, help2_text)
        sizer.Add(self.bottom_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(sizer)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.order_button_pressed, self.order_button)
        self.Bind(wx.EVT_BUTTON, self.invoice_button_pressed, self.invoice_button)
        self.Bind(wx.EVT_BUTTON, self.make_payment_button_pressed, self.make_payment_button)
        self.Bind(wx.EVT_BUTTON, self.quick_purchase_button_pressed, self.quick_purchase_button)
        self.Bind(wx.EVT_BUTTON, self.view_orders_button_pressed, self.view_orders_button)
        self.Bind(wx.EVT_BUTTON, self.credit_note_button_pressed, self.credit_note_button)
        self.Bind(wx.EVT_BUTTON, self.item_button_pressed, self.item_button)
        self.Bind(wx.EVT_BUTTON, self.new_supplier_button_pressed, self.new_supplier_button)
        self.Bind(wx.EVT_BUTTON, self.edit_supplier_button_pressed, self.edit_supplier_button)
        self.Bind(wx.EVT_BUTTON, self.contacts_button_pressed, self.contacts_button)
        self.Bind(wx.EVT_BUTTON, self.bank_button_pressed, self.bank_button)

    def get_supplier(self):
        "returns the supplier currently shown"
        if not self.all_suppliers: return None
        supplier_selected_list_item = self.edit_supplier_choice.GetCurrentSelection()
        assert supplier_selected_list_item != -1
        return self.all_suppliers[supplier_selected_list_item]


    # ----------------- Handler functions


    @button_wrap
    def order_button_pressed(self, event):
        "Create and show the order dialog"
        supplier = self.get_supplier()
        if not supplier: return
        make_order.order_dialog(self, supplier)


    @button_wrap
    def invoice_button_pressed(self, event):
        "List the open orders - to create an associated invoice"
        open_orders_dialog(self, receive="invoice")


    @button_wrap
    def make_payment_button_pressed(self, event):
        "List the open orders - to create an associated payment"
        open_orders_dialog(self, receive="payment")


    @button_wrap
    def quick_purchase_button_pressed(self, event):
        "Create and show the QuickPurchase dialog"
        supplier = self.get_supplier()
        if not supplier: return
        quick_purchase.quick_purchase_dialog(self, supplier)


    @button_wrap
    def view_orders_button_pressed(self, event):
        "Create the frame listing all purchase orders"
        view_orders.view_all_orders_frame()


    @button_wrap
    def credit_note_button_pressed(self, event):
        """Create and show the ReceiveCreditNoteDialog - which allows the user
           to choose to receive a creditnote with payment - or an invoice, against
           which the note is received"""
        supplier = self.get_supplier()
        if not supplier: return
        receive_creditnote_dialog(self, supplier)


    @button_wrap
    def item_button_pressed(self, event):
        "Create and show the AddItemDialog to edit items frequently purchased"
        add_item.add_item_dialog(self, "purchase")


    @button_wrap
    def new_supplier_button_pressed(self, event):
        "Create and show the NewOrganizationDialog to add a new supplier"
        organization.new_organization_dialog(self, "supplier")


    @button_wrap
    def edit_supplier_button_pressed(self, event):
        "Create and show the EditOrganizationDialog to edit an existing supplier"
        supplier = self.get_supplier()
        if not supplier: return
        organization.edit_organization_dialog(self, supplier)


    @button_wrap
    def contacts_button_pressed(self, event):
        "Create and show the ViewContactsDialog to edit contacts for an existing supplier"
        supplier = self.get_supplier()
        if not supplier: return
        view_contacts.view_contacts_dialog(self, supplier)


    @button_wrap
    def bank_button_pressed(self, event):
        "Create and show the BankDetailsDialog to edit bank info for an existing supplier"
        supplier = self.get_supplier()
        if not supplier: return
        view_bank.bank_details_dialog(self, supplier)


    def refresh_list(self, select_supplier=None):
        "Refresh list and set selection to select_supplier"
        all_suppliers = Supplier.get_all()
        if not all_suppliers:
            # empty list, no supplier has been created yet
            self.edit_supplier_choice.Enable(False)
            self.edit_supplier_button.Enable(False)
            self.contacts_button.Enable(False)
            self.bank_button.Enable(False)
            # refresh supplier selection
            self.edit_supplier_choice.Clear()
            self.edit_supplier_choice.AppendItems(" ")
            self.edit_supplier_choice.SetSelection(0)
            self.all_suppliers = []
            return

        if select_supplier is None:
            # Get the current supplier shown
            select_supplier = self.get_supplier()

        # A new supplier may have been added
        self.edit_supplier_choice.Clear()
        self.all_suppliers = all_suppliers

        # Set up new list
        supplier_list = [supplier.name for supplier in self.all_suppliers]
        self.edit_supplier_choice.AppendItems(supplier_list)

        for index, supplier in enumerate(self.all_suppliers):
            if select_supplier == supplier:
                self.edit_supplier_choice.SetSelection(index)
                break
        else:
            # Choose last
            self.edit_supplier_choice.SetSelection(len(supplier_list)-1)

        # enable buttons
        self.edit_supplier_choice.Enable(True)
        self.edit_supplier_button.Enable(True)
        self.contacts_button.Enable(True)
        self.bank_button.Enable(True)


    def refresh(self):
        self.order_button.Enable(False)
        self.invoice_button.Enable(False)
        self.make_payment_button.Enable(False)
        self.quick_purchase_button.Enable(False)
        self.credit_note_button.Enable(False)
        self.view_orders_button.Enable(False)
        # refresh supplier list and supplier buttons
        self.refresh_list()
        if self.all_suppliers:
            # Check registered address is complete
            company = Company()
            # Get the current VAT Scheme
            vat_scheme = VATScheme()
            if ( company and
                 company.registered_address and
                 company.name and
                   (vat_scheme.method == "NOT_REGISTERED" or company.vat_regnumber) ):
                self.order_button.Enable(True)
                self.quick_purchase_button.Enable(True)
                self.view_orders_button.Enable(True)
                self.credit_note_button.Enable(True)
                if count_openorders() > 0:
                    # count_openorders() is defined in the odd_functions module
                    self.invoice_button.Enable(True)
                    self.make_payment_button.Enable(True)


# ---------------- function to create the dialog to list open orders and then to
# ---------------- call the subsequent invoice, payment or creditnote dialog

def open_orders_dialog(parent, receive):
    """Creates an OpenOrdersDialog, and then the next dialog depending on the receive
       receive = 'invoice' to generate a supplier invoice received form
       receive = 'payment' to generate a payment form
       receive = 'creditnote' to generate a supplier creditnote received form"""

    open_orders_dialog = OpenOrdersDialog(parent, receive)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if open_orders_dialog.ShowModal() != wx.ID_OK:
        open_orders_dialog.Destroy()
        return

    # An order has been chosen
    order = open_orders_dialog.order
    open_orders_dialog.Destroy()
    
    busy_cursor_on()
    wx.Yield()
    debug_log("order picked id: %s" % order.id)

    # we now have to call the required dialog
    if receive == 'invoice':
        supplier_invoice.supplier_invoice_dialog(parent, order)
        return

    if receive == 'payment':
        make_payment.make_payment_dialog(parent, order)
        return

    if receive == 'creditnote':
        supplier_creditnote.supplier_creditnote_dialog(parent, order)

# ----------------- Dialog to list open orders

class OpenOrdersDialog(vsDialog):
    "The open orders dialog which creates a list of open orders"

    def __init__(self, parent, receive):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Open Purchase Orders")

        # This will be the order chosen
        self.order = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""\
This table shows orders currently open, double click on the
relevant order to input the invoice details received."""

        if receive == 'payment':
            help_text="""\
This table shows orders currently open, and with
invoices received against them. Double click
on the relevant order to input a payment."""

        if receive == "creditnote":
            help_text="""\
Credit notes are normally issued by a supplier when they cannot meet the full order,
and are instructing you not to pay the full invoice amount. Double click the order,
and you will be able to input the cost and VAT given in the credit note."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the openorders table

        self.openorders_table = OpenOrdersTable(self, receive)
        sizer.Add(self.openorders_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        sizer.Add(self.close_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)

    def close_button_pressed(self, event):
        "Close button pressed"
        self.EndModal(wx.ID_CANCEL)

    def close_with_order(self, order):
        """After an order is chosen with a double click, this sets
           self.order to the order, then ends with an OK"""
        self.order = order
        self.EndModal(wx.ID_OK)


class OpenOrdersTable(wx.grid.Grid):

    def __init__(self, parent, receive):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.receive = receive

        # These get orders functions are in vatstuff_packages.vs_database.odd_functions
        if self.receive == 'invoice':
            # get all openorders
            self.all_openorders = get_openorders()
        else:
            # otherwise just get those which have been invoiced
            self.all_openorders = get_open_and_invoiced_orders()

        num_rows = len(self.all_openorders)
        num_cols = 4

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Total")
        self.SetColLabelValue(3, "Paid")

        # Set the colums showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.SetColAttr(2, attr)
        self.SetColAttr(3, attr)

        for key, data in enumerate(self.all_openorders):
            self.SetRowLabelValue(key, str(data.number))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.description)
            self.SetCellValue(key, 2, to_currency_string(data.total))
            self.SetCellValue(key, 3, to_currency_string(data.paid))

        self.SetColSize(0, 100)
        self.SetColSize(1, 350)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.choose_order, self)


    def choose_order(self, event):
        "Order has been double - clicked, set self.order to the order chosen"
        try:
            # get the order chosen
            order = self.all_openorders[event.GetRow()]

            if self.receive == 'payment':
                # Check an invoice is available to be paid for this order
                if not order.get_openinvoices():
                    vs_message(self, """\
Cannot input any payment as there is no
invoice amount to be paid on this order.""", "No amount owed on order")
                    return

            if self.receive == 'creditnote':
                # Check an invoice amount is owed on this order
                if order.invoice_cost + order.invoice_vat <= order.paid:
                    vs_message(self, """\
Cannot input a credit note as there is
no invoiced amount owed on this order.""", "No amount owed on order")
                    return
            # Call the parent to set self.order with the order, and to close
            self.parent.close_with_order(order)
        except Exception, e:
            error_message("Failure in choosing customer", e)


########  Choose which method of supplier creditnote payment is used - either a payment comes with the
########  creditnote, or something is knocked off an existing order



def receive_creditnote_dialog(parent, supplier):
    """Creates a ReceiveCreditNoteDialog"""
    receive_creditnote_dialog = ReceiveCreditNoteChoiceDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if receive_creditnote_dialog.ShowModal() != wx.ID_OK:
        receive_creditnote_dialog.Destroy()
        return

    # A choice has been made, get which type of note, and open the next dialog
    creditnote_choice = receive_creditnote_dialog.creditnote_choice
    receive_creditnote_dialog.Destroy()

    busy_cursor_on()
    wx.Yield()

    if creditnote_choice:
        # note to be assigned against an invoice
        open_orders_dialog(parent, receive="creditnote")
    else:
        # note received with payment, no order to be chosen
        supplier_creditnote.supplier_creditnote_with_payment_dialog(parent, supplier)


class ReceiveCreditNoteChoiceDialog(vsDialog):
    """Called when a creditnote has been received from a supplier, gives the
       user the option to choose which type of creditnote - either one offsetting
       a received invoice, or one with a payment.  Sets the creditnote_choice
       attribute accordingly, if True - with invoice, if False with dialog"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Supplier Credit Note received")
        
        # False, if note to be received with a payment
        self.creditnote_choice = False
        # or set to True, if openorders is needed to choose an invoice

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        panel_sizer=wx.BoxSizer(wx.VERTICAL)
        sizer=wx.GridBagSizer(hgap=20, vgap=20)

        text1 = """\
Is this credit note to be used to reduce the payment of an existing
received invoice? The invoice must already have been received and
input against a purchase order, and must not yet have been paid."""
        sizer.Add(wx.StaticText(self, -1, text1), pos=(0,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # List Open Orders
        self.open_orders_button = wx.Button(self, label="List open orders", size=button_size)
        self.open_orders_button.SetToolTipString("""\
Choose the order, and payments can
be reduced by the credit note amount""")   
        sizer.Add(self.open_orders_button, pos=(1,0), flag=wx.ALIGN_RIGHT)

        button1_text = "Choose the purchase order"
        sizer.Add(wx.StaticText(self, -1, button1_text), pos=(1,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # are there any orders, with invoices received? 
        all_openorders = get_open_and_invoiced_orders()
        if not all_openorders:
            self.open_orders_button.Enable(False)

        text2 = """\
OR; is this credit note accompanied by a money payment from your
supplier, which must now be input into your bank account?"""
        sizer.Add(wx.StaticText(self, -1, text2), pos=(2,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # Input payment
        self.input_payment_button = wx.Button(self, label="Input payment", size=button_size)
        self.input_payment_button.SetToolTipString("Input payment received from your supplier")   
        sizer.Add(self.input_payment_button, pos=(3,0), flag=wx.ALIGN_RIGHT)

        button2_text = "Input payment received from your supplier"
        sizer.Add(wx.StaticText(self, -1, button2_text), pos=(3,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        panel_sizer.Add(sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel")
        panel_sizer.Add(self.cancel_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 40)
        
        # Set the size to the panel
        self.SetSizer(panel_sizer)
        panel_sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.open_orders_button_pressed, self.open_orders_button)
        self.Bind(wx.EVT_BUTTON, self.input_payment_button_pressed, self.input_payment_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    @logmethod
    def input_payment_button_pressed(self, event):
        "Show the creditnote with payment frame"
        self.creditnote_choice = False
        self.EndModal(wx.ID_OK)

    @logmethod
    def open_orders_button_pressed(self, event):
        "List the open orders - to input an associated credit note"
        self.creditnote_choice = True
        self.EndModal(wx.ID_OK)

    def cancel_button_pressed(self, event):
        "Close parent"
        self.EndModal(wx.ID_CANCEL)










