####### VATStuff #######
#
# view_makepayment.py  - creates a dialog to show a make payment detail 
#
# Version : 0.4
# Date : 20100420
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a dialog to show a make payment detail"""

import datetime
import decimal

import wx
import wx.grid






from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.makepayment_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing

class ShowMakePaymentDialog(vsDialog):
    """Called to show the makepayment"""

    def __init__(self, parent, payment):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Payment Sent")
        
        # Take a note of the variables
        self.payment = payment

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # payment memo

        memo_text = "Payment memo : %s" % self.payment.description

        sizer.Add(wx.StaticText(self, -1, memo_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)

        if payment.supplier_address:
            # payment address on the left of a horizontal sizer, payment details on the right
            details_sizer=wx.BoxSizer(wx.HORIZONTAL)
            supplier_text = "Supplier: %s\n%s" % (payment.supplier.name,
                                                      payment.supplier_address.lines_string)
            details_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)
        else:
            # details sizer does not exist, just create the pay sizer with the supplier field added
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=10)
            # supplier
            pay_sizer.Add(wx.StaticText(self, -1, "Supplier :"), 0, wx.ALIGN_RIGHT)
            supplier_text = payment.supplier.name
            pay_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.ALIGN_LEFT)

        # order number
        pay_sizer.Add(wx.StaticText(self, -1, "Purchase order :"), 0, wx.ALIGN_RIGHT)
        po_number_text = str(payment.purchaseorder.number)
        pay_sizer.Add(wx.StaticText(self, -1, po_number_text), 0, wx.ALIGN_LEFT)

        # date
        pay_sizer.Add(wx.StaticText(self, -1, "Payment date :"), 0, wx.ALIGN_RIGHT)
        date_text = payment.datestamp.strftime("%d %b %Y")
        pay_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # payment amount
        pay_sizer.Add(wx.StaticText(self, -1, "Payment amount :"), 0, wx.ALIGN_RIGHT)
        amount_text = to_currency_string(payment.amount)
        pay_sizer.Add(wx.StaticText(self, -1, amount_text), 0, wx.ALIGN_LEFT)

        # payment account
        pay_sizer.Add(wx.StaticText(self, -1, "Paid from account :"), 0, wx.ALIGN_RIGHT)
        account_text = payment.account.fullname
        pay_sizer.Add(wx.StaticText(self, -1, account_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        pay_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(payment.transaction.id)
        pay_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        if payment.supplier_address:
            # details sizer contains address on the left, pay on the right
            details_sizer.Add(pay_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 10)
            sizer.Add(details_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        else:
            # details sizer does not exist, just add the pay sizer
            sizer.Add(pay_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        sizer.Add(wx.StaticText(self, -1, "Payment applied to supplier invoices :"), 0, wx.TOP | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the invoices table

        self.invoices_table = InvoicesTable(self, payment)
        sizer.Add(self.invoices_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    @logmethod
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        try:
            # get last report pdf width and height
            width, height = vs_cfg["LAST_REPORT_PDF_SIZE"]
            page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, width, height)
            if page_setup_dialog.ShowModal() == wx.ID_OK:
                vs_cfg["LAST_REPORT_PDF_SIZE"] = (page_setup_dialog.width,
                                                  page_setup_dialog.height)          
            page_setup_dialog.Destroy()
        except Exception, e:
            exception_log(e)
            error_message("Unable to create the PageSetup Dialog")


    @logmethod
    def pdf_button_pressed(self, event):
        "Create a pdf file"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        try:
            # default filename
            filename= "payment_sent_%s_%s.pdf" % (self.payment.datestamp.isoformat(), self.payment.transaction.id)

            # Create the save file dialog.
            status, filepath = save_file_as(self, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)

            if status:
                if vatstuff_packages.vs_pdf.makepayment_form.create_pdf(filepath, self.payment):
                    # And show the file in the default viewer
                    if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                        vs_message(self, "pdf file created", "Done")
                else:
                    vs_message(self, "Unable to create the pdf file", "ERROR")
        except Exception, e:
            exception_log(e)
            error_message("Failure in creating the pdf file")


class InvoicesTable(wx.grid.Grid):

    def __init__(self, parent, payment):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        i_rows = []

        inv_list = payment.get_invoice_list()
        # inv_list being a list of lists - { "INVOICE":supplierinvoice,
        #                                    "COST":cost,
        #                                    "VAT":vat }
        #         one dictionary for each invoice this payment covers

        for item in inv_list:
            i_rows.append((str(item["INVOICE"].transaction.id),
                           item["INVOICE"].number,
                           to_currency_string(item["COST"]),
                           to_currency_string(item["VAT"])
                           ))

        num_rows = len(i_rows)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Supplier invoice number")
        self.SetColLabelValue(1, "Net Paid")
        self.SetColLabelValue(2, "VAT Paid")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(1, attr_right)
        self.SetColAttr(2, attr_right)

        self.SetColSize(0, 300)
        self.SetColSize(1, 100)
        self.SetColSize(2, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(i_rows):
            # Set row label as the invoice number
            self.SetRowLabelValue(row, item[0])
            # Description
            self.SetCellValue(row, 0, item[1])
            # Net
            self.SetCellValue(row, 1, item[2])
            # VAT
            self.SetCellValue(row, 2, item[3])



