####### VATStuff #######
#
# customer_reports.py  - dialog to choose a customer and report
#
# Version : 0.6
# Date : 20100818
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Dialog to choose a customer and report"""

import wx
import datetime





from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *


# The ChooseCustomerReportDialog is used to pick a customer,
# a report, and to set the two dates between which the report
# will be taken.  It is normally used by calling the function
# choose_customer_report

def choose_customer_report(parent):
    """Creates a dialog, returns (True, customer, start_date, end_date, report_name)
       on confirmation or (False, None, None, None, None) otherwise"""
    choose_customer_report_dialog = ChooseCustomerReportDialog(parent)

    busy_cursor_off()

    if choose_customer_report_dialog.ShowModal() != wx.ID_OK:
        choose_customer_report_dialog.Destroy()
        return False, None, None, None, None


    result = (True,
                choose_customer_report_dialog.customer,
                choose_customer_report_dialog.from_date,
                choose_customer_report_dialog.to_date,
                choose_customer_report_dialog.report_name)

    choose_customer_report_dialog.Destroy()
    return result



class ChooseCustomerReportDialog(vsDialog):
    """Called to chose a customer and report to generate"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Customer reports")

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        help_text = wx.StaticText(self, -1, "Choose a customer and the report to generate:")
        sizer.Add(help_text, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=20)


        # Choose a customer
        customer_choice_text=wx.StaticText(self, -1, "Choose a customer:")
        field_sizer.Add(customer_choice_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_customers = Customer.get_all()
        customer_list = [ customer.name for customer in self.all_customers ]

        # the customer selected on the sales panel
        customer_selected = top_frame().get_customer()
        if customer_selected and self.all_customers:
            customer_index = self.all_customers.index(customer_selected)
        else:
            customer_index = 0

        if customer_list:
            self.customer_choice_field = wx.Choice(self, -1, size=button_size, choices=customer_list)
            self.customer_choice_field.SetSelection(customer_index)
            self.customer = self.all_customers[customer_index]
        else:
            self.customer_choice_field = wx.Choice(self, -1, size=button_size, choices=[" "])
            self.customer_choice_field.SetSelection(0)
            self.customer = None
        self.customer_choice_field.SetToolTipString("Choose a customer") 

        field_sizer.Add(self.customer_choice_field, 0, wx.ALIGN_LEFT)


        # Get initial dates
        self.to_date = datetime.date.today()

        # set self.from_date as the beginning of this month,
        # or set self.from_date as the beginning of last month if the to_date is less than ten days into the month
        delta = datetime.timedelta(days=10)
        self.from_date = self.to_date - delta
        self.from_date = self.from_date.replace(day=1)

        wx_fromdate = date_to_wxdate(self.from_date)
        wx_todate = date_to_wxdate(self.to_date)

        # The from date
        field_sizer.Add(wx.StaticText(self, -1, "Generate report for the period From:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.from_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_fromdate, 
                                                 size=button_size,
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.from_date_field.SetToolTipString("The from date")
        field_sizer.Add(self.from_date_field, 0, wx.ALIGN_LEFT)

        # The to date
        field_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.to_date_field = wx.DatePickerCtrl(self, -1,
                                               wx_todate, 
                                               size=button_size,
                                               style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.to_date_field.SetToolTipString("The to date")
        field_sizer.Add(self.to_date_field, 0, wx.ALIGN_LEFT)

        # Invoices sent
        invoices_text=wx.StaticText(self, -1, "Invoices sent to this customer:")
        field_sizer.Add(invoices_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.invoices_button = wx.Button(self, label="Invoices Report", size=button_size)
        self.invoices_button.SetToolTipString("List invoices sent to this customer between the two dates")  
        field_sizer.Add(self.invoices_button, 0, wx.ALIGN_LEFT)
 
        # credit notes sent
        credit_notes_text=wx.StaticText(self, -1, "Credit notes sent to this customer:")
        field_sizer.Add(credit_notes_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.credit_notes_button = wx.Button(self, label="Credit Notes Report", size=button_size)
        self.credit_notes_button.SetToolTipString("List credit notes sent to this customer between the two dates")  
        field_sizer.Add(self.credit_notes_button, 0, wx.ALIGN_LEFT)

        # payments received
        payments_text=wx.StaticText(self, -1, "Payments received from this customer:")
        field_sizer.Add(payments_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.payments_button = wx.Button(self, label="Payments Report", size=button_size)
        self.payments_button.SetToolTipString("List payments received from this customer between the two dates")  
        field_sizer.Add(self.payments_button, 0, wx.ALIGN_LEFT)


        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        if not self.customer:
            self.customer_choice_field.Enable(False)
            self.from_date_field.Enable(False)
            self.to_date_field.Enable(False)
            self.invoices_button.Enable(False)
            self.credit_notes_button.Enable(False)
            self.payments_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to cancel.")
        sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_BUTTON, self.invoices_button_pressed, self.invoices_button)
        self.Bind(wx.EVT_BUTTON, self.credit_notes_button_pressed, self.credit_notes_button)
        self.Bind(wx.EVT_BUTTON, self.payments_button_pressed, self.payments_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    # ----------------- Handler functions

    def invoices_button_pressed(self, event):
        """Invoices report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "customer_invoices"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def credit_notes_button_pressed(self, event):
        """Credit_notes report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "customer_creditnotes"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def payments_button_pressed(self, event):
        """Payments report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "customer_payments"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def close_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)

    def get_selections(self):
        "Get the customer and dates"
        customer_selection = self.customer_choice_field.GetSelection()
        self.customer = self.all_customers[customer_selection]

        # Convert the dates input to python date objects
        # function wxdate_to_date is in module vatstuff_packages.vs_utilities.wx_utilities
        try:
            from_date = wxdate_to_date(self.from_date_field.GetValue())
        except Exception:
            vs_message(self, "Invalid From date", "ERROR")
            return False
        try:
            to_date = wxdate_to_date(self.to_date_field.GetValue())
        except Exception:
            vs_message(self, "Invalid To date", "ERROR")
            return False
        if to_date < from_date:
            vs_message(self, "The To date is earlier than the From date!", "ERROR")
            return False
        self.from_date = from_date
        self.to_date = to_date
        return True



