####### VATStuff #######
#
# list_accounts.py  - dialog to list accounts, by folder and type
#
# Version : 0.6
# Date : 20100524
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Dialog to list accounts, by folder and type"""

import wx





from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *


# The ChooseAccountsDialog is used to pick accounts in a Folder, or by Type 
# It is normally used by calling the function choose_accounts

def choose_accounts_dialog(parent):
    """Creates a dialog, returns (True, account_folder, account_type) on confirmation
       or (False, None, None) otherwise, where account_folder is None, if any folder,
       or is a folder if a specific one is chosen, and similarly
       account_type is None, if any type, or is a type if a specific one is chosen"""
    choose_accounts_dialog = ChooseAccountsDialog(parent)

    busy_cursor_off()

    if choose_accounts_dialog.ShowModal() != wx.ID_OK:
        choose_accounts_dialog.Destroy()
        return False, None, None

    account_folder = choose_accounts_dialog.account_folder
    account_type = choose_accounts_dialog.account_type
    choose_accounts_dialog.Destroy()
    return True, account_folder, account_type


class ChooseAccountsDialog(vsDialog):
    """Called to chose folder and type of accounts to list"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Choose accounts to list")

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        help_text = wx.StaticText(self, -1, "List accounts within a folder or of a type:")
        sizer.Add(help_text, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=40)

        # Get the list of all folders, to be used in the folders Choice option
        self.all_folders=Folder.get_all()
        # Create a folder list of folder names
        self.folder_list = [ f.name for f in self.all_folders ]
        self.all_folders.insert(0, None)
        self.folder_list.insert(0,"")

        folder_check_list = ['Any Folder', 'Choose a specific folder']

        self.folder_check_field = wx.RadioBox(self, -1, "",
                                            wx.DefaultPosition, wx.DefaultSize,
                                            folder_check_list, 0, wx.NO_BORDER | wx.RA_VERTICAL
                                            )

        self.folder_check_field.SetSelection(0)
        
        self.folder_check_field.SetItemToolTip(0,"Set to list accounts in any folder")
        self.folder_check_field.SetItemToolTip(1,"Set to list accounts from a specific folder")

        field_sizer.Add(self.folder_check_field, 0, wx.ALIGN_LEFT)

        self.folder_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.folder_list)
        self.folder_field.SetToolTipString("List accounts under this folder")
        self.folder_field.SetSelection(0)
        self.folder_field.Enable(False)
        field_sizer.Add(self.folder_field, 0, wx.ALIGN_LEFT | wx.ALIGN_BOTTOM)

        # Get the list of all account types, to be used in the account types Choice option
        self.all_account_types = AccountType.get_all()
        # Create a types list of type names
        self.acc_type_list = [acc_type.name for acc_type in self.all_account_types ]
        self.all_account_types.insert(0, None)
        self.acc_type_list.insert(0,"")

        type_check_list = ['Any Type', 'List accounts of a specific type']

        self.type_check_field = wx.RadioBox(self, -1, "",
                                            wx.DefaultPosition, wx.DefaultSize,
                                            type_check_list, 0, wx.NO_BORDER | wx.RA_VERTICAL
                                            )

        self.type_check_field.SetSelection(0)
        
        self.type_check_field.SetItemToolTip(0,"Set to list accounts of any type")
        self.type_check_field.SetItemToolTip(1,"Set to list accounts of a specific type")

        field_sizer.Add(self.type_check_field, 0, wx.ALIGN_LEFT)

        self.acc_type_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.acc_type_list)
        self.acc_type_field.SetToolTipString("List accounts with this type")
        self.acc_type_field.SetSelection(0)
        self.acc_type_field.Enable(False)
        field_sizer.Add(self.acc_type_field, 0, wx.ALIGN_LEFT | wx.ALIGN_BOTTOM)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to generate the report.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_RADIOBOX, self.folder_check_field_pressed, self.folder_check_field)
        self.Bind(wx.EVT_RADIOBOX, self.type_check_field_pressed, self.type_check_field)


    # ----------------- Handler functions


    def folder_check_field_pressed(self, event):
        "The folder radio choice has been changed"
        try:
            # If the choice is any, disable the self.folder_field
            # and set its selection to zero
            if self.folder_check_field.GetSelection():
                self.folder_field.Enable(True)
            else:
                self.folder_field.Enable(False)
                self.folder_field.SetSelection(0)
        except Exception, e:
            error_message("Error when checking the folder choice", e)


    def type_check_field_pressed(self, event):
        "The account type radio choice has been changed"
        try:
            # If the choice is any, disable the self.acc_type_field
            # and set its selection to zero
            if self.type_check_field.GetSelection():
                self.acc_type_field.Enable(True)
            else:
                self.acc_type_field.Enable(False)
                self.acc_type_field.SetSelection(0)
        except Exception, e:
            error_message("Error when checking the type choice", e)


    def apply_button_pressed(self, event):
        """ Apply button - get folder and type fields"""
        try:
            folder_selection = self.folder_field.GetSelection()
            self.account_folder = self.all_folders[folder_selection]
            type_selection = self.acc_type_field.GetSelection()
            self.account_type = self.all_account_types[type_selection]
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error when reading the selections", e)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)

