####### VATStuff #######
#
# reports_tab.py  - defines the reports panel
#
# Version : 0.6
# Date : 20100719
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" This is a module in the vatstuff_packages.vs_gui package, and defines the reports panel

It is run as part of the vatstuff gui interface.
"""

import datetime

import wx

from vatstuff_packages.vs_database import report_functions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.reports import view_transactions
from vatstuff_packages.vs_gui.reports import search_contacts
from vatstuff_packages.vs_gui.reports import make_report
from vatstuff_packages.vs_gui.reports import list_accounts
from vatstuff_packages.vs_gui.reports import customer_reports
from vatstuff_packages.vs_gui.reports import supplier_reports
from vatstuff_packages.vs_gui.reports import search


class ReportsPanel(wx.Panel):
    """The Reports panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])

        sizer=wx.BoxSizer(wx.VERTICAL)
             
        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=30, vgap=20)

        # View Transactions
        self.transactions_button = wx.Button(self, label="Transactions", size=button_size)
        self.transactions_button.SetToolTipString("View transactions")  
        b_sizer.Add(self.transactions_button, 0, wx.ALIGN_RIGHT) 
 
        transactions_text=wx.StaticText(self, -1, "View all transactions")
        b_sizer.Add(transactions_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Accounts
        self.accounts_button = wx.Button(self, label="Accounts", size=button_size)
        self.accounts_button.SetToolTipString("View list of accounts")  
        b_sizer.Add(self.accounts_button, 0, wx.ALIGN_RIGHT) 
 
        accounts_text=wx.StaticText(self, -1, "Accounts report, by folder or type")
        b_sizer.Add(accounts_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Payments received
        self.payments_button = wx.Button(self, label="Payments received", size=button_size)
        self.payments_button.SetToolTipString("View list of payments received")  
        b_sizer.Add(self.payments_button, 0, wx.ALIGN_RIGHT) 
 
        payments_text=wx.StaticText(self, -1, "Payments received report")
        b_sizer.Add(payments_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Payments to suppliers
        self.makepayments_button = wx.Button(self, label="Payments made", size=button_size)
        self.makepayments_button.SetToolTipString("View list of payments made")  
        b_sizer.Add(self.makepayments_button, 0, wx.ALIGN_RIGHT) 
 
        makepayments_text=wx.StaticText(self, -1, "Payments made report")
        b_sizer.Add(makepayments_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Customers reports
        self.customers_button = wx.Button(self, label="Customer reports", size=button_size)
        self.customers_button.SetToolTipString("List sale details for a given customer")  
        b_sizer.Add(self.customers_button, 0, wx.ALIGN_RIGHT) 

        customers_text=wx.StaticText(self, -1, "Invoices and payments per customer")
        b_sizer.Add(customers_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # disable button if no customers have been created yet
        if not Customer.get_all(): self.customers_button.Enable(False)

        # Supplier reports
        self.suppliers_button = wx.Button(self, label="Supplier reports", size=button_size)
        self.suppliers_button.SetToolTipString("List purchase details for a given supplier")  
        b_sizer.Add(self.suppliers_button, 0, wx.ALIGN_RIGHT) 
 
        suppliers_text=wx.StaticText(self, -1, "Orders and payments per supplier")
        b_sizer.Add(suppliers_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # disable button if no suppliers have been created yet
        if not Supplier.get_all(): self.suppliers_button.Enable(False)

        sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # The search option and button

        search_text=wx.StaticText(self, -1, "To search for an item; choose an option and value, then press the Search button")
        sizer.Add(search_text, 0, wx.BOTTOM | wx.ALIGN_CENTER, 10)

        search_sizer = wx.BoxSizer(wx.HORIZONTAL)

        search_sizer.Add(wx.StaticText(self, -1, "Find :"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)

        self.search_option_list = [ "Contacts, where contact name",
                                    "Contacts, where organisation name",
                                    "Transaction, where transaction number",
                                    "Invoices sent, where customer order number",
                                    "Invoice sent, where invoice number",
                                    "Payments received, where customer reference",
                                    "Credit note sent, where credit note number",
                                    "Project, where project reference",
                                    "Purchase order sent, where order number",
                                    "Purchase order, where supplier invoice number",
                                    "Purchase order, where supplier reference",
                                    "Supplier invoice received, where invoice number",
                                    "Credit note received, where credit note number" ]

        self.search_option_choice = wx.Choice(self, -1, size=(400, vs_cfg["BUTTON_HEIGHT"]), choices=self.search_option_list)
        self.search_option_choice.SetSelection(0)
        self.search_option_choice.SetToolTipString("Choose a search option") 
        search_sizer.Add(self.search_option_choice, 0, wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)

        search_sizer.Add(wx.StaticText(self, -1, "="), 0, wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)

        self.search_value_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.search_value_field.SetToolTipString("Choose a search value")
        self.search_value_field.SetMaxLength(60)
        search_sizer.Add(self.search_value_field, 0, wx.ALIGN_CENTER_VERTICAL)

        sizer.Add(search_sizer, 0, wx.BOTTOM | wx.ALIGN_CENTER, 10)

        self.search_button = wx.Button(self, label="Search", size=button_size)
        self.search_button.SetToolTipString("Click to search for the given option")
        sizer.Add(self.search_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 30)


        
        # Set the size to the panel
        self.SetSizer(sizer)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_BUTTON, self.transactions_button_pressed, self.transactions_button)
        self.Bind(wx.EVT_BUTTON, self.accounts_button_pressed, self.accounts_button)
        self.Bind(wx.EVT_BUTTON, self.payments_button_pressed, self.payments_button)
        self.Bind(wx.EVT_BUTTON, self.makepayments_button_pressed, self.makepayments_button)
        self.Bind(wx.EVT_BUTTON, self.customers_button_pressed, self.customers_button)
        self.Bind(wx.EVT_BUTTON, self.suppliers_button_pressed, self.suppliers_button)
        self.Bind(wx.EVT_BUTTON, self.search_button_pressed, self.search_button)


    # ----------------- Handler functions

    @button_wrap
    def transactions_button_pressed(self, event):
        "Create the frame listing all transactions"
        view_transactions.view_all_transactions_frame()


    @button_wrap
    def payments_button_pressed(self, event):
        """Get the two dates between which the report is generated and show payments received"""
        help_text = """\
A report will be generated showing
payments received between the two dates:"""
        status, from_date, to_date = two_dates(self, "Payments Received", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "received_payments%s" % to_date.isoformat()
        make_report.make_report_and_frame("payments",
                                          "Payments received",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def makepayments_button_pressed(self, event):
        """Get the two dates between which the report is generated and show payments made"""
        help_text = """\
A report will be generated showing
payments made between the two dates:"""
        status, from_date, to_date = two_dates(self, "Payments Made", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "payments_made%s" % to_date.isoformat()
        make_report.make_report_and_frame("makepayments",
                                          "Payments made",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def accounts_button_pressed(self, event):
        """List the accounts, by folder or by type"""
        status, account_folder, account_type = list_accounts.choose_accounts_dialog(self)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()
        filename = "accounts_%s" % datetime.date.today().isoformat()
        make_report.make_report_and_frame("accounts",
                                          "Accounts List",
                                          filename,
                                          None,
                                          None,
                                          account_folder = account_folder,
                                          account_type = account_type)


    @button_wrap
    def customers_button_pressed(self, event):
        """Choose a customer and report"""
        status, customer_chosen, from_date, to_date, report_name = customer_reports.choose_customer_report(self)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()
        filename = "%s%s" % (report_name, datetime.date.today().isoformat())
        make_report.make_report_and_frame(report_name,
                                          "Customer Report",
                                          filename,
                                          from_date,
                                          to_date,
                                          customer = customer_chosen)


    @button_wrap
    def suppliers_button_pressed(self, event):
        """Choose a supplier and report"""
        status, supplier_chosen, from_date, to_date, report_name = supplier_reports.choose_supplier_report(self)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()
        filename = "%s%s" % (report_name, datetime.date.today().isoformat())
        make_report.make_report_and_frame(report_name,
                                          "Supplier Report",
                                          filename,
                                          from_date,
                                          to_date,
                                          supplier = supplier_chosen)


    @button_wrap
    def search_button_pressed(self, event):
        """Choose a search option, with a value, and search for the option with that value"""
        search_option_selected_list_item = self.search_option_choice.GetCurrentSelection()
        assert search_option_selected_list_item != -1
        search_option_selected = self.search_option_list[search_option_selected_list_item]

        search_value = self.search_value_field.GetValue().strip()

        # Call implement_search in the search module to implement the
        # search, and to show the results frame
        search.implement_search(self, search_option_selected, search_value)

    def refresh(self):
        "Enables the customer/supplier reports buttons if a customer or supplier exists"
        if not self.suppliers_button.IsEnabled():
            if Supplier.get_all(): self.suppliers_button.Enable(True)
        if not self.customers_button.IsEnabled():
            if Customer.get_all(): self.customers_button.Enable(True)

