####### VATStuff #######
#
# search.py  - search for a given option and value
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,20010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a Frame and table to search for
   a given option and value, and display the results.
"""

import wx
import wx.grid

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *






from vatstuff_packages.vs_gui.sales.view_invoices import show_invoice_frame, show_customer_payment_dialog
from vatstuff_packages.vs_gui.sales.view_creditnotes import show_creditnote_frame
from vatstuff_packages.vs_gui.reports import view_transactions, search_contacts

from vatstuff_packages.vs_gui.purchases.view_orders import (show_order_frame,
                                          show_supplier_creditnote_dialog,
                                          show_supplier_invoice_and_order_dialog,
                                          view_project_frame)


def implement_search(parent, search_option_selected, search_value):
    "Called from reports_tab to implement the search"

    # search contacts can have an empty value
    if search_option_selected == "Contacts, where contact name":
        if vs_cfg["SEARCH_CONTACTS_FRAME"]:
            vs_cfg["SEARCH_CONTACTS_FRAME"].Raise()
            vs_cfg["SEARCH_CONTACTS_FRAME"].refresh(organization_text="",
                                            name_text=search_value,
                                            use_current=False)
            return
        search_contacts_frame = search_contacts.SearchContactsFrame(parent)
        search_contacts_frame.Show()
        search_contacts_frame.refresh(organization_text="",
                                            name_text=search_value,
                                            use_current=False)
        return

    if search_option_selected == "Contacts, where organisation name":
        if vs_cfg["SEARCH_CONTACTS_FRAME"]:
            vs_cfg["SEARCH_CONTACTS_FRAME"].Raise()
            vs_cfg["SEARCH_CONTACTS_FRAME"].refresh(organization_text=search_value,
                                            name_text="",
                                            use_current=False)
            return
        search_contacts_frame = search_contacts.SearchContactsFrame(parent)
        search_contacts_frame.Show()
        search_contacts_frame.refresh(organization_text=search_value,
                                            name_text="",
                                            use_current=False)
        return

    # all remaining searches must have a value
    if not search_value:
        vs_message(parent, "A search value must be input", "ERROR")
        return

    # Check values and do the search
    if search_option_selected == "Transaction, where transaction number":
        try:
            transaction_id = int(search_value)
        except Exception, e:
            vs_message(parent, "The transaction number must be numeric", "ERROR")
            return
        try:
            # search_result is the transaction
            search_result = Transaction(transaction_id)
        except VS_ERROR, e:
            vs_message(parent, "No transaction with that number has been found", "No result")
            return
        # if this is a vat correction, use the dialog in gui_functions
        if search_result.system == 11:
            show_transaction_dialog(parent, search_result)
        else:
            # Show the transaction with a button to open associated objects
            view_transactions.show_transaction_with_view_button(parent, search_result)
        return

    elif search_option_selected == "Invoices sent, where customer order number":
        # search_result is a list of invoices
        search_result = get_invoices_from_customer_ordernumber(search_value)
        if not search_result:
            vs_message(parent, "No invoices with that customer order number have been found", "No result")
            return
        if len(search_result) == 1:
            # only one has been found, show it
            show_invoice_frame(search_result[0])
            return
        # pick one from the list
        invoice = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if invoice:
            busy_cursor_on()
            wx.Yield()
            show_invoice_frame(invoice)
        return

    elif search_option_selected == "Invoice sent, where invoice number":
        try:
            invoice_number = int(search_value)
        except Exception, e:
            vs_message(parent, "The invoice number must be numeric", "ERROR")
            return
        # search_result is an invoice
        search_result = Invoice.from_number(invoice_number)
        if not search_result:
            vs_message(parent, "No invoice with that number has been found", "No result")
            return
        # Since this is an invoice - show the invoice
        show_invoice_frame(search_result)
        return

    elif search_option_selected == "Payments received, where customer reference":
        # search_result is a list of received payments
        search_result = get_payments_from_customer_reference(search_value)
        if not search_result:
            vs_message(parent, "No payments with that customer reference have been found", "No result")
            return
        if len(search_result) == 1:
            # only one has been found, show it
            show_customer_payment_dialog(parent, search_result[0])
            return
        # pick one from the list
        payment = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if payment:
            busy_cursor_on()
            wx.Yield()
            show_customer_payment_dialog(parent, payment)
        return

    elif search_option_selected == "Credit note sent, where credit note number":
        try:
            creditnote_number = int(search_value)
        except Exception, e:
            vs_message(parent, "The credit note number must be numeric", "ERROR")
            return
        # search_result is a credit note
        search_result = CreditNote.from_number(creditnote_number)
        if not search_result:
            vs_message(parent, "No credit note with that number has been found", "No result")
            return
        # Show the creditnote
        show_creditnote_frame(search_result)
        return

    elif search_option_selected == "Project, where project reference":
        project = Project.from_number(search_value)
        if not project:
            vs_message(parent, "No project with that reference has been found", "No result")
            return
        view_project_frame(project)
        return

    elif search_option_selected == "Purchase order sent, where order number":
        try:
            order_number = int(search_value)
        except Exception, e:
            vs_message(parent, "The order number must be numeric", "ERROR")
            return
        # search_result is an order
        search_result = PurchaseOrder.from_number(order_number)
        if not search_result:
            vs_message(parent, "No purchase order with that number has been found", "No result")
            return
        # Since this is a purchase order - create a ShowOrderFrame
        show_order_frame(search_result)
        return

    elif search_option_selected == "Purchase order, where supplier invoice number":
        # search_result is a list of orders
        search_result = get_orders_from_supplier_invoice_number(search_value)
        if not search_result:
            vs_message(parent, "No supplier invoice with that number has been found", "No result")
            return
        if len(search_result) == 1:
            # Show the purchase order
            show_order_frame(search_result[0])
            return
        # pick one from the list
        order = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if order:
            busy_cursor_on()
            wx.Yield()
            show_order_frame(order)
        return

    elif search_option_selected == "Purchase order, where supplier reference":
        # search_result is a list of orders
        search_result = get_orders_from_supplier_reference(search_value)
        if not search_result:
            vs_message(parent, "No purchase order with that reference has been found", "No result")
            return
        if len(search_result) == 1:
            # Show the purchase order
            show_order_frame(search_result[0])
            return
        # pick one from the list
        order = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if order:
            busy_cursor_on()
            wx.Yield()
            show_order_frame(order)
        return

    elif search_option_selected == "Supplier invoice received, where invoice number":
        # search_result is a list of supplier invoices
        search_result = get_supplierinvoices_from_supplierinvoice_number(search_value)
        if not search_result:
            vs_message(parent, "No supplier invoice with that number has been found", "No result")
            return
        if len(search_result) == 1:
            # Show the supplier invoice
            show_supplier_invoice_and_order_dialog(parent, search_result[0])
            return
        # pick one from the list
        sup_inv = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if sup_inv:
            busy_cursor_on()
            wx.Yield()
            show_supplier_invoice_and_order_dialog(parent, sup_inv)
        return

    elif search_option_selected == "Credit note received, where credit note number":
        # search_result is a list of supplier credit notes
        search_result = get_suppliercreditnotes_from_suppliercreditnote_number(search_value)
        if not search_result:
            vs_message(parent, "No supplier credit note with that number has been found", "No result")
            return
        if len(search_result) == 1:
            # Show the supplier creditnote
            show_supplier_creditnote_dialog(parent, search_result[0])
            return
        # pick one from the list
        sup_note = search_result_list_dialog(parent, search_option_selected, search_value, search_result)
        if sup_note:
            busy_cursor_on()
            wx.Yield()
            show_supplier_creditnote_dialog(parent, sup_note)
        return

    else:
        raise VS_ERROR, "The search option has not been recognised"


def search_result_list_dialog(parent, search_option_selected, search_value, search_result):
    """Creates a dialog showing the list of objects found, on an object being clicked, returns its value"""
    search_result_list_dialog = SearchResultListDialog(parent, search_option_selected, search_value, search_result)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if search_result_list_dialog.ShowModal() != wx.ID_OK:
        search_result_list_dialog.Destroy()
        return None

    object_chosen = search_result_list_dialog.object_chosen
    search_result_list_dialog.Destroy()
    return object_chosen


class SearchResultListDialog(vsDialog):
    "The dialog to list the objects found by the search"

    def __init__(self, parent, search_option_selected, search_value, search_result):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Search Result")

        # This is the object chosen, by the user clicking on a list item
        self.object_chosen = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if search_option_selected == "Invoices sent, where customer order number":
            help_text = "Invoices sent with customer order number %s\ndouble click on an invoice to view it." % search_value
            results_table = InvoicesTable(self, search_result)
        elif search_option_selected == "Payments received, where customer reference":
            help_text = "Payments received with customer reference %s\ndouble click on a payment to view it." % search_value
            results_table = CustomerPaymentsTable(self, search_result)
        elif search_option_selected == "Purchase order, where supplier invoice number":
            help_text = """\
Purchase orders, with supplier invoices received
which have invoice number %s
double click an order to view it.""" % search_value
            results_table = OrdersTable(self, search_result)
        elif search_option_selected == "Purchase order, where supplier reference":
            help_text = """\
Purchase orders, with supplier reference: %s
double click an order to view it.""" % search_value
            results_table = OrdersTable(self, search_result)
        elif search_option_selected == "Supplier invoice received, where invoice number":
            help_text = """\
Supplier invoices received which have invoice
number %s, double click an invoice to view it.""" % search_value
            results_table = SupplierInvoicesTable(self, search_result)
        elif search_option_selected == "Credit note received, where credit note number":
            help_text = """\
Supplier credit notes received which have credit note
number %s, double click on a note to view it.""" % search_value
            results_table = SupplierCreditNotesTable(self, search_result)
        else:
            raise VS_ERROR, "The search option has not been recognised"

        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        sizer.Add(results_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)

    def close_button_pressed(self, event):
        """Close dialog"""
        self.EndModal(wx.ID_CANCEL)


#### Create a different table for each search


############# list invoices

class InvoicesTable(wx.grid.Grid):

    def __init__(self, parent, invoice_list):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice_list = invoice_list

        # self.row records the last row double clicked on
        self.row = 0

        num_rows = len(self.invoice_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Customer")
        self.SetColLabelValue(2, "Description")

        for key, data in enumerate(self.invoice_list):
            self.SetRowLabelValue(key, str(data.number))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.customer.name)
            self.SetCellValue(key, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_invoice, self)


    @logmethod
    def pick_invoice(self, event):
        "Invoice has been double - clicked"
        try:
            self.parent.object_chosen = self.invoice_list[event.GetRow()]
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error choosing an invoice", e)



############# list received customer payments

class CustomerPaymentsTable(wx.grid.Grid):

    def __init__(self, parent, payments_list):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.payments_list = payments_list

        # self.row records the last row double clicked on
        self.row = 0

        num_rows = len(self.payments_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Customer")
        self.SetColLabelValue(2, "Description")

        for key, data in enumerate(self.payments_list):
            self.SetRowLabelValue(key, str(data.transaction.id))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.customer.name)
            self.SetCellValue(key, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_payment, self)


    @logmethod
    def pick_payment(self, event):
        "Payment has been double - clicked"
        try:
            self.parent.object_chosen = self.payments_list[event.GetRow()]
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error choosing a customer payment", e)


############# list orders

class OrdersTable(wx.grid.Grid):

    def __init__(self, parent, order_list):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.order_list = order_list

        num_rows = len(self.order_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Supplier")
        self.SetColLabelValue(2, "Description")

        for row, data in enumerate(self.order_list):
            self.SetRowLabelValue(row, str(data.number))
            self.SetCellValue(row, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, data.supplier.name)
            self.SetCellValue(row, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_order, self)

    @logmethod
    def pick_order(self, event):
        "Order has been double - clicked"
        try:
            self.parent.object_chosen = self.order_list[event.GetRow()]
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error choosing an order", e)


############# list supplier invoices

class SupplierInvoicesTable(wx.grid.Grid):

    def __init__(self, parent, invoice_list):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice_list = invoice_list

        # self.row records the last row double clicked on
        self.row = 0

        num_rows = len(self.invoice_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Supplier")
        self.SetColLabelValue(2, "Description")

        for key, data in enumerate(self.invoice_list):
            self.SetRowLabelValue(key, str(data.transaction.id))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.supplier.name)
            self.SetCellValue(key, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_invoice, self)


    @logmethod
    def pick_invoice(self, event):
        "Supplier invoice has been double - clicked"
        try:
            self.parent.object_chosen = self.invoice_list[event.GetRow()]
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error choosing a supplier invoice", e)


############# list supplier credit notes

class SupplierCreditNotesTable(wx.grid.Grid):

    def __init__(self, parent, creditnote_list):
        wx.grid.Grid.__init__(self, parent, size=(800,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.creditnote_list = creditnote_list

        # self.row records the last row double clicked on
        self.row = 0

        num_rows = len(self.creditnote_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Supplier")
        self.SetColLabelValue(2, "Description")

        for key, data in enumerate(self.creditnote_list):
            self.SetRowLabelValue(key, str(data.transaction.id))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.supplier.name)
            self.SetCellValue(key, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_creditnote, self)


    @logmethod
    def pick_creditnote(self, event):
        "Supplier credit note has been double - clicked"
        try:
            self.parent.object_chosen = self.creditnote_list[event.GetRow()]
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error choosing a supplier creditnote", e)


