####### VATStuff #######
#
# search_contacts.py  - search and view contacts
#
# Version : 0.6
# Date : 20100723
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,20010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a Frame and table to search and view contacts

It is run as part of the vatstuff gui interface.
"""

import sys
import datetime
import csv

import wx
import wx.grid

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *







class SearchContactsFrame(vsFrame):
    "Called to create a frame to do the search and view the results"

    def __init__(self, parent):
        "Create the frame - holding a SearchContactsPanel"
        vsFrame.__init__(self, parent, title="Contacts Search")

        # Create the SearchContactsPanel
        self.search_contacts_panel=SearchContactsPanel(self)

        # Get the panel size
        panel_size = self.search_contacts_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the global reference to this frame
        global vs_cfg
        vs_cfg["SEARCH_CONTACTS_FRAME"]=self

        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self, organization_text="", name_text="", use_current=True):
        """refreshes search with given values, or ignores these and uses
           current values if use_current is True"""
        self.search_contacts_panel.refresh(organization_text, name_text, use_current)


    def onclose(self, evt):
        "Close the frame, removing self from the global variable"
        global vs_cfg
        vs_cfg["SEARCH_CONTACTS_FRAME"]=None
        self.Destroy()


# ----------------------SearchContactsPanel

class SearchContactsPanel(wx.Panel):
    """The panel of the SearchContacts frame"""

    def __init__(self, parent):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent
        self.all_contacts = []

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=2, cols=3, hgap=40, vgap=20)

        # The company_name
        field_sizer.Add(wx.StaticText(self, -1, "Company name :"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.company_name_field = wx.TextCtrl(self, -1, "", size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.company_name_field.SetToolTipString("The company name, or leave blank.")
        field_sizer.Add(self.company_name_field, 0, wx.ALIGN_LEFT)

        # text
        text =  """\
Input all or part of the company name and
the persons name you wish to search on."""
        field_sizer.Add(wx.StaticText(self, -1, text), 0, wx.ALIGN_LEFT)


        # The contact_name
        field_sizer.Add(wx.StaticText(self, -1, "Contact name :"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.contact_name_field = wx.TextCtrl(self, -1, "", size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.contact_name_field.SetToolTipString("The contact name, or leave blank.")
        field_sizer.Add(self.contact_name_field, 0, wx.ALIGN_LEFT)

        # text
        text =  """\
Leaving a field blank will search
through all instances of the field."""
        field_sizer.Add(wx.StaticText(self, -1, text), 0, wx.ALIGN_LEFT)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # ---------------------- Search Close CSV  buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # search button
        self.search_button = wx.Button(self, label="Search", size=button_size)
        self.search_button.SetToolTipString("Click to search for contacts.")
        button_sizer.Add(self.search_button, 0, wx.RIGHT, 20)

        # csv button
        self.csv_button = wx.Button(self, label="CSV File", size=button_size)
        self.csv_button.SetToolTipString("Click to export the search results to a csv file.")
        button_sizer.Add(self.csv_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Create the results table

        self.results_table = ResultsTable(self)
        sizer.Add(self.results_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.search_button_pressed, self.search_button)
        self.Bind(wx.EVT_BUTTON, self.csv_button_pressed, self.csv_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    # ----------------- Handler functions

    @button_wrap
    def search_button_pressed(self, event):
        "get names from fields and call the refresh function on the table to do and show the search"
        self.organization_text = self.company_name_field.GetValue().strip()
        self.name_text = self.contact_name_field.GetValue().strip()
        self.all_contacts = self.results_table.refresh(self.organization_text, self.name_text)


    def refresh(self, organization_text="", name_text="", use_current=True):
        "refreshes search with given values, or uses current if use_current is True"
        if use_current:
            # retain original search
            self.organization_text = self.company_name_field.GetValue().strip()
            self.name_text = self.contact_name_field.GetValue().strip()
        else:
            # use new given values
            self.organization_text = organization_text.strip()
            self.name_text = name_text.strip()
            self.company_name_field.SetValue(self.organization_text)
            self.contact_name_field.SetValue(self.name_text)

        self.all_contacts = self.results_table.refresh(self.organization_text, self.name_text)


    @button_wrap
    def csv_button_pressed(self, event):
        "Create the save file dialog"
        if not self.all_contacts:
            vs_message(self, "There are no search results to export", "ERROR")
            return
        fp=None
        try:
            filename = "contacts%s.csv" % datetime.date.today().isoformat()
            status, path = save_file_as(self, 'csv', vs_cfg["EXPORT_FOLDER"], filename)
            if not status:
                return
            busy_cursor_on()
            wx.Yield()

            # open the csv file
            fp=open(path,"wb")
            # save file in csv format
            writer = csv.writer(fp, dialect="excel")
            writer.writerow(["Company",
                             "Name",
                             "Email",
                             "Phone",
                             "Mobile",
                             "Notes"])

            for data in self.all_contacts:
                row = list(data)
                # Note, the csv module requires utf-8 encoding
                for key, item in enumerate(row):
                    row[key] = item.encode('utf-8')
                writer.writerow(row)
        finally:
            if fp: fp.close()
        # save the directory to the database and to vs_cfg["EXPORT_FOLDER"]
        directory = os.path.dirname(path)
        if directory != vs_cfg["EXPORT_FOLDER"]:
            set_directory("EXPORT_FOLDER", directory)
        info_log("CSV file created: %s" % path)
        vs_message(self, "Your file has been saved", "Saved")


    def close_button_pressed(self, event):
        """Close parent"""
        self.parent.Close()
        return


class ResultsTable(wx.grid.Grid):

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, size=(900,200), style=wx.WANTS_CHARS)

        self.num_rows = 4
        self.num_cols = 6

        self.CreateGrid(self.num_rows, self.num_cols)
        self.EnableEditing(False)

        # create column labels
        self.SetColLabelValue(0, "Company")
        self.SetColLabelValue(1, "Name")
        self.SetColLabelValue(2, "Email")
        self.SetColLabelValue(3, "Phone")
        self.SetColLabelValue(4, "Mobile")
        self.SetColLabelValue(5, "Notes")

        for key in range(self.num_rows):
            for col in range(self.num_cols):
                self.SetCellValue(key, col, "")

        self.SetColSize(0, 150)
        self.SetColSize(1, 200)
        self.SetColSize(2, 150)
        self.SetColSize(3, 150)
        self.SetColSize(4, 150)
        self.SetColSize(5, 150)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)


    def refresh(self, organization_text="", name_text=""):
        "Call the search function, and fill in the table with the results"

        # search_for_contacts is in odd_functions
        all_contacts = search_for_contacts(organization_text, name_text)

        new_number_of_rows = len(all_contacts)

        if new_number_of_rows > self.num_rows:
            self.AppendRows(numRows = new_number_of_rows - self.num_rows)
            self.num_rows = new_number_of_rows

        for row, contact in enumerate(all_contacts):
            for col, item in enumerate(contact):
                self.SetCellValue(row, col, item)

        if new_number_of_rows < self.num_rows:
            for row in range(new_number_of_rows, self.num_rows):
                for col in range(self.num_cols):
                    self.SetCellValue(row, col, "")

        return all_contacts
        

