####### VATStuff #######
#
# supplier_reports.py  - dialog to choose a supplier and report
#
# Version : 0.6
# Date : 20100818
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Dialog to choose a supplier and report"""

import wx
import datetime





from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *


# The ChooseSupplierReportDialog is used to pick a supplier,
# a report, and to set the two dates between which the report
# will be taken.  It is normally used by calling the function
# choose_supplier_report

def choose_supplier_report(parent):
    """Creates a dialog, returns (True, supplier, start_date, end_date, report_name)
       on confirmation or (False, None, None, None, None) otherwise"""
    choose_supplier_report_dialog = ChooseSupplierReportDialog(parent)

    busy_cursor_off()

    if choose_supplier_report_dialog.ShowModal() != wx.ID_OK:
        choose_supplier_report_dialog.Destroy()
        return False, None, None, None, None


    result = (True,
                choose_supplier_report_dialog.supplier,
                choose_supplier_report_dialog.from_date,
                choose_supplier_report_dialog.to_date,
                choose_supplier_report_dialog.report_name)

    choose_supplier_report_dialog.Destroy()
    return result


class ChooseSupplierReportDialog(vsDialog):
    """Called to chose a supplier and report to generate"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Supplier reports")

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        help_text = wx.StaticText(self, -1, "Choose a supplier and the report to generate:")
        sizer.Add(help_text, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=7, cols=2, hgap=20, vgap=20)


        # Choose a supplier
        supplier_choice_text=wx.StaticText(self, -1, "Choose a supplier:")
        field_sizer.Add(supplier_choice_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_suppliers = Supplier.get_all()
        supplier_list = [ supplier.name for supplier in self.all_suppliers ]

        # the supplier selected on the purchase panel
        supplier_selected = top_frame().get_supplier()
        if supplier_selected and self.all_suppliers:
            supplier_index = self.all_suppliers.index(supplier_selected)
        else:
            supplier_index = 0            

        if supplier_list:
            self.supplier_choice_field = wx.Choice(self, -1, size=button_size, choices=supplier_list)
            self.supplier_choice_field.SetSelection(supplier_index)
            self.supplier = self.all_suppliers[supplier_index]
        else:
            self.supplier_choice_field = wx.Choice(self, -1, size=button_size, choices=[" "])
            self.supplier_choice_field.SetSelection(0)
            self.supplier = None
        self.supplier_choice_field.SetToolTipString("Choose a supplier") 

        field_sizer.Add(self.supplier_choice_field, 0, wx.ALIGN_LEFT)


        # Get initial dates
        self.to_date = datetime.date.today()

        # set self.from_date as the beginning of this month,
        # or set self.from_date as the beginning of last month if the to_date is less than ten days into the month
        delta = datetime.timedelta(days=10)
        self.from_date = self.to_date - delta
        self.from_date = self.from_date.replace(day=1)

        wx_fromdate = date_to_wxdate(self.from_date)
        wx_todate = date_to_wxdate(self.to_date)

        # The from date
        field_sizer.Add(wx.StaticText(self, -1, "Generate report for the period From:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.from_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_fromdate, 
                                                 size=button_size,
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.from_date_field.SetToolTipString("The from date")
        field_sizer.Add(self.from_date_field, 0, wx.ALIGN_LEFT)

        # The to date
        field_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.to_date_field = wx.DatePickerCtrl(self, -1,
                                               wx_todate, 
                                               size=button_size,
                                               style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.to_date_field.SetToolTipString("The to date")
        field_sizer.Add(self.to_date_field, 0, wx.ALIGN_LEFT)

        # Orders sent
        orders_text=wx.StaticText(self, -1, "Purchase orders sent to this supplier:")
        field_sizer.Add(orders_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.orders_button = wx.Button(self, label="Orders Report", size=button_size)
        self.orders_button.SetToolTipString("List orders sent to this supplier between the two dates")  
        field_sizer.Add(self.orders_button, 0, wx.ALIGN_LEFT)

        # Invoices received
        invoices_text=wx.StaticText(self, -1, "Invoices received from this supplier:")
        field_sizer.Add(invoices_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.invoices_button = wx.Button(self, label="Invoices Report", size=button_size)
        self.invoices_button.SetToolTipString("List invoices received from this supplier between the two dates")  
        field_sizer.Add(self.invoices_button, 0, wx.ALIGN_LEFT)
 
        # credit notes received
        credit_notes_text=wx.StaticText(self, -1, "Credit notes received from this supplier:")
        field_sizer.Add(credit_notes_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.credit_notes_button = wx.Button(self, label="Credit Notes Report", size=button_size)
        self.credit_notes_button.SetToolTipString("List credit notes received from this supplier between the two dates")  
        field_sizer.Add(self.credit_notes_button, 0, wx.ALIGN_LEFT)

        # payments sent
        payments_text=wx.StaticText(self, -1, "Payments sent to this supplier:")
        field_sizer.Add(payments_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.payments_button = wx.Button(self, label="Payments Report", size=button_size)
        self.payments_button.SetToolTipString("List payments sent to this supplier between the two dates")  
        field_sizer.Add(self.payments_button, 0, wx.ALIGN_LEFT)


        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        if not self.supplier:
            self.supplier_choice_field.Enable(False)
            self.from_date_field.Enable(False)
            self.to_date_field.Enable(False)
            self.orders_button.Enable(False)
            self.invoices_button.Enable(False)
            self.credit_notes_button.Enable(False)
            self.payments_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to cancel.")
        sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_BUTTON, self.orders_button_pressed, self.orders_button)
        self.Bind(wx.EVT_BUTTON, self.invoices_button_pressed, self.invoices_button)
        self.Bind(wx.EVT_BUTTON, self.credit_notes_button_pressed, self.credit_notes_button)
        self.Bind(wx.EVT_BUTTON, self.payments_button_pressed, self.payments_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    # ----------------- Handler functions

    def orders_button_pressed(self, event):
        """Orders report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "supplier_orders"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def invoices_button_pressed(self, event):
        """Invoices report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "supplier_invoices"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def credit_notes_button_pressed(self, event):
        """Credit_notes report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "supplier_creditnotes"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def payments_button_pressed(self, event):
        """Payments report button pressed"""
        try:
            if not self.get_selections():
                return
            self.report_name = "supplier_payments"
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error reading the form", e)

    def close_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)

    def get_selections(self):
        "Get the supplier and dates"
        supplier_selection = self.supplier_choice_field.GetSelection()
        self.supplier = self.all_suppliers[supplier_selection]

        # Convert the dates input to python date objects
        # function wxdate_to_date is in module vatstuff_packages.vs_utilities.wx_utilities
        try:
            from_date = wxdate_to_date(self.from_date_field.GetValue())
        except Exception:
            vs_message(self, "Invalid From date", "ERROR")
            return False
        try:
            to_date = wxdate_to_date(self.to_date_field.GetValue())
        except Exception:
            vs_message(self, "Invalid To date", "ERROR")
            return False
        if to_date < from_date:
            vs_message(self, "The To date is earlier than the From date!", "ERROR")
            return False
        self.from_date = from_date
        self.to_date = to_date
        return True



