####### VATStuff #######
#
# sales_tab.py  - defines the sales panel
#
# Version : 0.6
# Date : 20100719
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the sales panel

It is run as part of the vatstuff gui interface.
"""

import os

import wx

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.sales import make_invoice
from vatstuff_packages.vs_gui.sales import receive_payment
from vatstuff_packages.vs_gui.sales import make_creditnote
from vatstuff_packages.vs_gui.sales import view_invoices
from vatstuff_packages.vs_gui.sales import view_creditnotes

from vatstuff_packages.vs_gui import view_contacts
from vatstuff_packages.vs_gui import view_bank
from vatstuff_packages.vs_gui import organization
from vatstuff_packages.vs_gui import add_item

from vatstuff_packages.vs_gui.gui_functions import *


class SalesPanel(wx.Panel):
    "The Sales panel"

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create top 'Create invoice -> 'Cash received' buttons in a horizontal sizer

        top_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Create Invoice
        self.inv_button = wx.Button(self, label="Create invoice", size=button_size)
        self.inv_button.SetToolTipString("Press to create an invoice")   
        top_sizer.Add(self.inv_button, 0, wx.ALIGN_RIGHT)

        # Arrow to right image
        arrow_right_png=os.path.join(vs_cfg["IMAGES_FOLDER"],'arrow_right.png')
        arrow_right_image= wx.Image(arrow_right_png, wx.BITMAP_TYPE_PNG)
        arrow_right=wx.StaticBitmap(self, -1, wx.BitmapFromImage(arrow_right_image))
        top_sizer.Add(arrow_right, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Cash received
        self.cash_button = wx.Button(self, label="Payment Received", size=button_size)
        self.cash_button.SetToolTipString("Input payment recieved for invoice")   
        top_sizer.Add(self.cash_button, 0, wx.ALIGN_LEFT)

        sizer.Add(top_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 40)

        help_text="""\
Choose 'Create invoice' to produce an invoice to be sent to your customer.
When payments are recieved choose 'Payment Received' to input the amounts which will
be placed in the appropriate accounts."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create buttons and descriptions in a grid sizer
        button_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=20)

        # View invoices
        self.view_invoices_button = wx.Button(self, label="View invoices", size=button_size)
        self.view_invoices_button.SetToolTipString("View invoices")   
        button_sizer.Add(self.view_invoices_button, 0, wx.ALIGN_RIGHT)
        view_invoices_text=wx.StaticText(self, -1, "View or cancel previous invoices")
        button_sizer.Add(view_invoices_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Create a credit note
        self.credit_note_button = wx.Button(self, label="Create Credit Note", size=button_size)
        self.credit_note_button.SetToolTipString("Create a credit note")   
        button_sizer.Add(self.credit_note_button, 0, wx.ALIGN_RIGHT)
        credit_note_text=wx.StaticText(self, -1, "Create a credit note to issue to a customer")
        button_sizer.Add(credit_note_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # View credit notes
        self.view_credit_notes_button = wx.Button(self, label="View Credit Notes", size=button_size)
        self.view_credit_notes_button.SetToolTipString("List and view Credit Notes")   
        button_sizer.Add(self.view_credit_notes_button, 0, wx.ALIGN_RIGHT)
        view_credit_notes_text=wx.StaticText(self, -1, "View previous credit notes")
        button_sizer.Add(view_credit_notes_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Add sale item
        self.item_button = wx.Button(self, label="Add item to sell", size=button_size)
        self.item_button.SetToolTipString("Add item to sell")   
        button_sizer.Add(self.item_button, 0, wx.ALIGN_RIGHT)
        item_text=wx.StaticText(self, -1, "Create a list of items you sell")
        button_sizer.Add(item_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Add a new customer
        self.new_customer_button = wx.Button(self, label="Add a new customer", size=button_size)
        self.new_customer_button.SetToolTipString("Press to add a new customer")   
        button_sizer.Add(self.new_customer_button, 0, wx.ALIGN_RIGHT)
        new_customer_text=wx.StaticText(self, -1, "Press to add a new customer")
        button_sizer.Add(new_customer_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # Edit a customer
        self.all_customers = Customer.get_all()
        customer_list = [ customer.name for customer in self.all_customers ]

        if customer_list:
            self.edit_customer_choice = wx.Choice(self, -1, size=button_size, choices=customer_list)
            # set selection to last item on list - looks best under gnome
            self.edit_customer_choice.SetSelection(len(customer_list)-1)
        else:
            self.edit_customer_choice = wx.Choice(self, -1, size=button_size, choices=[" "])
            self.edit_customer_choice.SetSelection(0)
        self.edit_customer_choice.SetToolTipString("Choose to edit a customer details") 

        button_sizer.Add(self.edit_customer_choice, 0, wx.ALIGN_RIGHT)

        # Edit, contacts and banking buttons for this customer
        eandc_sizer = wx.BoxSizer(wx.HORIZONTAL)

        self.edit_customer_button = wx.Button(self, label="Edit", size=wx.DefaultSize)
        self.edit_customer_button.SetToolTipString("Press to edit the chosen customer")
        eandc_sizer.Add(self.edit_customer_button, 0, wx.ALIGN_LEFT)

        self.contacts_button = wx.Button(self, label="Contacts", size=wx.DefaultSize)
        self.contacts_button.SetToolTipString("Edit and list contacts for the chosen customer")   
        eandc_sizer.Add(self.contacts_button, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        self.bank_button = wx.Button(self, label="Bank", size=wx.DefaultSize)
        self.bank_button.SetToolTipString("Edit and show bank details for the chosen customer")   
        eandc_sizer.Add(self.bank_button, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        button_sizer.Add(eandc_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # sort out which buttons are enabled or not
        self.refresh()

        sizer.Add(button_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        help2_text="""\
Choose a customer and press Edit to change or add further invoice or delivery
addresses. Press Contacts or Bank to optionally store further customer details."""
        
        self.bottom_text=wx.StaticText(self, -1, help2_text)
        sizer.Add(self.bottom_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(sizer)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.inv_button_pressed, self.inv_button)
        self.Bind(wx.EVT_BUTTON, self.cash_button_pressed, self.cash_button)
        self.Bind(wx.EVT_BUTTON, self.view_invoices_button_pressed, self.view_invoices_button)
        self.Bind(wx.EVT_BUTTON, self.credit_note_button_pressed, self.credit_note_button)
        self.Bind(wx.EVT_BUTTON, self.view_credit_notes_button_pressed, self.view_credit_notes_button)
        self.Bind(wx.EVT_BUTTON, self.item_button_pressed, self.item_button)
        self.Bind(wx.EVT_BUTTON, self.new_customer_button_pressed, self.new_customer_button)
        self.Bind(wx.EVT_BUTTON, self.edit_customer_button_pressed, self.edit_customer_button)
        self.Bind(wx.EVT_BUTTON, self.contacts_button_pressed, self.contacts_button)
        self.Bind(wx.EVT_BUTTON, self.bank_button_pressed, self.bank_button)


    def get_customer(self):
        "returns the customer currently shown"
        if not self.all_customers: return None
        customer_selected_list_item = self.edit_customer_choice.GetCurrentSelection()
        assert customer_selected_list_item != -1
        return self.all_customers[customer_selected_list_item]


    # ----------------- Handler functions

    @button_wrap
    def inv_button_pressed(self, event):
        "Invoice button pressed, create and show the invoice dialog"
        customer = self.get_customer()
        if not customer: return
        make_invoice.invoice_dialog(self, customer)


    @button_wrap
    def cash_button_pressed(self, event):
        "Receive payment button pressed, show list of customers and then the payment dialog"
        invoiced_customers_dialog(self)


    @button_wrap
    def view_invoices_button_pressed(self, event):
        "Create the frame listing all invoices"
        view_invoices.view_all_invoices_frame()


    @button_wrap
    def credit_note_button_pressed(self, event):
        "Create a credit note button pressed, show the SendCreditNoteDialog"
        customer = self.get_customer()
        if not customer: return
        send_creditnote_dialog(self, customer)


    @button_wrap
    def view_credit_notes_button_pressed(self, event):
        "Create the frame listing all creditnotes"
        view_creditnotes.view_all_creditnotes_frame()


    @button_wrap
    def item_button_pressed(self, event):
        "Create and show the AddItemDialog to edit items frequently sold"
        add_item.add_item_dialog(self, "sale")


    @button_wrap
    def new_customer_button_pressed(self, event):
        "Create and show the dialog to add a new customer"
        organization.new_organization_dialog(self, "customer")


    @button_wrap
    def edit_customer_button_pressed(self, event):
        "Create and show the EditOrganizationDialog to edit an existing customer"
        customer = self.get_customer()
        if not customer: return
        organization.edit_organization_dialog(self, customer)


    @button_wrap
    def contacts_button_pressed(self, event):
        "View and edit the contacts for this customer"
        customer = self.get_customer()
        if not customer: return
        view_contacts.view_contacts_dialog(self, customer)


    @button_wrap
    def bank_button_pressed(self, event):
        "Create and show the BankDetailsDialog to edit bank info for an existing customer"
        customer = self.get_customer()
        if not customer: return
        view_bank.bank_details_dialog(self, customer)


    def refresh_list(self, select_customer=None):
        "Refresh list and set selection to select_customer"
        all_customers = Customer.get_all()
        if not all_customers:
            # empty list, no customer has been created yet
            self.edit_customer_choice.Enable(False)
            self.edit_customer_button.Enable(False)
            self.contacts_button.Enable(False)
            self.bank_button.Enable(False)
            # refresh customer selection
            self.edit_customer_choice.Clear()
            self.edit_customer_choice.AppendItems(" ")
            self.edit_customer_choice.SetSelection(0)
            self.all_customers = []
            return

        if select_customer is None:
            # Get the current customer shown
            select_customer = self.get_customer()

        # A new customer may have been added
        self.edit_customer_choice.Clear()
        self.all_customers = all_customers

        # Set up new list
        customer_list = [customer.name for customer in self.all_customers]
        self.edit_customer_choice.AppendItems(customer_list)

        for index, customer in enumerate(self.all_customers):
            if select_customer == customer:
                self.edit_customer_choice.SetSelection(index)
                break
        else:
            # Choose last
            self.edit_customer_choice.SetSelection(len(customer_list)-1)

        # enable buttons
        self.edit_customer_choice.Enable(True)
        self.edit_customer_button.Enable(True)
        self.contacts_button.Enable(True)
        self.bank_button.Enable(True)


    def refresh(self):
        self.inv_button.Enable(False)
        self.cash_button.Enable(False)
        self.view_invoices_button.Enable(False)
        self.credit_note_button.Enable(False)
        self.view_credit_notes_button.Enable(False)
        # refresh customer list and customer buttons
        self.refresh_list()
        if self.all_customers:
            # Check registered address is complete
            company = Company()
            # Get the current VAT Scheme
            vat_scheme = VATScheme()
            if ( company and
                 company.registered_address and
                 company.name and (company.vat_regnumber or vat_scheme.method == "NOT_REGISTERED")):
                self.view_invoices_button.Enable(True)
                self.inv_button.Enable(True)
                self.view_credit_notes_button.Enable(True)
                self.credit_note_button.Enable(True)
                if count_openinvoices() > 0:
                    # count_openinvoices() is defined in the odd_functions module
                    self.cash_button.Enable(True)


#------------ List and choose a customer with an outstanding open invoice
#------------ and then the payment dialog for that customer

def invoiced_customers_dialog(parent):
    """Creates an InvoicedCustomersDialog, and then a payment dialog
       depending on the customer chosen"""

    invoiced_customers_dialog = InvoicedCustomersDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if invoiced_customers_dialog.ShowModal() != wx.ID_OK:
        invoiced_customers_dialog.Destroy()
        return

    # A customer has been chosen
    customer = invoiced_customers_dialog.customer
    invoiced_customers_dialog.Destroy()

    busy_cursor_on()
    wx.Yield()
    debug_log("customer picked id: %s" % customer.id)
    # we now have to call the customer payment received dialog
    receive_payment.customer_payment_received(parent, customer)


# ----------------- Dialog to list customers with open invoices

class InvoicedCustomersDialog(vsDialog):
    "Create the frame showing a list of customers with open invoices"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Choose customer")
        
        # The self.customer attribute is set by a double click
        self.customer = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""\
This table shows customers with invoices currently open, double
click on the customer from whom you have received a payment."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the InvoicedCustomers table

        self.invoicedcustomers_table = InvoicedCustomersTable(self)
        sizer.Add(self.invoicedcustomers_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        sizer.Add(self.close_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    def close_button_pressed(self, event):
        "Close button pressed"
        self.EndModal(wx.ID_CANCEL)

    def close_with_customer(self, customer):
        """After a customer is chosen with a double click, this sets
           self.customer to the customer, then ends with an OK"""
        self.customer = customer
        self.EndModal(wx.ID_OK)


class InvoicedCustomersTable(wx.grid.Grid):

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, size=(700,200), style=wx.WANTS_CHARS)

        self.parent = parent

        self.customers_with_openinvoices = Customer.with_open_invoices()

        num_rows = len(self.customers_with_openinvoices)
        num_cols = 2

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Customer")
        self.SetColLabelValue(1, "Debit balance")

        # Set the columns showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.SetColAttr(1, attr)

        for row, customer in enumerate(self.customers_with_openinvoices):
            self.SetCellValue(row, 0, customer.name)
            self.SetCellValue(row, 1, customer.account.currency)

        self.SetColSize(0, 300)
        self.SetColSize(1, 200)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.choose_customer, self)


    # No decorator as this is a quick choose customer and close operation
    def choose_customer(self, event):
        "Customer has been double - clicked"
        try:
            # get the customer clicked
            self.row = event.GetRow()
            customer = self.customers_with_openinvoices[self.row]
            # Call the parent to set self.customer with the customer, and to close
            self.parent.close_with_customer(customer)
        except Exception, e:
            error_message("Failure in choosing customer", e)



########  Choose which method of creditnote is used - either a payment is sent with the
########  creditnote, or something is knocked off an existing invoice


def send_creditnote_dialog(parent, customer):
    """Creates a SendCreditNoteDialog to choose which type of note to send.

       If a creditnote with a payment is required it calls
       make_creditnote.creditnote_with_payment_dialog.
       However if a creditnote against an invoice is required, it calls
       open_invoices_dialog, to choose an invoice, and then
       make_creditnote.new_credit_note_dialog to create the creditnote."""

    # open the dialog
    send_creditnote_dialog = SendCreditNoteDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if send_creditnote_dialog.ShowModal() != wx.ID_OK:
        send_creditnote_dialog.Destroy()
        return

    # A choice has been made, get which type of note
    creditnote_choice = send_creditnote_dialog.creditnote_choice
    send_creditnote_dialog.Destroy()

    busy_cursor_on()
    wx.Yield()

    if not creditnote_choice:
        # note to be sent with payment, no invoice to be chosen
        make_creditnote.creditnote_with_payment_dialog(parent, customer)
        return

    # Pick an open invoice
    open_invoices_dialog = OpenInvoicesDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if open_invoices_dialog.ShowModal() != wx.ID_OK:
        open_invoices_dialog.Destroy()
        return

    # A choice has been made, get which invoice
    invoice = open_invoices_dialog.invoice
    open_invoices_dialog.Destroy()

    busy_cursor_on()
    wx.Yield()

    debug_log("invoice picked id: %s" % invoice.id)

    # An invoice has now been obtained, create the creditnote
    make_creditnote.new_credit_note_dialog(parent, invoice)


class SendCreditNoteDialog(vsDialog):
    "Called when a creditnote is to be sent to a customer"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Create Credit Note")
        
        # False, if note to be sent with a payment
        self.creditnote_choice = False
        # or set to True, if openinvoices is needed to choose an invoice

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        panel_sizer=wx.BoxSizer(wx.VERTICAL)
        sizer=wx.GridBagSizer(hgap=20, vgap=20)

        text1 = """\
Has the customer agreed to accept this credit note, and will use its
value to reduce the payment of an existing invoice you have already
sent? The invoice must not yet have been paid."""
        sizer.Add(wx.StaticText(self, -1, text1), pos=(0,0), span=(1,2), flag=wx.ALIGN_CENTER)


        # List Open invoices
        self.open_invoices_button = wx.Button(self, label="List open invoices", size=button_size)
        self.open_invoices_button.SetToolTipString("""\
Choose the invoice, and payments can
be reduced by the credit note amount""")   
        sizer.Add(self.open_invoices_button, pos=(1,0), flag=wx.ALIGN_RIGHT)

        button1_text = "Choose the invoice"
        sizer.Add(wx.StaticText(self, -1, button1_text), pos=(1,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # are there any open invoices? 
        all_openinvoices = get_openinvoices()
        if not all_openinvoices:
            self.open_invoices_button.Enable(False)

        text2 = """\
OR; is this credit note accompanied by a money payment to your
customer, which must now be taken from your bank account?"""
        sizer.Add(wx.StaticText(self, -1, text2), pos=(2,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # cash payment
        self.cash_payment_button = wx.Button(self, label="Creditnote and Payment", size=button_size)
        self.cash_payment_button.SetToolTipString("Input payment to be sent to your customer")   
        sizer.Add(self.cash_payment_button, pos=(3,0), flag=wx.ALIGN_RIGHT)

        button2_text = "Input payment to be sent to your customer"
        sizer.Add(wx.StaticText(self, -1, button2_text), pos=(3,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        panel_sizer.Add(sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel")
        panel_sizer.Add(self.cancel_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 40)
        
        # Set the size to the panel
        self.SetSizer(panel_sizer)
        panel_sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.open_invoices_button_pressed, self.open_invoices_button)
        self.Bind(wx.EVT_BUTTON, self.cash_payment_button_pressed, self.cash_payment_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    @logmethod
    def cash_payment_button_pressed(self, event):
        "A credit note, together with payment is to be created"
        self.creditnote_choice = False
        self.EndModal(wx.ID_OK)


    @logmethod
    def open_invoices_button_pressed(self, event):
        "A credit note is to be made against an existing invoice, list the open invoices"
        "List the open orders - to input an associated credit note"
        self.creditnote_choice = True
        self.EndModal(wx.ID_OK)

    def cancel_button_pressed(self, event):
        """Close down"""
        self.EndModal(wx.ID_CANCEL)


#------------- List and choose an open invoice

class OpenInvoicesDialog(vsDialog):
    "Create the dialog showing a list of open invoices"

    def __init__(self, parent):
        """List open invoices, choose an invoice on double clicking"""
        vsDialog.__init__(self, parent, "Open Invoices")
        
        # This invoice is the attribute to be chosen
        self.invoice = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""This table shows invoices currently open, double click on
the invoice for which you wish to create a Credit Note."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the openinvoices table

        self.openinvoices_table = OpenInvoicesTable(self)
        sizer.Add(self.openinvoices_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        sizer.Add(self.close_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    def close_button_pressed(self, event):
        """Close down"""
        self.EndModal(wx.ID_CANCEL)


class OpenInvoicesTable(wx.grid.Grid):

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, size=(800,200),style=wx.WANTS_CHARS)

        self.parent = parent
        self.all_openinvoices = get_openinvoices()

        num_rows = len(self.all_openinvoices)
        num_cols = 4

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Total")
        self.SetColLabelValue(3, "Outstanding")

        # Set the colums showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.SetColAttr(2, attr)
        self.SetColAttr(3, attr)

        for key, data in enumerate(self.all_openinvoices):
            self.SetRowLabelValue(key, str(data.number))
            self.SetCellValue(key, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(key, 1, data.description)
            self.SetCellValue(key, 2, to_currency_string(data.total))
            self.SetCellValue(key, 3, to_currency_string(data.outstanding))

        self.SetColSize(0, 100)
        self.SetColSize(1, 350)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.pick_invoice, self)


    @logmethod
    def pick_invoice(self, event):
        "Invoice has been double - clicked"
        row = event.GetRow()
        self.parent.invoice = self.all_openinvoices[row]
        self.parent.EndModal(wx.ID_OK)

