####### VATStuff #######
#
# vat_tab.py  - defines the vat panel and buttons
#
# Version : 0.6
# Date : 20100813
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" This is a module in the vatstuff_packages.vs_gui package, and defines the vat panel

It is run as part of the vatstuff gui interface.
"""

import wx

from vatstuff_packages.vs_gui.vat import edit_rates
from vatstuff_packages.vs_gui.vat import vat_correction
from vatstuff_packages.vs_gui.vat import set_vat_scheme
from vatstuff_packages.vs_gui.vat import vt_100
from vatstuff_packages.vs_gui.vat import previous_returns

from vatstuff_packages.vs_gui.reports import make_report

from vatstuff_packages.vs_gui.gui_functions import *






from vatstuff_packages.vs_utilities import *


class VatPanel(wx.Panel):
    """The VAT panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        main_sizer=wx.BoxSizer(wx.VERTICAL)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=9, cols=2, hgap=20, vgap=20)

        # vt100 form
        self.vt100_button = wx.Button(self, label="VAT Return", size=button_size)
        self.vt100_button.SetToolTipString("Press to generate a VAT 100 form")   
        b_sizer.Add(self.vt100_button, 0, wx.ALIGN_RIGHT)
        vt100_text=wx.StaticText(self, -1, "Press to generate a VAT Return form, and to submit a VAT payment")
        b_sizer.Add(vt100_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat correction
        self.vat_correction_button = wx.Button(self, label="VAT Correction", size=button_size)
        self.vat_correction_button.SetToolTipString("Press to generate VAT correction payments")   
        b_sizer.Add(self.vat_correction_button, 0, wx.ALIGN_RIGHT)
        vat_correction_text=wx.StaticText(self, -1, "Corrective payment to, or received from, HMRC")
        b_sizer.Add(vat_correction_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat report
        self.vat_report_button = wx.Button(self, label="VAT report", size=button_size)
        self.vat_report_button.SetToolTipString("Press to generate a basic VAT report")   
        b_sizer.Add(self.vat_report_button, 0, wx.ALIGN_RIGHT)
        vat_report_text=wx.StaticText(self, -1, "Press to generate a basic VAT report")
        b_sizer.Add(vat_report_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat detail
        self.vat_detail_button = wx.Button(self, label="VAT detail", size=button_size)
        self.vat_detail_button.SetToolTipString("Press to generate a detailed VAT report")   
        b_sizer.Add(self.vat_detail_button, 0, wx.ALIGN_RIGHT)
        vat_detail_text=wx.StaticText(self, -1, "Press to generate a detailed VAT report")
        b_sizer.Add(vat_detail_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # list vat payments
        self.list_vat_payments_button = wx.Button(self, label="List VAT payments", size=button_size)
        self.list_vat_payments_button.SetToolTipString("Press to generate a list of VAT payments made to HMRC")   
        b_sizer.Add(self.list_vat_payments_button, 0, wx.ALIGN_RIGHT)
        list_vat_payments_text=wx.StaticText(self, -1, "Press to generate a list of VAT payments made to HMRC")
        b_sizer.Add(list_vat_payments_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # EC Sales List
        self.ec_sales_list_button = wx.Button(self, label="EC Sales List", size=button_size)
        self.ec_sales_list_button.SetToolTipString("Press to generate an EC Sales List")   
        b_sizer.Add(self.ec_sales_list_button, 0, wx.ALIGN_RIGHT)
        ec_sales_list_text=wx.StaticText(self, -1, "Press to generate an EC Sales List")
        b_sizer.Add(ec_sales_list_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat previous returns
        self.vat_previous_form_button = wx.Button(self, label="Previous VAT returns", size=button_size)
        self.vat_previous_form_button.SetToolTipString("Press to view previous VAT returns")   
        b_sizer.Add(self.vat_previous_form_button, 0, wx.ALIGN_RIGHT)
        vat_previous_form_text=wx.StaticText(self, -1, "Press to view previous VAT returns")
        b_sizer.Add(vat_previous_form_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat rates table
        self.vat_rates_button = wx.Button(self, label="Set VAT rates", size=button_size)
        self.vat_rates_button.SetToolTipString("Press to set VAT rates")   
        b_sizer.Add(self.vat_rates_button, 0, wx.ALIGN_RIGHT)
        vat_rates_text=wx.StaticText(self, -1, "Press to set VAT rates")
        b_sizer.Add(vat_rates_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        # vat scheme
        self.vat_scheme_button = wx.Button(self, label="Set VAT scheme", size=button_size)
        self.vat_scheme_button.SetToolTipString("Press to set VAT scheme")   
        b_sizer.Add(self.vat_scheme_button, 0, wx.ALIGN_RIGHT)
        vat_scheme_text=wx.StaticText(self, -1, "Press to set VAT scheme")
        b_sizer.Add(vat_scheme_text, 0, wx.LEFT | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 20)

        main_sizer.Add(b_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 30) 
        
        # Set the size to the panel
        self.SetSizer(main_sizer)

        # sort out which buttons are enabled or not
        self.refresh()

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.vt100_button_pressed, self.vt100_button)
        self.Bind(wx.EVT_BUTTON, self.vat_correction_button_pressed, self.vat_correction_button)
        self.Bind(wx.EVT_BUTTON, self.vat_report_button_pressed, self.vat_report_button)
        self.Bind(wx.EVT_BUTTON, self.vat_detail_button_pressed, self.vat_detail_button)
        self.Bind(wx.EVT_BUTTON, self.list_vat_payments_button_pressed, self.list_vat_payments_button)
        self.Bind(wx.EVT_BUTTON, self.ec_sales_list_button_pressed, self.ec_sales_list_button)
        self.Bind(wx.EVT_BUTTON, self.vat_previous_form_button_pressed, self.vat_previous_form_button)
        self.Bind(wx.EVT_BUTTON, self.vat_rates_button_pressed, self.vat_rates_button)
        self.Bind(wx.EVT_BUTTON, self.vat_scheme_button_pressed, self.vat_scheme_button)

    # ----------------- Handler functions

    @button_wrap
    def vt100_button_pressed(self, event):
        "Create the VAT Return Form"

        # Do not ask for two dates, if the vatreturnframe is already open
        for frm in vs_cfg["TRANSACTION_REFRESH"]:
            if isinstance(frm, vt_100.VATReturnFrame):
                frm.Raise()
                return

        # It's not open - so generate a new one, ask for two dates first

        help_text = """\
A VAT return will be generated for
the period between the two dates:"""

        # Get the dates of the VAT return form
        vat_scheme = VATScheme()
        current_vat_period = vat_scheme.period

        # Start of current VAT start_date - this is a python datetime object
        from_date = vat_scheme.start_date
        to_date = vat_scheme.quarter_date

        # Use these as defaults in the following two dates function, with the from date
        # disabled so it cannot be changed
        status, from_date, to_date = two_dates(self, "VAT Return", help_text,
                                                from_date, to_date,
                                                from_enable=False, to_enable=True)
        if not status:
            return

        busy_cursor_on()
        wx.Yield()
        vt_100.vat_return_frame(from_date, to_date)


    @button_wrap
    def vat_correction_button_pressed(self, event):
        "show a dialog to input a VAT correction payment, or view previous ones"
        vat_correction.vat_correction_dialog(self)


    @button_wrap
    def vat_report_button_pressed(self, event):
        """Call list_vat_returns to choose a period and then generates a basic report"""

        # get the vat period for the report
        current_vat_scheme = VATScheme()
        if current_vat_scheme.method == 'NOT_REGISTERED':
            # if the current vat scheme is NOT_REGISTERED don't
            # do a detail report for the current period
            show_current = False
        else:
            show_current = True

        status, vat_scheme, vat_return, previous_vat_return, from_date, to_date = list_vat_returns(self,
                                                                                                    show_current,
                                                                                                    show_all = False)
        if not status:
            return

        busy_cursor_on()
        wx.Yield()

        method = vat_scheme.method

        if method == "STANDARD":
            report_type = "vat_report_standard"
        elif method == "CASH":
            report_type = "vat_report_cash"
        elif method == "FLAT-BASIC":
            report_type = "vat_report_flat_basic"
        elif method == "FLAT-CASH":
            report_type = "vat_report_flat_cash"
        else:
            raise VS_ERROR, "VAT Scheme not recognised"

        filename = "VAT_report%s" % to_date.isoformat()
        make_report.make_report_and_frame(report_type,
                                          "VAT Report",
                                          filename,
                                          from_date,
                                          to_date,
                                          vat_return = vat_return,
                                          vat_scheme = vat_scheme)


    @button_wrap
    def vat_detail_button_pressed(self, event):
        """Call list_vat_returns to choose a period and then generate a detailed report"""

        # get the vat period for the report
        current_vat_scheme = VATScheme()
        if current_vat_scheme.method == 'NOT_REGISTERED':
            # if the current vat scheme is NOT_REGISTERED don't
            # do a detail report for the current period
            show_current = False
        else:
            show_current = True

        status, vat_scheme, vat_return, previous_vat_return, from_date, to_date = list_vat_returns(self,
                                                                                                    show_current,
                                                                                                    show_all = False)
        if not status:
            return

        busy_cursor_on()
        wx.Yield()

        method = vat_scheme.method

        if method == "STANDARD":
            report_type = "vat_detail_standard"
        elif method == "CASH":
            report_type = "vat_detail_cash"
        elif method == "FLAT-BASIC":
            report_type = "vat_detail_flat_basic"
        elif method == "FLAT-CASH":
            report_type = "vat_detail_flat_cash"
        else:
            raise VS_ERROR, "VAT Scheme not recognised"

        filename = "VAT_detail_report%s" % to_date.isoformat()
        make_report.make_report_and_frame(report_type,
                                          "VAT Detail Report",
                                          filename,
                                          from_date,
                                          to_date,
                                          vat_return = vat_return,
                                          vat_scheme = vat_scheme)


    @button_wrap
    def list_vat_payments_button_pressed(self, event):
        """Get the two dates between which the report is generated and show vat payments"""

        help_text = """\
A report will be generated showing VAT
payments to HMRC between the two dates:"""

        status, from_date, to_date = two_dates(self, "VAT Payments", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "vat_payments%s" % to_date.isoformat()
        make_report.make_report_and_frame("vatpayments",
                                          "VAT Payments",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def ec_sales_list_button_pressed(self, event):
        """Choose two dates (previouse vat return by default) and then generate the report"""

        help_text = """\
Display an EC Sales List showing sales amounts to
VAT Registered EC customers between the two dates:"""

        # get default from and to dates as the previous vat return
        previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme()
        from_date, to_date = get_vat_return_dates(previous_vatreturn)
        # ask the user for the from and two dates
        status, from_date, to_date = two_dates(self, "EC Sales List", help_text, from_date, to_date)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "ec_sales_list%s" % to_date.isoformat()
        make_report.make_report_and_frame("ecsaleslist",
                                          "EC Sales List",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def vat_previous_form_button_pressed(self, event):
        "Call ListVATReturnsFrame to show a previouse VAT return"

        # get the vat period for the report
        status, vat_scheme, vat_return, previous_vat_return, from_date, to_date = list_vat_returns(self,
                                                                                    show_current=False,
                                                                                    show_all=True)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        # generate and display the report
        previous_returns.previous_vat_return_frame(vat_return)



    @button_wrap
    def vat_rates_button_pressed(self, event):
        "Call EditRatesFrame to edit VAT percentage rates"
        edit_rates.edit_rates_dialog(self)


    @button_wrap
    def vat_scheme_button_pressed(self, event):
        "Show the VATSchemeDialog to view or change the VAT Scheme used"
        set_vat_scheme.vat_scheme_dialog(self)


    def refresh(self):
        "refresh the panel - enable or disable buttons"
        # Enable previous vat form button if there is a previous vat return
        last_vat_return = get_config_item("LAST_VAT_RETURN_ID")
        if (not last_vat_return) or last_vat_return == '0':
            self.vat_previous_form_button.Enable(False)
        else:
            self.vat_previous_form_button.Enable(True)
        # the vat return, vat reports and EC Sales list buttons can only be enabled if the company exists
        company = Company()
        if not (company and company.registered_address and company.name):
            self.vt100_button.Enable(False)
            self.vat_report_button.Enable(False)
            self.vat_detail_button.Enable(False)
            self.ec_sales_list_button.Enable(False)
            return

        vat_scheme = VATScheme()
        if vat_scheme.method != 'NOT_REGISTERED' and (not company.vat_regnumber):
            # If the company is registered, but there is no vat number - disable buttons
            self.vt100_button.Enable(False)
            self.vat_report_button.Enable(False)
            self.vat_detail_button.Enable(False)
            self.ec_sales_list_button.Enable(False)
            return

        if vat_scheme.method != 'NOT_REGISTERED':
            self.vat_report_button.Enable(True)
            self.vat_detail_button.Enable(True)
            self.vt100_button.Enable(True)
            self.ec_sales_list_button.Enable(True)
        else:
            # If the vat scheme method is NOT_REGISTERED, disable the VAT return button
            self.vt100_button.Enable(False)
            # and only enable vat_detail and EC sales list buttons if there are previous VAT returns
            if last_vat_return == '0':
                self.vat_report_button.Enable(False)
                self.vat_detail_button.Enable(False)
                self.ec_sales_list_button.Enable(False)
            else:
                self.vat_report_button.Enable(True)
                self.vat_detail_button.Enable(True)
                self.ec_sales_list_button.Enable(True)


# The ListVATReturnsDialog is used to view a list of previous VAT
# returns and choose one. 
# It is normally used by calling the function list_vat_returns

def list_vat_returns(parent, show_current, show_all=True):
    """Creates a ListVATReturnsDialog dialog.
       show_current should be True if the last item on the list is the current period,
       or False if only previous submitted returns should be shown.
       show_all should be true if all returns should be shown, or False if those
       returns which cover a period of deleted date should not be shown.
       The function returns:
       (status, vat_scheme, vat_return, previous_vat_return, from_date, to_date)
       where status is True if the dialog apply was pressed, or False if the
       dialog was cancelled - in which case the remaining parameters are all None.
       if status is True, the vat_scheme, vat_return, previous_vat_return, from_date and to_date returned
       apply to the period chosen, however if the current period is chosen, for which a vat
       return has not been submitted, then the vat_return parameter will be None.
       If this is the very first current period, and there have been no vat returns
       submitted, then both vat_return and previous_vat_return will be None"""

    # Get the current VAT Scheme
    vat_scheme = VATScheme()

    # Get the current period
    from_date = vat_scheme.start_date
    to_date = vat_scheme.quarter_date

    last_vat_return_id = int(get_config_item("LAST_VAT_RETURN_ID"))
    if last_vat_return_id == 0:
        # There are no previous vat returns
        if show_current:
            # Return the current period, but no vat_return, nor previous_vat_return
            return (True, vat_scheme, None, None, from_date, to_date)
        else:
            # the current vat period is not wanted, so return False
            return (False, None, None, None, None, None)

    last_vatreturn = VATReturn(last_vat_return_id)
    if last_vatreturn.to_date >= to_date:
        # The current date falls inside the last vat return,
        show_current = False

    # Call the dialog box
    list_vat_returns_dialog = ListVATReturnsDialog(parent, show_current, show_all, from_date, to_date)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if list_vat_returns_dialog.ShowModal() != wx.ID_OK:
        # the dialog was cancelled
        list_vat_returns_dialog.Destroy()
        return (False, None, None, None, None, None)

    # Must return True
    vat_return = list_vat_returns_dialog.vat_return
    list_vat_returns_dialog.Destroy()

    if vat_return:
        return (True, vat_return.vatscheme,
                      vat_return,
                      vat_return.previous_vatreturn,
                      vat_return.from_date,
                      vat_return.to_date)

    # no vat return was selected, so either the current period is required
    # or the last vat return should be returned

    if show_current:
        # return the current period
        return (True, vat_scheme, None, last_vatreturn, from_date, to_date)

    # current period is not wanted, so return the last vatreturn
    return (True, last_vatreturn.vatscheme,
                  last_vatreturn,
                  last_vatreturn.previous_vatreturn,
                  last_vatreturn.from_date,
                  last_vatreturn.to_date)


class ListVATReturnsDialog(vsDialog):
    """Called to list the vat returns"""

    def __init__(self, parent, show_current, show_all, from_date, to_date):
        """Create the Dialog, listing VAT returns.
           show_current is true if the last item on the list of vat periods should be
           the current period given by the two dates from_date and to_date.
           If show_current is false, only vat periods for which vat returns have been submitted
           will be shown, and the two dates are not used
           show_all is true to show all vatreturns held in the database, if it is False
           then only those returns covering periods in which there has been no old data deleted
           will be shown - this is so that a 'vat detail' report does not attempt to show detail
           in which the detail is missing."""
        vsDialog.__init__(self, parent, "Choose a VAT period")

        # set self.all_vatreturns to be the list of all vatreturns
        # If show_all is False and a vatreturn covers a period for which data has
        # has been deleted then do not include it in the list

        if (not show_all) and vs_cfg["DELETE_TO_DATE"]:
            self.all_vatreturns = [ vatreturn for vatreturn in VATReturn.get_all() if vatreturn.from_date >= vs_cfg["DELETE_TO_DATE"] ]
        else:
            self.all_vatreturns = VATReturn.get_all()

        # This is the attribute read by the calling function
        self.vat_return = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""Choose a VAT period and press Apply"""
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the vatreturns list

        period_list = [ "From : %s - To : %s" %  (data.from_date.strftime("%d %b %Y"),
                                                  data.to_date.strftime("%d %b %Y"))
                        for data in self.all_vatreturns ]

        if show_current:
            period_list.append( "From : %s - To : %s" % (from_date.strftime("%d %b %Y"),
                                                         to_date.strftime("%d %b %Y") ))

        self.period_list_field = wx.ListBox(self, -1, size=(-1, 300), choices = period_list)
        self.period_list_field.SetToolTipString("Select a period for the report")
        sizer.Add(self.period_list_field, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to choose the period.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button"
        # Get the selection
        selection = self.period_list_field.GetSelection()
        self.vat_return = None
        if selection > -1:
            if selection < len(self.all_vatreturns):
                self.vat_return = self.all_vatreturns[selection]
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)



