####### VATStuff #######
#
# view_bank.py  - Creates a dialog to view and edit bank details
#
# Version : 0.6
# Date : 20100803
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a dialog to view and edit bank details

It is run as part of the vatstuff gui interface.
"""

import wx

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *







def bank_details_dialog(parent, organization):
    """Creates a BankDetailsDialog"""
    bank_details_dialog = BankDetailsDialog(parent, organization)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    bank_details_dialog.ShowModal()
    bank_details_dialog.Destroy()


class BankDetailsDialog(vsDialog):
    "Called when the bank button is pressed"

    def __init__(self, parent, organization):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Bank details")
        
        # Take a note of the variables
        self.organization = organization

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if self.organization.name:
            help_text="Bank details for %s." % self.organization.name
        else:
            help_text="Company bank details"
  
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create descriptions and fields in a grid sizer
        f_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=20)

        # sort code
        sc_text=wx.StaticText(self, -1, "Bank sort code:")
        f_sizer.Add(sc_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.sort_code_field = wx.TextCtrl(self, -1, self.organization.sortcode, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.sort_code_field.SetToolTipString("Set the sort code")
        self.sort_code_field.SetMaxLength(8)
        f_sizer.Add(self.sort_code_field, 0, wx.ALIGN_LEFT)

        # bank account
        ba_text=wx.StaticText(self, -1, "Bank account:")
        f_sizer.Add(ba_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.bank_account_field = wx.TextCtrl(self, -1, self.organization.bank_account, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.bank_account_field.SetToolTipString("Set the bank account")
        self.bank_account_field.SetMaxLength(45)
        f_sizer.Add(self.bank_account_field, 0, wx.ALIGN_LEFT)

        # iban
        iban_text=wx.StaticText(self, -1, "IBAN:")
        f_sizer.Add(iban_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.iban_field = wx.TextCtrl(self, -1, self.organization.iban, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.iban_field.SetToolTipString("IBAN")
        self.iban_field.SetMaxLength(45)
        f_sizer.Add(self.iban_field, 0, wx.ALIGN_LEFT)

        # bic
        bic_text=wx.StaticText(self, -1, "BIC:")
        f_sizer.Add(bic_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.bic_field = wx.TextCtrl(self, -1, self.organization.bic, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.bic_field.SetToolTipString("BIC")
        self.bic_field.SetMaxLength(20)
        f_sizer.Add(self.bic_field, 0, wx.ALIGN_LEFT)


        sizer.Add(f_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """ Apply button - check and apply all fields"""
        sortcode = self.sort_code_field.GetValue()
        bank_account = self.bank_account_field.GetValue()
        iban = self.iban_field.GetValue()
        bic = self.bic_field.GetValue()
    
        self.organization.sortcode = sortcode.strip()
        self.organization.bank_account = bank_account.strip()
        self.organization.iban = iban.strip()
        self.organization.bic = bic.strip()

        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close down"""
        self.EndModal(wx.ID_CANCEL)






