# test_odd_functions.py  - tests on the odd_functions.py module
#
# Version : 0.7.0
# Date : 20100927
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Exercises functions in odd_functions.py"""

import unittest
import os
import os.path
import sqlite3

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions


# Create the unittest class

class TestOddFunctions(unittest.TestCase):
    """A test class for odd_functions.py"""

    def test_01_get_config_item(self):
        "odd_functions.get_config_item"
        result = odd_functions.get_config_item("PROGRAM_ID")
        self.assertEqual(result, "VATStuff")
        # Check return value if key does not exist
        result = odd_functions.get_config_item("Not a key")
        self.assertFalse(result)


    def test_02_set_config_item(self):
        "odd_functions.set_config_item"
        result = odd_functions.set_config_item("PROGRAM_ID", "New ID")
        self.assertTrue(result)
        result = odd_functions.get_config_item("PROGRAM_ID")
        self.assertEqual(result, "New ID")
        result = odd_functions.set_config_item("PROGRAM_ID", "VATStuff")
        self.assertTrue(result)
        result = odd_functions.get_config_item("PROGRAM_ID")
        self.assertEqual(result, "VATStuff")
        # Check return value if key does not exist
        result = odd_functions.set_config_item("Not a key", "Not a value")
        self.assertFalse(result)


    def test_03_get_program_id(self):
        "odd_functions.get_program_id"
        result = odd_functions.get_program_id()
        self.assertEqual(result, "VATStuff")


    def test_04_get_database_version(self):
        "odd_functions.get_database_version"
        result = odd_functions.get_database_version()
        self.assertEqual(result, config.vs_cfg["DATABASE_VERSION"])


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestOddFunctions)

