####### VATStuff #######
#
# vatstuff_packages.vs_utilities  - package of config data, and utility functions
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""This package is intended to be imported by other modules in
VATStuff using the full import:

from vatstuff_packages.vs_utilities import *

thus giving other modules immediate access to the most commonly
required functions and classes.

This __init__.py imports these from modules:

config         -  Holds the vs_cfg dictionary of configuration values plus
                  several utility functions, dealing in logging, string
                  to currency conversions, and the function that opens the
                  database.

objects        -  Holds the object-relational mapper, being a number of classes
                  each representing a table on the database, an instance of
                  each class being a row.

odd_functions  -  A variety of miscellaneous functions

These modules only import from the standard library, they do not import
wx or other third party libraries - therefore vatstuff_packages.vs_utilities can be easily
imported and used from the command line interpreter and by test routines.
"""

from vatstuff_packages.vs_utilities.config import *
from vatstuff_packages.vs_utilities.objects import *
from vatstuff_packages.vs_utilities.odd_functions import *

# Note: to avoid circular imports
#
# vatstuff_packages.vs_utilities.config only imports from the standard library
# vatstuff_packages.vs_utilities.objects only imports from config and the standard library
# vatstuff_packages.vs_utilities.odd_functions only imports from objects, config and the standard library

# All other packages can use 'from vatstuff_packages.vs_utilities import *' and get the functions
# defined in these three modules without any circular import worries

# config.py excludes some functions by using an __all__ variable - these have
# to be imported specifically - see the module for details.

